@@ -1,37 +1,43 @@
-// $Id: CartridgeSlotManager.cc,v 1.31 2006/05/26 12:24:51 m9710797 Exp $
+// $Id: CartridgeSlotManager.cc 6157 2007-03-11 22:22:34Z m9710797 $
 
 #include "CartridgeSlotManager.hh"
 #include "MSXMotherBoard.hh"
 #include "ExtensionConfig.hh"
-#include "Command.hh"
+#include "RecordedCommand.hh"
 #include "CommandException.hh"
+#include "CommandController.hh"
+#include "FileContext.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
 #include "openmsx.hh"
+#include "MSXCliComm.hh"
 #include <cassert>
 
 using std::string;
 using std::vector;
+using std::set;
 
 namespace openmsx {
 
-class CartCmd : public Command
+class CartCmd : public RecordedCommand
 {
 public:
-	CartCmd(CartridgeSlotManager& manager, const std::string& commandName);
-	virtual void execute(const vector<TclObject*>& tokens, TclObject& result);
+	CartCmd(CartridgeSlotManager& manager, MSXMotherBoard& motherBoard,
+	        const std::string& commandName);
+	virtual string execute(const vector<string>& tokens, const EmuTime& time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	const ExtensionConfig* getExtensionConfig(const string& cartname);
 	CartridgeSlotManager& manager;
+	MSXCliComm& cliComm;
 };
 
 
 CartridgeSlotManager::CartridgeSlotManager(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
-	, cartCmd(new CartCmd(*this, "cart"))
+	, cartCmd(new CartCmd(*this, motherBoard, "cart"))
 {
 }
 
@@ -49,6 +55,13 @@ int CartridgeSlotManager::getSlotNum(const string& slot)
 		return -(1 + slot[0] - 'a');
 	} else if (slot == "any") {
 		return -256;
+	} else if ((slot.size() == 2) && (slot[0] == '?')) {
+		int result = slot[1] - '0';
+		if ((result < 0) || (4 <= result)) {
+			throw MSXException(
+				"Invalid slot specification: " + slot);
+		}
+		return result-128;
 	} else {
 		int result = StringOp::stringToInt(slot);
 		if ((result < 0) || (4 <= result)) {
@@ -73,9 +86,11 @@ void CartridgeSlotManager::createExternalSlot(int ps, int ss)
 		if (!slots[slot].exists()) {
 			slots[slot].ps = ps;
 			slots[slot].ss = ss;
-			string slotname = "carta";
-			slotname[4] += slot;
-			slots[slot].command = new CartCmd(*this, slotname);
+			string slotName = "carta";
+			slotName[4] += slot;
+			motherBoard.getMSXCliComm().update(
+				CliComm::HARDWARE, slotName, "add");
+			slots[slot].command = new CartCmd(*this, motherBoard, slotName);
 			return;
 		}
 	}
@@ -117,6 +132,9 @@ void CartridgeSlotManager::removeExternalSlot(int ps, int ss)
 {
 	int slot = getSlot(ps, ss);
 	assert(!slots[slot].used());
+	const string& slotName = slots[slot].command->getName();
+	motherBoard.getMSXCliComm().update(
+		CliComm::HARDWARE, slotName, "remove");
 	delete slots[slot].command;
 	slots[slot].command = NULL;
 }
@@ -201,9 +219,14 @@ bool CartridgeSlotManager::isExternalSlot(int ps, int ss, bool convert) const
 
 
 // CartCmd
-CartCmd::CartCmd(CartridgeSlotManager& manager_, const string& commandName)
-	: Command(manager_.motherBoard.getCommandController(), commandName)
+CartCmd::CartCmd(CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard,
+                 const string& commandName)
+	: RecordedCommand(motherBoard.getMSXCommandController(),
+	                  motherBoard.getMSXEventDistributor(),
+	                  motherBoard.getScheduler(),
+	                  commandName)
 	, manager(manager_)
+	, cliComm(motherBoard.getMSXCliComm())
 {
 }
 
@@ -218,28 +241,43 @@ const ExtensionConfig* CartCmd::getExtensionConfig(const string& cartname)
 	return static_cast<const ExtensionConfig*>(conf);
 }
 
-void CartCmd::execute(const vector<TclObject*>& tokens, TclObject& result)
+string CartCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 {
-	string cartname = tokens[0]->getString();
+	string result;
+	string cartname = tokens[0];
+	
+	// strip namespace qualification
+	//  TODO investigate whether it's a good idea to strip namespace at a
+	//       higher level for all commands. How does that interact with
+	//       the event recording feature?
+	string::size_type pos = cartname.rfind("::");
+	if (pos != string::npos) {
+		cartname = cartname.substr(pos + 2);
+	}
 	if (tokens.size() == 1) {
 		// query name of cartridge
 		const ExtensionConfig* extConf = getExtensionConfig(cartname);
-		result.addListElement(cartname + ':');
-		result.addListElement(extConf ? extConf->getName() : "");
-		TclObject options(result.getInterpreter());
+		TclObject object(getCommandController().getInterpreter());
+		object.addListElement(cartname + ':');
+		object.addListElement(extConf ? extConf->getName() : "");
+		TclObject options(getCommandController().getInterpreter());
 		if (!extConf) {
 			options.addListElement("empty");
 		}
 		if (options.getListLength() != 0) {
-			result.addListElement(options);
+			object.addListElement(options);
 		}
-		
-	} else if (tokens[1]->getString() == "-eject") {
+		result = object.getString();
+	} else if ( (tokens[1] == "eject") || (tokens[1] == "-eject") ) {
 		// remove cartridge (or extension)
+		if (tokens[1] == "-eject") {
+			result += "Warning: use of '-eject' is deprecated, instead use the 'eject' subcommand";		
+		}
 		const ExtensionConfig* extConf = getExtensionConfig(cartname);
 		if (extConf) {
 			try {
 				manager.motherBoard.removeExtension(*extConf);
+				cliComm.update(CliComm::MEDIA, cartname, "");
 			} catch (MSXException& e) {
 				throw CommandException("Can't remove cartridge: " +
 				                       e.getMessage());
@@ -248,32 +286,51 @@ void CartCmd::execute(const vector<TclObject*>& tokens, TclObject& result)
 
 	} else {
 		// insert cartridge
-		try {
-			string slotname = (cartname.size() == 5)
-			                ? string(1, cartname[4])
-			                : "any";
-			vector<string> options;
-			for (unsigned i = 2; i < tokens.size(); ++i) {
-				options.push_back(tokens[i]->getString());
+		string slotname = (cartname.size() == 5)
+			? string(1, cartname[4])
+			: "any";
+		int extensionNameToken = 1;
+		if (tokens[1] == "insert") {
+			if (tokens.size() > 2) {
+				extensionNameToken = 2;
+			} else {
+				throw CommandException("Missing argument to insert subcommand");
 			}
+		}
+		vector<string> options;
+		for (unsigned i = (extensionNameToken + 1); i < tokens.size(); ++i) {
+			options.push_back(tokens[i]);
+		}
+		try {
 			ExtensionConfig& extension =
 				manager.motherBoard.loadRom(
-					tokens[1]->getString(), slotname, options);
-			result.setString(extension.getName());
+					tokens[extensionNameToken], slotname, options);
+			cliComm.update(CliComm::MEDIA, cartname, tokens[extensionNameToken]);
+			result = extension.getName();
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
 		}
 	}
+	return result;
 }
 
-string CartCmd::help(const vector<string>& /*tokens*/) const
+string CartCmd::help(const vector<string>& tokens) const
 {
-	return "Insert a ROM cartridge.";
+	return tokens[0] + " eject              : remove the ROM cartridge from this slot\n" +
+	       tokens[0] + " insert <filename>  : insert ROM cartridge with <filename>\n" +
+	       tokens[0] + " <filename>         : insert ROM cartridge with <filename>\n" +
+	       tokens[0] + "                    : show which ROM cartridge is in this slot";
 }
 
 void CartCmd::tabCompletion(vector<string>& tokens) const
 {
-	completeFileName(tokens);
+	set<string> extra;
+	if (tokens.size() < 3) {
+		extra.insert("eject");
+		extra.insert("insert");
+	}
+	UserFileContext context(getCommandController());
+	completeFileName(tokens, context, extra);
 }
 
 } // namespace openmsx
