@@ -1,6 +1,7 @@
-// $Id: CliExtension.cc,v 1.35 2006/03/15 20:47:19 m9710797 Exp $
+// $Id: CliExtension.cc 5746 2006-10-05 16:57:09Z m9710797 $
 
 #include "CliExtension.hh"
+#include "CommandLineParser.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXException.hh"
 #include <cassert>
@@ -12,7 +13,7 @@ namespace openmsx {
 CliExtension::CliExtension(CommandLineParser& cmdLineParser_)
 	: cmdLineParser(cmdLineParser_)
 {
-	cmdLineParser.registerOption("-ext", this, 7);
+	cmdLineParser.registerOption("-ext", *this, 7);
 }
 
 bool CliExtension::parseOption(const string& option, std::list<string>& cmdLine)
