@@ -1,19 +1,21 @@
-// $Id: CommandLineParser.cc,v 1.158 2006/05/31 19:24:12 m9710797 Exp $
+// $Id: CommandLineParser.cc 5993 2007-01-12 22:12:12Z m9710797 $
 
 #include "CommandLineParser.hh"
+#include "CLIOption.hh"
 #include "CommandController.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileOperations.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "Version.hh"
 #include "MSXRomCLI.hh"
 #include "CliExtension.hh"
-#include "CassettePlayer.hh"
+#include "CassettePlayerCLI.hh"
 #include "DiskImageCLI.hh"
 #include "HDImageCLI.hh"
 #include "CDImageCLI.hh"
+#include "MSXEventRecorderReplayerCLI.hh"
 #include "ConfigException.hh"
 #include "FileException.hh"
 #include "EnumSetting.hh"
@@ -33,22 +35,97 @@ using std::vector;
 
 namespace openmsx {
 
-// class CLIOption
-
-string CLIOption::getArgument(const string& option, list<string>& cmdLine) const
+class HelpOption : public CLIOption
 {
-	if (cmdLine.empty()) {
-		throw FatalError("Missing argument for option \"" + option + "\"");
-	}
-	string argument = cmdLine.front();
-	cmdLine.pop_front();
-	return argument;
-}
+public:
+	explicit HelpOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
 
-string CLIOption::peekArgument(const list<string>& cmdLine) const
+class VersionOption : public CLIOption
 {
-	return cmdLine.empty() ? "" : cmdLine.front();
-}
+public:
+	explicit VersionOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
+
+class ControlOption : public CLIOption
+{
+public:
+	explicit ControlOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
+
+class ScriptOption : public CLIOption
+{
+public:
+	const CommandLineParser::Scripts& getScripts() const;
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser::Scripts scripts;
+};
+
+class MachineOption : public CLIOption
+{
+public:
+	explicit MachineOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
+
+class SettingOption : public CLIOption
+{
+public:
+	explicit SettingOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
+
+class NoMMXOption : public CLIOption
+{
+public:
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+};
+
+class NoMMXEXTOption : public CLIOption {
+public:
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+};
+
+class TestConfigOption : public CLIOption
+{
+public:
+	explicit TestConfigOption(CommandLineParser& parser);
+	virtual bool parseOption(const std::string& option,
+		std::list<std::string>& cmdLine);
+	virtual const std::string& optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
 
 
 // class CommandLineParser
@@ -57,36 +134,40 @@ CommandLineParser::CommandLineParser(Reactor& reactor_)
 	: parseStatus(UNPARSED)
 	, reactor(reactor_)
 	, settingsConfig(reactor.getCommandController().getSettingsConfig())
-	, output(reactor.getCliComm())
-	, helpOption(*this)
-	, versionOption(*this)
-	, controlOption(*this)
-	, machineOption(*this)
-	, settingOption(*this)
-	, testConfigOption(*this)
+	, output(reactor.getGlobalCliComm())
+	, helpOption(new HelpOption(*this))
+	, versionOption(new VersionOption(*this))
+	, controlOption(new ControlOption(*this))
+	, scriptOption(new ScriptOption())
+	, machineOption(new MachineOption(*this))
+	, settingOption(new SettingOption(*this))
+	, noMMXOption(new NoMMXOption())
+	, noMMXEXTOption(new NoMMXEXTOption())
+	, testConfigOption(new TestConfigOption(*this))
 	, msxRomCLI(new MSXRomCLI(*this))
 	, cliExtension(new CliExtension(*this))
-	, cassettePlayerCLI(new MSXCassettePlayerCLI(*this))
+	, cassettePlayerCLI(new CassettePlayerCLI(*this))
 	, diskImageCLI(new DiskImageCLI(*this))
 	, hdImageCLI(new HDImageCLI(*this))
 	, cdImageCLI(new CDImageCLI(*this))
+	, eventRecorderReplayerCLI(new MSXEventRecorderReplayerCLI(*this))
 {
 	haveConfig = false;
 	haveSettings = false;
 
-	registerOption("-machine",    &machineOption, 4);
-	registerOption("-setting",    &settingOption, 2);
-	registerOption("-h",          &helpOption, 1, 1);
-	registerOption("--help",      &helpOption, 1, 1);
-	registerOption("-v",          &versionOption, 1, 1);
-	registerOption("--version",   &versionOption, 1, 1);
-	registerOption("-control",    &controlOption, 1, 1);
-	registerOption("-script",     &scriptOption, 1, 1);
+	registerOption("-machine",    *machineOption, 4);
+	registerOption("-setting",    *settingOption, 2);
+	registerOption("-h",          *helpOption, 1, 1);
+	registerOption("--help",      *helpOption, 1, 1);
+	registerOption("-v",          *versionOption, 1, 1);
+	registerOption("--version",   *versionOption, 1, 1);
+	registerOption("-control",    *controlOption, 1, 1);
+	registerOption("-script",     *scriptOption, 1, 1);
 	#ifdef ASM_X86
-	registerOption("-nommx",      &noMMXOption, 1, 1);
-	registerOption("-nommxext",   &noMMXEXTOption, 1, 1);
+	registerOption("-nommx",      *noMMXOption, 1, 1);
+	registerOption("-nommxext",   *noMMXEXTOption, 1, 1);
 	#endif
-	registerOption("-testconfig", &testConfigOption, 1, 1);
+	registerOption("-testconfig", *testConfigOption, 1, 1);
 	
 	registerFileTypes();
 }
@@ -96,19 +177,19 @@ CommandLineParser::~CommandLineParser()
 }
 
 void CommandLineParser::registerOption(
-	const string& str, CLIOption* cliOption, byte prio, byte length)
+	const string& str, CLIOption& cliOption, unsigned prio, unsigned length)
 {
 	OptionData temp;
-	temp.option = cliOption;
+	temp.option = &cliOption;
 	temp.prio = prio;
 	temp.length = length;
 	optionMap[str] = temp;
 }
 
 void CommandLineParser::registerFileClass(
-	const string& str, CLIFileType* cliFileType)
+	const string& str, CLIFileType& cliFileType)
 {
-	fileClassMap[str] = cliFileType;
+	fileClassMap[str] = &cliFileType;
 }
 
 void CommandLineParser::registerFileTypes()
@@ -144,7 +225,7 @@ void CommandLineParser::registerFileTypes()
 }
 
 bool CommandLineParser::parseOption(
-	const string& arg, list<string>& cmdLine, byte priority)
+	const string& arg, list<string>& cmdLine, unsigned priority)
 {
 	map<string, OptionData>::const_iterator it1 = optionMap.find(arg);
 	if (it1 != optionMap.end()) {
@@ -225,7 +306,23 @@ void CommandLineParser::parse(int argc, char** argv)
 				const string& machine =
 					reactor.getMachineSetting().getValueString();
 				output.printInfo("Using default machine: " + machine);
-				loadMachine(machine);
+				try {
+					reactor.createMotherBoard(machine);
+				} catch (MSXException& e) {
+					output.printInfo(
+						"Failed to initialize default machine: " + e.getMessage()
+						);
+					// Default machine is broken; fall back to C-BIOS config.
+					const string& fallbackMachine =
+						reactor.getMachineSetting().getRestoreValueString();
+					output.printInfo("Using fallback machine: " + fallbackMachine);
+					try {
+						reactor.createMotherBoard(fallbackMachine);
+					} catch (MSXException& e2) {
+						// Fallback machine failed as well; we're out of options.
+						throw FatalError(e2.getMessage());
+					}
+				}
 				haveConfig = true;
 			}
 			break;
@@ -245,7 +342,7 @@ void CommandLineParser::parse(int argc, char** argv)
 						map<string, OptionData>::const_iterator it1 =
 							optionMap.find(arg);
 						if (it1 != optionMap.end()) {
-							for (int i = 0; i < it1->second.length - 1; ++i) {
+							for (unsigned i = 0; i < it1->second.length - 1; ++i) {
 								if (!cmdLine.empty()) {
 									backupCmdLine.push_back(cmdLine.front());
 									cmdLine.pop_front();
@@ -282,7 +379,7 @@ CommandLineParser::ParseStatus CommandLineParser::getParseStatus() const
 
 const CommandLineParser::Scripts& CommandLineParser::getStartupScripts() const
 {
-	return scriptOption.getScripts();
+	return scriptOption->getScripts();
 }
 
 Reactor& CommandLineParser::getReactor() const
@@ -295,45 +392,22 @@ MSXMotherBoard* CommandLineParser::getMotherBoard() const
 	return getReactor().getMotherBoard();
 }
 
-void CommandLineParser::loadMachine(const string& machine)
-{
-	try {
-		getReactor().createMotherBoard(machine);
-	} catch (MSXException& e) {
-		throw FatalError(e.getMessage());
-	}
-}
-
 // Control option
 
-CommandLineParser::ControlOption::ControlOption(CommandLineParser& parser_)
+ControlOption::ControlOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::ControlOption::parseOption(const string& option,
-		list<string>& cmdLine)
+bool ControlOption::parseOption(const string& option, list<string>& cmdLine)
 {
-	string type_name, arguments;
-	StringOp::splitOnFirst(getArgument(option, cmdLine),
-	                       ":", type_name, arguments);
-
-	map<string, CommandLineParser::ControlType> controlTypeMap;
-	controlTypeMap["stdio"] = IO_STD;
-#ifdef _WIN32
-	controlTypeMap["pipe"] = IO_PIPE;
-#endif
-	if (controlTypeMap.find(type_name) == controlTypeMap.end()) {
-		throw FatalError("Unknown control type: '"  + type_name + "'");
-	}
-	CommandLineParser::ControlType type = controlTypeMap[type_name];
-
-	parser.getReactor().getCliComm().startInput(type, arguments);
-	parser.parseStatus = CONTROL;
+	parser.getReactor().getGlobalCliComm().startInput(
+		getArgument(option, cmdLine));
+	parser.parseStatus = CommandLineParser::CONTROL;
 	return true;
 }
 
-const string& CommandLineParser::ControlOption::optionHelp() const
+const string& ControlOption::optionHelp() const
 {
 	static const string text("Enable external control of openMSX process");
 	return text;
@@ -342,20 +416,19 @@ const string& CommandLineParser::ControlOption::optionHelp() const
 
 // Script option
 
-const CommandLineParser::Scripts&
-CommandLineParser::ScriptOption::getScripts() const
+const CommandLineParser::Scripts& ScriptOption::getScripts() const
 {
 	return scripts;
 }
 
-bool CommandLineParser::ScriptOption::parseOption(const string& option,
+bool ScriptOption::parseOption(const string& option,
 		list<string>& cmdLine)
 {
 	scripts.push_back(getArgument(option, cmdLine));
 	return true;
 }
 
-const string& CommandLineParser::ScriptOption::optionHelp() const
+const string& ScriptOption::optionHelp() const
 {
 	static const string text("Run extra startup script");
 	return text;
@@ -427,12 +500,12 @@ static void printItemMap(const map<string, set<string> >& itemMap)
 	}
 }
 
-CommandLineParser::HelpOption::HelpOption(CommandLineParser& parser_)
+HelpOption::HelpOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
+bool HelpOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
 	if (!parser.haveSettings) {
@@ -447,8 +520,8 @@ bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
 	cout << "  this is the list of supported options:" << endl;
 
 	map<string, set<string> > optionMap;
-	for (map<string, OptionData>::const_iterator it = parser.optionMap.begin();
-	     it != parser.optionMap.end(); ++it) {
+	for (map<string, CommandLineParser::OptionData>::const_iterator it =
+	        parser.optionMap.begin(); it != parser.optionMap.end(); ++it) {
 		optionMap[it->second.option->optionHelp()].insert(it->first);
 	}
 	printItemMap(optionMap);
@@ -457,17 +530,18 @@ bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
 	cout << "  this is the list of supported file types:" << endl;
 
 	map<string, set<string> > extMap;
-	for (FileTypeMap::const_iterator it = parser.fileTypeMap.begin();
+	for (CommandLineParser::FileTypeMap::const_iterator it =
+	         parser.fileTypeMap.begin();
 	     it != parser.fileTypeMap.end(); ++it) {
 		extMap[it->second->fileTypeHelp()].insert(it->first);
 	}
 	printItemMap(extMap);
 
-	parser.parseStatus = EXIT;
+	parser.parseStatus = CommandLineParser::EXIT;
 	return true;
 }
 
-const string& CommandLineParser::HelpOption::optionHelp() const
+const string& HelpOption::optionHelp() const
 {
 	static const string text("Shows this text");
 	return text;
@@ -476,20 +550,20 @@ const string& CommandLineParser::HelpOption::optionHelp() const
 
 // class VersionOption
 
-CommandLineParser::VersionOption::VersionOption(CommandLineParser& parser_)
+VersionOption::VersionOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::VersionOption::parseOption(const string& /*option*/,
+bool VersionOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
 	cout << Version::FULL_VERSION << endl;
-	parser.parseStatus = EXIT;
+	parser.parseStatus = CommandLineParser::EXIT;
 	return true;
 }
 
-const string& CommandLineParser::VersionOption::optionHelp() const
+const string& VersionOption::optionHelp() const
 {
 	static const string text("Prints openMSX version and exits");
 	return text;
@@ -498,24 +572,27 @@ const string& CommandLineParser::VersionOption::optionHelp() const
 
 // Machine option
 
-CommandLineParser::MachineOption::MachineOption(CommandLineParser& parser_)
+MachineOption::MachineOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::MachineOption::parseOption(const string& option,
-		list<string>& cmdLine)
+bool MachineOption::parseOption(const string& option, list<string>& cmdLine)
 {
 	if (parser.haveConfig) {
 		throw FatalError("Only one machine option allowed");
 	}
 	string machine(getArgument(option, cmdLine));
 	parser.output.printInfo("Using specified machine: " + machine);
-	parser.loadMachine(machine);
+	try {
+		parser.getReactor().createMotherBoard(machine);
+	} catch (MSXException& e) {
+		throw FatalError(e.getMessage());
+	}
 	parser.haveConfig = true;
 	return true;
 }
-const string& CommandLineParser::MachineOption::optionHelp() const
+const string& MachineOption::optionHelp() const
 {
 	static const string text("Use machine specified in argument");
 	return text;
@@ -524,13 +601,12 @@ const string& CommandLineParser::MachineOption::optionHelp() const
 
 // Setting Option
 
-CommandLineParser::SettingOption::SettingOption(CommandLineParser& parser_)
+SettingOption::SettingOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::SettingOption::parseOption(const string &option,
-		list<string> &cmdLine)
+bool SettingOption::parseOption(const string &option, list<string> &cmdLine)
 {
 	if (parser.haveSettings) {
 		throw FatalError("Only one setting option allowed");
@@ -550,7 +626,7 @@ bool CommandLineParser::SettingOption::parseOption(const string &option,
 	return true;
 }
 
-const string& CommandLineParser::SettingOption::optionHelp() const
+const string& SettingOption::optionHelp() const
 {
 	static const string text("Load an alternative settings file");
 	return text;
@@ -559,7 +635,7 @@ const string& CommandLineParser::SettingOption::optionHelp() const
 
 // class NoMMXOption
 
-bool CommandLineParser::NoMMXOption::parseOption(const string& /*option*/,
+bool NoMMXOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
 	cout << "Disabling MMX " << endl;
@@ -567,7 +643,7 @@ bool CommandLineParser::NoMMXOption::parseOption(const string& /*option*/,
 	return true;
 }
 
-const string& CommandLineParser::NoMMXOption::optionHelp() const
+const string& NoMMXOption::optionHelp() const
 {
 	static const string text(
 		"Disables usage of MMX, including extensions (for debugging)");
@@ -577,7 +653,7 @@ const string& CommandLineParser::NoMMXOption::optionHelp() const
 
 // class NoMMXEXTOption
 
-bool CommandLineParser::NoMMXEXTOption::parseOption(const string& /*option*/,
+bool NoMMXEXTOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
 	cout << "Disabling MMX extensions" << endl;
@@ -585,7 +661,7 @@ bool CommandLineParser::NoMMXEXTOption::parseOption(const string& /*option*/,
 	return true;
 }
 
-const string& CommandLineParser::NoMMXEXTOption::optionHelp() const
+const string& NoMMXEXTOption::optionHelp() const
 {
 	static const string text(
 		"Disables usage of MMX extensions (for debugging)");
@@ -593,21 +669,21 @@ const string& CommandLineParser::NoMMXEXTOption::optionHelp() const
 }
 
 
-// TestConfig option
+// class TestConfigOption
 
-CommandLineParser::TestConfigOption::TestConfigOption(CommandLineParser& parser_)
+TestConfigOption::TestConfigOption(CommandLineParser& parser_)
 	: parser(parser_)
 {
 }
 
-bool CommandLineParser::TestConfigOption::parseOption(const string& /*option*/,
+bool TestConfigOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
-	parser.parseStatus = TEST;
+	parser.parseStatus = CommandLineParser::TEST;
 	return true;
 }
 
-const string& CommandLineParser::TestConfigOption::optionHelp() const
+const string& TestConfigOption::optionHelp() const
 {
 	static const string text("Test if the specified config works and exit");
 	return text;
