@@ -1,4 +1,4 @@
-// $Id: HostCPU.cc,v 1.9 2006/03/26 17:05:00 mthuurne Exp $
+// $Id: HostCPU.cc 5789 2006-10-18 17:13:07Z andete $
 
 #include "HostCPU.hh"
 #include "openmsx.hh"
@@ -9,7 +9,7 @@ HostCPU::HostCPU()
 {
 	mmxFlag = false;
 	mmxExtFlag = false;
-	#ifdef ASM_X86
+	#ifdef ASM_X86_32
 		// Note: On Mac OS X, EBX is in use by the OS, so we have to restore it.
 		// Is CPUID instruction supported?
 		unsigned hasCPUID;
@@ -62,6 +62,11 @@ HostCPU::HostCPU()
 			}
 		}
 	#endif
+	#ifdef ASM_X86_64
+		// X86_64 machines always have mmx and mmxExt
+		mmxFlag = true;
+		mmxExtFlag = true;
+	#endif
 
 	PRT_DEBUG("MMX:              " << mmxFlag);
 	PRT_DEBUG("MMX extensions:   " << mmxExtFlag);
