@@ -1,7 +1,7 @@
-// $Id: I8255.cc,v 1.16 2005/11/09 21:35:49 m9710797 Exp $
+// $Id: I8255.cc 5929 2006-12-15 18:21:09Z manuelbi $
 
 #include "I8255.hh"
-#include <cassert>
+#include "MSXCliComm.hh"
 
 namespace openmsx {
 
@@ -22,8 +22,11 @@ const int BIT_NR       = 0x0E;
 const int SET_RESET    = 0x01;
 
 
-I8255::I8255(I8255Interface& interf, const EmuTime& time)
+I8255::I8255(I8255Interface& interf, const EmuTime& time, 
+		MSXCliComm& cliComm_)
 	: interface(interf)
+	, cliComm(cliComm_)
+	, warningPrinted(false)
 {
 	reset(time);
 }
@@ -52,10 +55,9 @@ byte I8255::readPortA(const EmuTime& time)
 			//output
 			return latchPortA;		// output is latched
 		}
-	case MODEA_1:		//TODO but not relevant for MSX
+	case MODEA_1: //TODO but not relevant for MSX
 	case MODEA_2: case MODEA_2_:
 	default:
-		assert (false);
 		return 255;	// avoid warning
 	}
 }
@@ -69,7 +71,7 @@ byte I8255::peekPortA(const EmuTime& time) const
 		} else {
 			return latchPortA;		// output is latched
 		}
-	case MODEA_1:		//TODO but not relevant for MSX
+	case MODEA_1: //TODO but not relevant for MSX
 	case MODEA_2: case MODEA_2_:
 	default:
 		return 255;
@@ -87,9 +89,8 @@ byte I8255::readPortB(const EmuTime& time)
 			//output
 			return latchPortB;		// output is latched
 		}
-	case MODEB_1:		// TODO but not relevant for MSX
+	case MODEB_1: // TODO but not relevant for MSX
 	default:
-		assert (false);
 		return 255;	// avoid warning
 	}
 }
@@ -103,9 +104,9 @@ byte I8255::peekPortB(const EmuTime& time) const
 		} else {
 			return latchPortB;		// output is latched
 		}
-	case MODEB_1:		// TODO but not relevant for MSX
+	case MODEB_1: // TODO but not relevant for MSX
 	default:
-		return 255;
+		return 255; 
 	}
 }
 
@@ -116,16 +117,18 @@ byte I8255::readPortC(const EmuTime& time)
 	case MODEA_0:
 		// do nothing
 		break;
-	case MODEA_1:		// TODO but not relevant for MSX
+	case MODEA_1:
 	case MODEA_2: case MODEA_2_:
-		assert (false);
+		// TODO but not relevant for MSX
+		break;
 	}
 	switch (control & MODE_B) {
 	case MODEB_0:
 		// do nothing
 		break;
-	case MODEB_1:		// TODO but not relevant for MSX
-		assert (false);
+	case MODEB_1:
+		// TODO but not relevant for MSX
+		break;
 	}
 	return tmp;
 }
@@ -186,9 +189,10 @@ void I8255::writePortA(byte value, const EmuTime& time)
 	case MODEA_0:
 		// do nothing
 		break;
-	case MODEA_1:		// TODO but not relevant for MSX
+	case MODEA_1:
 	case MODEA_2: case MODEA_2_:
-		assert (false);
+		// TODO but not relevant for MSX
+		break;
 	}
 	outputPortA(value, time);
 }
@@ -199,8 +203,9 @@ void I8255::writePortB(byte value, const EmuTime& time)
 	case MODEB_0:
 		// do nothing
 		break;
-	case MODEB_1:		// TODO but not relevant for MSX
-		assert (false);
+	case MODEB_1:
+		// TODO but not relevant for MSX
+		break;
 	}
 	outputPortB(value, time);
 }
@@ -211,16 +216,18 @@ void I8255::writePortC(byte value, const EmuTime& time)
 	case MODEA_0:
 		// do nothing
 		break;
-	case MODEA_1:		// TODO but not relevant for MSX
+	case MODEA_1:
 	case MODEA_2: case MODEA_2_:
-		assert (false);
+		// TODO but not relevant for MSX
+		break;
 	}
 	switch (control & MODE_B) {
 	case MODEB_0:
 		// do nothing
 		break;
-	case MODEB_1:		// TODO but not relevant for MSX
-		assert (false);
+	case MODEB_1:
+		// TODO but not relevant for MSX
+		break;
 	}
 	outputPortC(value, time);
 }
@@ -261,6 +268,12 @@ void I8255::writeControlPort(byte value, const EmuTime& time)
 	if (value & SET_MODE) {
 		// set new control mode
 		control = value;
+		if ((control & (MODE_A | MODE_B)) && !warningPrinted) {
+			warningPrinted = true;
+			cliComm.printWarning("Invalid PPI mode selected. "
+			"This is not yet correctly emulated. "
+			"On a real MSX this will most likely hang.");
+		}
 		outputPortA(latchPortA, time);
 		outputPortB(latchPortB, time);
 		outputPortC(latchPortC, time);
@@ -281,16 +294,18 @@ void I8255::writeControlPort(byte value, const EmuTime& time)
 		case MODEA_0:
 			// do nothing
 			break;
-		case MODEA_1:		// TODO but not relevant for MSX
+		case MODEA_1:
 		case MODEA_2: case MODEA_2_:
-			assert (false);
+			// TODO but not relevant for MSX
+			break;
 		}
 		switch (control & MODE_B) {
 		case MODEB_0:
 			// do nothing
 			break;
-		case MODEB_1:		// TODO but not relevant for MSX
-			assert (false);
+		case MODEB_1:
+			// TODO but not relevant for MSX
+			break;
 		}
 	}
 }
