@@ -1,16 +1,16 @@
-// $Id: IPSPatch.cc,v 1.8 2006/01/25 19:26:31 m9710797 Exp $
+// $Id: IPSPatch.cc 5615 2006-09-08 09:07:05Z m9710797 $
 
 #include "IPSPatch.hh"
 #include "File.hh"
 #include "MSXException.hh"
-#include <string.h>
+#include <cstring>
 #include <cassert>
 
 using std::vector;
 
 namespace openmsx {
 
-unsigned IPSPatch::getStop(const IPSPatch::PatchMap::const_iterator& it)
+static unsigned getStop(const IPSPatch::PatchMap::const_iterator& it)
 {
 	return it->first + it->second.size();
 }
