@@ -1,11 +1,11 @@
-// $Id: MSXDevice.cc,v 1.85 2006/04/06 16:28:00 m9710797 Exp $
+// $Id: MSXDevice.cc 5814 2006-10-26 19:11:44Z arnoldmnl $
 
 #include "MSXDevice.hh"
 #include "XMLElement.hh"
 #include "MSXMotherBoard.hh"
 #include "CartridgeSlotManager.hh"
 #include "MSXCPUInterface.hh"
-#include "CPU.hh"
+#include "CacheLine.hh"
 #include "StringOp.hh"
 #include "MSXException.hh"
 #include <set>
@@ -159,8 +159,8 @@ void MSXDevice::registerSlots(const XMLElement& config)
 				getName() + " should be in range "
 				"[0x0000,0x10000).");
 		}
-		if ((base & CPU::CACHE_LINE_LOW) || (size & CPU::CACHE_LINE_LOW)) {
-			int tmp = CPU::CACHE_LINE_SIZE; // solves link error
+		if ((base & CacheLine::LOW) || (size & CacheLine::LOW)) {
+			int tmp = CacheLine::SIZE; // solves link error
 			throw MSXException(
 				"Invalid memory specification for device " +
 				getName() + " should be aligned at " +
@@ -191,6 +191,13 @@ void MSXDevice::registerSlots(const XMLElement& config)
 			throw MSXException("Invalid memory specification");
 		}
 	}
+	if ( (0 > ss ) && (ss >= -128) ) {
+		if ( (0 <= ps) && (ps<4) 
+			 && motherBoard.getCPUInterface().isExpanded(ps))
+			ss += 128;
+		else
+			ss = 0;
+	}
 
 	if (ps == -256) {
 		externalSlotID = slotManager.getAnyFreeSlot(
@@ -333,10 +340,10 @@ void MSXDevice::writeMem(word address, byte /*value*/,
 
 byte MSXDevice::peekMem(word address, const EmuTime& /*time*/) const
 {
-	word base = address & CPU::CACHE_LINE_HIGH;
+	word base = address & CacheLine::HIGH;
 	const byte* cache = getReadCacheLine(base);
 	if (cache) {
-		word offset = address & CPU::CACHE_LINE_LOW;
+		word offset = address & CacheLine::LOW;
 		return cache[offset];
 	} else {
 		PRT_DEBUG("MSXDevice: peek not supported for this device");
