@@ -1,4 +1,4 @@
-// $Id: MSXMatsushita.cc,v 1.27 2005/12/08 20:58:42 m9710797 Exp $
+// $Id: MSXMatsushita.cc 5680 2006-09-22 21:07:46Z m9710797 $
 
 #include "MSXMatsushita.hh"
 #include "MSXCPU.hh"
@@ -14,7 +14,8 @@ MSXMatsushita::MSXMatsushita(MSXMotherBoard& motherBoard,
                              const XMLElement& config, const EmuTime& time)
 	: MSXDevice(motherBoard, config, time)
 	, MSXSwitchedDevice(motherBoard, ID)
-	, firmwareSwitch(new FirmwareSwitch(motherBoard.getCommandController()))
+	, firmwareSwitch(
+	     new FirmwareSwitch(motherBoard.getCommandController(), config))
 	, sram(new SRAM(motherBoard, getName() + " SRAM", 0x800, config))
 {
 	// TODO find out what ports 0x41 0x45 0x46 are used for
