@@ -1,4 +1,4 @@
-// $Id: MSXS1990.cc,v 1.25 2006/01/21 10:51:58 m9710797 Exp $
+// $Id: MSXS1990.cc 5680 2006-09-22 21:07:46Z m9710797 $
 
 #include "MSXS1990.hh"
 #include "MSXCPU.hh"
@@ -23,7 +23,8 @@ private:
 MSXS1990::MSXS1990(MSXMotherBoard& motherBoard, const XMLElement& config,
                    const EmuTime& time)
 	: MSXDevice(motherBoard, config, time)
-	, firmwareSwitch(new FirmwareSwitch(motherBoard.getCommandController()))
+	, firmwareSwitch(
+	      new FirmwareSwitch(motherBoard.getCommandController(), config))
 	, debuggable(new S1990Debuggable(motherBoard, *this))
 {
 	reset(time);
