@@ -1,4 +1,4 @@
-// $Id: Math.cc,v 1.1 2006/03/20 20:26:10 mthuurne Exp $
+// $Id: Math.cc 5930 2006-12-15 19:49:51Z m9710797 $
 
 #include "Math.hh"
 
@@ -8,9 +8,14 @@ namespace Math {
 
 unsigned powerOfTwo(unsigned a)
 {
-	unsigned res = 1;
-	while (a > res) res <<= 1;
-	return res;
+	// classical implementation:
+	//   unsigned res = 1;
+	//   while (a > res) res <<= 1;
+	//   return res;
+
+	// optimized version
+	a += (a == 0); // can be removed if argument is never zero
+	return floodRight(a - 1) + 1;
 }
 
 void gaussian2(double& r1, double& r2)
