@@ -1,16 +1,17 @@
-// $Id: RealTime.cc,v 1.101 2006/05/25 12:40:08 m9710797 Exp $
+// $Id: RealTime.cc 5601 2006-09-06 13:55:13Z m9710797 $
 
 #include "RealTime.hh"
 #include "Scheduler.hh"
 #include "Timer.hh"
 #include "EventDistributor.hh"
-#include "UserInputEventDistributor.hh"
+#include "EventDelay.hh"
 #include "Event.hh"
 #include "FinishFrameEvent.hh"
 #include "GlobalSettings.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "ThrottleManager.hh"
+#include "checked_cast.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -21,11 +22,11 @@ const unsigned long long ALLOWED_LAG   =  20000; // us
 
 RealTime::RealTime(Scheduler& scheduler,
                    EventDistributor& eventDistributor_,
-                   UserInputEventDistributor& userInputEventDistributor_,
+                   EventDelay& eventDelay_,
                    GlobalSettings& globalSettings)
 	: Schedulable(scheduler)
 	, eventDistributor(eventDistributor_)
-	, userInputEventDistributor(userInputEventDistributor_)
+	, eventDelay(eventDelay_)
 	, throttleManager(globalSettings.getThrottleManager())
 	, speedSetting   (globalSettings.getSpeedSetting())
 	, pauseSetting   (globalSettings.getPauseSetting())
@@ -115,7 +116,7 @@ void RealTime::internalSync(const EmuTime& time, bool allowSleep)
 		}
 	}
 	if (allowSleep) {
-		userInputEventDistributor.sync(time);
+		eventDelay.sync(time);
 	}
 
 	emuTime = time;
@@ -133,18 +134,20 @@ const std::string& RealTime::schedName() const
 	return name;
 }
 
-void RealTime::signalEvent(const Event& event)
+bool RealTime::signalEvent(shared_ptr<const Event> event)
 {
-	if (event.getType() == OPENMSX_FINISH_FRAME_EVENT) {
-		const FinishFrameEvent& ffe = static_cast<const FinishFrameEvent&>(event);
+	if (event->getType() == OPENMSX_FINISH_FRAME_EVENT) {
+		const FinishFrameEvent& ffe =
+			checked_cast<const FinishFrameEvent&>(*event);
 		if (ffe.isSkipped()) {
 			// sync but don't sleep
 			sync(getScheduler().getCurrentTime(), false);
 		}
-	} else if (event.getType() == OPENMSX_FRAME_DRAWN_EVENT) {
+	} else if (event->getType() == OPENMSX_FRAME_DRAWN_EVENT) {
 		// sync and possibly sleep
 		sync(getScheduler().getCurrentTime(), true);
 	}
+	return true;
 }
 
 void RealTime::update(const Setting& /*setting*/)
