@@ -1,4 +1,4 @@
-// $Id: CassettePort.cc,v 1.39 2006/02/21 18:06:48 m9710797 Exp $
+// $Id: CassettePort.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 #include "CassettePort.hh"
 #include "CassettePlayer.hh"
@@ -80,9 +80,12 @@ CassettePort::CassettePort(MSXMotherBoard& motherBoard)
 	, prevTime(EmuTime::zero)
 {
 	cassettePlayer.reset(new CassettePlayer(
-		motherBoard.getCommandController(),
-		motherBoard.getMixer(),
-		motherBoard.getScheduler()));
+		motherBoard.getMSXCommandController(),
+		motherBoard.getMSXMixer(),
+		motherBoard.getScheduler(),
+		motherBoard.getMSXEventDistributor(),
+		motherBoard.getEventDistributor(),
+		motherBoard.getMSXCliComm()));
 	pluggingController.registerConnector(*this);
 	pluggingController.registerPluggable(cassettePlayer.get());
 #ifdef COMPONENT_JACK
