@@ -1,4 +1,4 @@
-// $Id: Interpreter.cc,v 1.30 2006/07/28 19:13:04 manuelbi Exp $
+// $Id: Interpreter.cc 6127 2007-03-01 09:30:39Z m9710797 $
 
 #include "Interpreter.hh"
 #include "EventDistributor.hh"
@@ -9,6 +9,9 @@
 #include "InterpreterOutput.hh"
 #include "openmsx.hh"
 #include "FileOperations.hh"
+#include "MSXMotherBoard.hh"
+#include "Reactor.hh"
+#include "MSXCommandController.hh"
 //#include <tk.h>
 
 using std::set;
@@ -57,8 +60,9 @@ void Interpreter::init(const char* programName)
 	Tcl_FindExecutable(programName);
 }
 
-Interpreter::Interpreter(EventDistributor& eventDistributor_)
+Interpreter::Interpreter(EventDistributor& eventDistributor_, Reactor& reactor_)
 	: eventDistributor(eventDistributor_)
+	, reactor(reactor_)
 {
 	interp = Tcl_CreateInterp();
 	Tcl_Preserve(interp);
@@ -228,9 +232,8 @@ const char* Interpreter::getVariable(const string& name) const
 	return Tcl_GetVar(interp, name.c_str(), 0);
 }
 
-void Interpreter::registerSetting(Setting& variable)
+void Interpreter::registerSetting(Setting& variable, const string& name)
 {
-	const string& name = variable.getName();
 	const char* tclVarValue = getVariable(name);
 	if (tclVarValue) {
 		// TCL var already existed, use this value
@@ -244,9 +247,8 @@ void Interpreter::registerSetting(Setting& variable)
 	             traceProc, static_cast<ClientData>(&variable));
 }
 
-void Interpreter::unregisterSetting(Setting& variable)
+void Interpreter::unregisterSetting(Setting& variable, const string& name)
 {
-	const string& name = variable.getName();
 	Tcl_UntraceVar(interp, name.c_str(),
 	               TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
 	               traceProc, static_cast<ClientData>(&variable));
@@ -296,22 +298,42 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 	return NULL;
 }
 
-void Interpreter::splitList(const string& list, vector<string>& result)
+void Interpreter::createNamespace(const std::string& name)
+{
+	execute("namespace eval " + name + " {}");
+}
+
+void Interpreter::deleteNamespace(const std::string& name)
+{
+	execute("namespace delete " + name);
+}
+
+void Interpreter::splitList(const std::string& list,
+	                    std::vector<std::string>& result)
+{
+	splitList(list, result, interp);
+}
+
+void Interpreter::splitList(const string& list, vector<string>& result,
+                            Tcl_Interp* interp)
 {
 	int argc;
 	const char** argv;
 	if (Tcl_SplitList(interp, list.c_str(), &argc, &argv) == TCL_ERROR) {
-		throw CommandException(Tcl_GetStringResult(interp));
+		string message = interp ? Tcl_GetStringResult(interp)
+		                        : "splitList error";
+		throw CommandException(message);
 	}
 	result.assign(argv, argv + argc);
 	Tcl_Free((char*)argv);
 }
 
-void Interpreter::signalEvent(const Event& event)
+bool Interpreter::signalEvent(shared_ptr<const Event> event)
 {
 	(void)event;
-	assert(event.getType() == OPENMSX_POLL_EVENT);
+	assert(event->getType() == OPENMSX_POLL_EVENT);
 	poll();
+	return true;
 }
 
 void Interpreter::poll()
