@@ -1,4 +1,4 @@
-// $Id: HardwareConfig.cc,v 1.21 2006/04/06 16:26:21 m9710797 Exp $
+// $Id: HardwareConfig.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 #include "HardwareConfig.hh"
 #include "XMLLoader.hh"
@@ -6,12 +6,13 @@
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
+#include "FileOperations.hh"
 #include "MSXMotherBoard.hh"
 #include "CartridgeSlotManager.hh"
 #include "MSXCPUInterface.hh"
 #include "DeviceFactory.hh"
 #include "Scheduler.hh"
-#include "CliComm.hh"
+#include "MSXCliComm.hh"
 #include <cassert>
 
 #include <iostream>
@@ -103,7 +104,8 @@ void HardwareConfig::setConfig(std::auto_ptr<XMLElement> newConfig)
 	config = newConfig;
 }
 
-void HardwareConfig::load(const string& path, const string& hwName)
+std::auto_ptr<XMLElement> HardwareConfig::loadConfig(
+		const string& path, const string& hwName)
 {
 	SystemFileContext context;
 	string filename;
@@ -115,19 +117,16 @@ void HardwareConfig::load(const string& path, const string& hwName)
 	}
 	try {
 		File file(filename);
-		config = XMLLoader::loadXML(file.getLocalName(),
-		                            "msxconfig2.dtd");
+		std::auto_ptr<XMLElement> result = XMLLoader::loadXML(
+			file.getLocalName(), "msxconfig2.dtd");
 
-		// get url
-		string url(file.getURL());
-		string::size_type pos = url.find_last_of('/');
-		assert(pos != string::npos);	// protocol must contain a '/'
-		url = url.substr(0, pos);
+		string baseName = FileOperations::getBaseName(file.getURL());
 
 		// TODO get user name
 		string userName;
-		config->setFileContext(std::auto_ptr<FileContext>(
-			new ConfigFileContext(url + '/', hwName, userName)));
+		result->setFileContext(std::auto_ptr<FileContext>(
+			new ConfigFileContext(baseName, hwName, userName)));
+		return result;
 	} catch (XMLException& e) {
 		throw MSXException(
 			"Loading of hardware configuration failed: " +
@@ -135,6 +134,11 @@ void HardwareConfig::load(const string& path, const string& hwName)
 	}
 }
 
+void HardwareConfig::load(const string& path, const string& hwName)
+{
+	setConfig(loadConfig(path, hwName));
+}
+
 void HardwareConfig::parseSlots()
 {
 	// TODO this code does parsing for both 'expanded' and 'external' slots
@@ -163,7 +167,13 @@ void HardwareConfig::parseSlots()
 			const string& secSlot = (*it2)->getAttribute("slot");
 			int ss = CartridgeSlotManager::getSlotNum(secSlot);
 			if (ss < 0) {
-				continue;
+				if ( (ss >= -128) && (0 <= ps) && (ps<4) 
+					 && motherBoard.getCPUInterface().isExpanded(ps))
+					ss += 128;
+				else {
+					ss = -256;
+					continue;
+				}
 			}
 			if (ps < 0) {
 				ps = getFreePrimarySlot();
@@ -197,7 +207,7 @@ void HardwareConfig::createDevices(const XMLElement& elem)
 			if (device.get()) {
 				addDevice(device.release());
 			} else {
-				motherBoard.getCliComm().printWarning(
+				motherBoard.getMSXCliComm().printWarning(
 					"Deprecated device: \"" +
 					name + "\", please upgrade your "
 					"hardware descriptions.");
