@@ -1,4 +1,4 @@
-// $Id: SettingsConfig.cc,v 1.30 2006/07/17 18:58:14 m9710797 Exp $
+// $Id: SettingsConfig.cc 5779 2006-10-14 16:17:32Z m9710797 $
 
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
@@ -9,7 +9,7 @@
 #include "CliComm.hh"
 #include "HotKey.hh"
 #include "CommandException.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "Command.hh"
 #include <memory>
 #include <cassert>
@@ -25,9 +25,9 @@ class SaveSettingsCommand : public SimpleCommand
 public:
 	SaveSettingsCommand(CommandController& commandController,
 			    SettingsConfig& settingsConfig);
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help   (const std::vector<std::string>& tokens) const;
-	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+	virtual string execute(const vector<string>& tokens);
+	virtual string help   (const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	SettingsConfig& settingsConfig;
 };
@@ -37,15 +37,15 @@ class LoadSettingsCommand : public SimpleCommand
 public:
 	LoadSettingsCommand(CommandController& commandController,
 			    SettingsConfig& settingsConfig);
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help   (const std::vector<std::string>& tokens) const;
-	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+	virtual string execute(const vector<string>& tokens);
+	virtual string help   (const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	SettingsConfig& settingsConfig;
 };
 
 
-SettingsConfig::SettingsConfig(CommandController& commandController_,
+SettingsConfig::SettingsConfig(GlobalCommandController& commandController_,
                                HotKey& hotKey_)
 	: XMLElement("settings")
 	, commandController(commandController_)
@@ -148,7 +148,8 @@ string SaveSettingsCommand::help(const vector<string>& /*tokens*/) const
 void SaveSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		completeFileName(tokens);
+		SystemFileContext context;
+		completeFileName(tokens, context);
 	}
 }
 
@@ -181,7 +182,8 @@ string LoadSettingsCommand::help(const vector<string>& /*tokens*/) const
 void LoadSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		completeFileName(tokens);
+		SystemFileContext context;
+		completeFileName(tokens, context);
 	}
 }
 
