@@ -1,4 +1,4 @@
-// $Id: XMLElement.cc,v 1.11 2006/05/20 19:35:20 mthuurne Exp $
+// $Id: XMLElement.cc 5615 2006-09-08 09:07:05Z m9710797 $
 
 #include "XMLElement.hh"
 #include "StringOp.hh"
@@ -78,7 +78,7 @@ double XMLElement::getDataAsDouble() const
 	return StringOp::stringToDouble(getData());
 }
 
-void XMLElement::setName(const std::string& name_)
+void XMLElement::setName(const string& name_)
 {
 	name = name_;
 }
