@@ -1,4 +1,4 @@
-// $Id: OSDConsoleRenderer.cc,v 1.78 2006/02/27 21:49:16 m9710797 Exp $
+// $Id: OSDConsoleRenderer.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 #include "OSDConsoleRenderer.hh"
 #include "CommandConsole.hh"
@@ -14,7 +14,7 @@
 #include "Timer.hh"
 #include "DummyFont.hh"
 #include "FileContext.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "Reactor.hh"
 #include <algorithm>
 
@@ -27,8 +27,7 @@ namespace openmsx {
 OSDConsoleRenderer::OSDConsoleRenderer(Reactor& reactor_)
 	: Layer(COVER_NONE, Z_CONSOLE)
 	, reactor(reactor_)
-	, consoleSetting(reactor.getCommandController().getGlobalSettings().
-	                    getConsoleSetting())
+	, consoleSetting(reactor.getGlobalSettings().getConsoleSetting())
 {
 	destX = destY = destW = destH = 0; // avoid UMR
 	font.reset(new DummyFont());
@@ -98,7 +97,7 @@ void OSDConsoleRenderer::initConsole()
 	try {
 		backgroundSetting->setChecker(this);
 	} catch (MSXException& e) {
-		reactor.getCliComm().printWarning(e.getMessage());
+		reactor.getGlobalCliComm().printWarning(e.getMessage());
 	}
 }
 
