@@ -1,4 +1,4 @@
-// $Id: SDLFont.cc,v 1.18 2005/06/12 14:12:35 mthuurne Exp $
+// $Id: SDLFont.cc 5565 2006-08-05 11:42:41Z manuelbi $
 
 /*  Written By: Garrett Banuk <mongoose@mongeese.org>
  *  This is free, just be sure to give me credit when using it
@@ -26,7 +26,7 @@ SDLFont::SDLFont(File* file, SDL_Surface* surface)
 	// load the font bitmap
 	SDL_Surface* image1;
 	if (!(image1 = IMG_Load(file->getLocalName().c_str()))) {
-		throw MSXException("Can't load font");
+		throw MSXException("Can't load font " + (file->getLocalName()));
 	}
 	fontSurface = SDL_DisplayFormatAlpha(image1);
 	SDL_FreeSurface(image1);
@@ -40,7 +40,7 @@ SDLFont::SDLFont(File* file, SDL_Surface* surface)
 		format->Rmask, format->Gmask, format->Bmask, 0);
 	if (!workImage) {
 		SDL_FreeSurface(fontSurface);
-		throw MSXException("Can't load font");
+		throw MSXException("Can't create surface for font " + (file->getLocalName()));
 	}
 }
 
