@@ -1,4 +1,4 @@
-// $Id: BreakPoint.cc,v 1.4 2006/04/20 18:13:15 m9710797 Exp $
+// $Id: BreakPoint.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 #include "BreakPoint.hh"
 #include "TclObject.hh"
@@ -7,7 +7,7 @@ namespace openmsx {
 
 unsigned BreakPoint::lastId = 0;
 
-BreakPoint::BreakPoint(CliComm& cliComm, word address_,
+BreakPoint::BreakPoint(MSXCliComm& cliComm, word address_,
                        std::auto_ptr<TclObject> command,
                        std::auto_ptr<TclObject> condition)
 	: BreakPointBase(cliComm, command, condition)
