@@ -1,9 +1,10 @@
-// $Id: MSXCPU.cc,v 1.78 2006/01/24 18:03:35 m9710797 Exp $
+// $Id: MSXCPU.cc 6112 2007-02-23 19:45:08Z m9710797 $
 
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
 #include "Debugger.hh"
 #include "SimpleDebuggable.hh"
+#include "MSXCommandController.hh"
 #include "BooleanSetting.hh"
 #include "CPUCore.hh"
 #include "Z80.hh"
@@ -20,7 +21,7 @@ namespace openmsx {
 class TimeInfoTopic : public InfoTopic
 {
 public:
-	TimeInfoTopic(CommandController& commandController,
+	TimeInfoTopic(InfoCommand& machineInfoCommand,
 	              MSXCPU& msxcpu);
 	virtual void execute(const vector<TclObject*>& tokens,
 	                     TclObject& result) const;
@@ -43,16 +44,19 @@ private:
 MSXCPU::MSXCPU(MSXMotherBoard& motherboard_)
 	: motherboard(motherboard_)
 	, traceSetting(new BooleanSetting(motherboard.getCommandController(),
-	                              "cputrace", "CPU tracing on/off", false))
+	                              "cputrace", "CPU tracing on/off", false,
+	                              Setting::DONT_SAVE))
 	, z80 (new CPUCore<Z80TYPE> (motherboard, "z80",  *traceSetting,
 	                             EmuTime::zero))
 	, r800(new CPUCore<R800TYPE>(motherboard, "r800", *traceSetting,
 	                             EmuTime::zero))
 	, reference(EmuTime::zero)
-	, timeInfo(new TimeInfoTopic(motherboard.getCommandController(), *this))
+	, timeInfo(new TimeInfoTopic(
+	     motherboard.getMSXCommandController().getMachineInfoCommand(), *this))
 	, debuggable(new MSXCPUDebuggable(motherboard_, *this))
 {
 	activeCPU = z80.get();	// setActiveCPU(CPU_Z80);
+	newCPU = 0;
 
 	motherboard.getDebugger().setCPU(this);
 	traceSetting->attach(*this);
@@ -80,25 +84,23 @@ void MSXCPU::doReset(const EmuTime& time)
 
 void MSXCPU::setActiveCPU(CPUType cpu)
 {
-	CPU* newCPU;
+	CPU* tmp;
 	switch (cpu) {
 		case CPU_Z80:
 			PRT_DEBUG("Active CPU: Z80");
-			newCPU = z80.get();
+			tmp = z80.get();
 			break;
 		case CPU_R800:
 			PRT_DEBUG("Active CPU: R800");
-			newCPU = r800.get();
+			tmp = r800.get();
 			break;
 		default:
 			assert(false);
-			newCPU = NULL;	// prevent warning
+			tmp = NULL;	// prevent warning
 	}
-	if (newCPU != activeCPU) {
-		newCPU->warp(activeCPU->getCurrentTime());
-		newCPU->invalidateMemCache(0x0000, 0x10000);
-		exitCPULoop();
-		activeCPU = newCPU;
+	if (tmp != activeCPU) {
+		exitCPULoopSync();
+		newCPU = tmp;
 	}
 }
 
@@ -109,12 +111,22 @@ void MSXCPU::setDRAMmode(bool dram)
 
 void MSXCPU::execute()
 {
+	if (newCPU) {
+		newCPU->warp(activeCPU->getCurrentTime());
+		newCPU->invalidateMemCache(0x0000, 0x10000);
+		activeCPU = newCPU;
+		newCPU = 0;
+	}
 	activeCPU->execute();
 }
 
-void MSXCPU::exitCPULoop()
+void MSXCPU::exitCPULoopSync()
 {
-	activeCPU->exitCPULoop();
+	activeCPU->exitCPULoopSync();
+}
+void MSXCPU::exitCPULoopAsync()
+{
+	activeCPU->exitCPULoopAsync();
 }
 
 
@@ -175,7 +187,7 @@ void MSXCPU::update(const Setting& setting)
 {
 	(void)setting;
 	assert(&setting == traceSetting.get());
-	exitCPULoop();
+	exitCPULoopSync();
 }
 
 // Command
@@ -230,9 +242,9 @@ void MSXCPU::setPaused(bool paused)
 
 // class TimeInfoTopic
 
-TimeInfoTopic::TimeInfoTopic(CommandController& commandController,
+TimeInfoTopic::TimeInfoTopic(InfoCommand& machineInfoCommand,
                              MSXCPU& msxcpu_)
-	: InfoTopic(commandController, "time")
+	: InfoTopic(machineInfoCommand, "time")
 	, msxcpu(msxcpu_)
 {
 }
@@ -281,7 +293,7 @@ byte MSXCPUDebuggable::read(unsigned address)
 	case 24:
 		return regs.I;
 	case 25:
-		return regs.R;
+		return (regs.R & 0x7F) | (regs.R2 & 0x80);
 	case 26:
 		return regs.IM;
 	case 27:
@@ -310,7 +322,7 @@ void MSXCPUDebuggable::write(unsigned address, byte value)
 		regs.I = value;
 		break;
 	case 25:
-		regs.R = value;
+		regs.R = regs.R2 = value;
 		break;
 	case 26:
 		if (value < 3) {
