@@ -1,9 +1,9 @@
-// $Id: MSXCPUInterface.cc,v 1.107 2006/06/05 15:10:39 m9710797 Exp $
+// $Id: MSXCPUInterface.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 #include "MSXCPUInterface.hh"
 #include "DummyDevice.hh"
 #include "SimpleDebuggable.hh"
-#include "Command.hh"
+#include "MSXCommandController.hh"
 #include "InfoTopic.hh"
 #include "CommandException.hh"
 #include "TclObject.hh"
@@ -11,19 +11,22 @@
 #include "MSXCPU.hh"
 #include "XMLElement.hh"
 #include "VDPIODelay.hh"
-#include "CliComm.hh"
+#include "MSXCliComm.hh"
 #include "MSXMultiIODevice.hh"
 #include "MSXMultiMemDevice.hh"
 #include "MSXWatchIODevice.hh"
 #include "MSXException.hh"
 #include "CartridgeSlotManager.hh"
 #include "DeviceFactory.hh"
+#include "StringOp.hh"
+#include "checked_cast.hh"
 #include <cstdio>
 #include <memory> // for auto_ptr
 #include <set>
 #include <sstream>
 #include <iomanip>
 #include <algorithm>
+#include <iostream>
 
 using std::auto_ptr;
 using std::ostringstream;
@@ -72,11 +75,11 @@ private:
 class SlotInfo : public InfoTopic
 {
 public:
-	SlotInfo(CommandController& commandController,
+	SlotInfo(InfoCommand& machineInfoCommand,
 	         MSXCPUInterface& interface);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	virtual void execute(const vector<TclObject*>& tokens,
 	                     TclObject& result) const;
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
 private:
 	MSXCPUInterface& interface;
 };
@@ -84,11 +87,11 @@ private:
 class SubSlottedInfo : public InfoTopic
 {
 public:
-	SubSlottedInfo(CommandController& commandController,
+	SubSlottedInfo(InfoCommand& machineInfoCommand,
 		       MSXCPUInterface& interface);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	virtual void execute(const vector<TclObject*>& tokens,
 			     TclObject& result) const;
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
 private:
 	MSXCPUInterface& interface;
 };
@@ -96,11 +99,11 @@ private:
 class ExternalSlotInfo : public InfoTopic
 {
 public:
-	ExternalSlotInfo(CommandController& commandController,
+	ExternalSlotInfo(InfoCommand& machineInfoCommand,
 			 CartridgeSlotManager& manager);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	virtual void execute(const vector<TclObject*>& tokens,
 			     TclObject& result) const;
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
 private:
 	CartridgeSlotManager& manager;
 };
@@ -108,11 +111,11 @@ private:
 class IOInfo : public InfoTopic
 {
 public:
-	IOInfo(CommandController& commandController,
+	IOInfo(InfoCommand& machineInfoCommand,
 	       MSXCPUInterface& interface, bool input);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	virtual void execute(const vector<TclObject*>& tokens,
 	                     TclObject& result) const;
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
 private:
 	MSXCPUInterface& interface;
 	bool input;
@@ -136,17 +139,19 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard)
 	, slottedMemoryDebug(new SlottedMemoryDebug(*this, motherBoard))
 	, ioDebug           (new IODebug           (*this, motherBoard))
 	, slotInfo        (new SlotInfo(
-		motherBoard.getCommandController(), *this))
+		motherBoard.getMSXCommandController().getMachineInfoCommand(), *this))
 	, subSlottedInfo  (new SubSlottedInfo(
-		motherBoard.getCommandController(), *this))
+		motherBoard.getMSXCommandController().getMachineInfoCommand(), *this))
 	, externalSlotInfo(new ExternalSlotInfo(
-		motherBoard.getCommandController(),
+		motherBoard.getMSXCommandController().getMachineInfoCommand(),
 		motherBoard.getSlotManager()))
-	, inputPortInfo (new IOInfo(motherBoard.getCommandController(), *this, true))
-	, outputPortInfo(new IOInfo(motherBoard.getCommandController(), *this, false))
+	, inputPortInfo (new IOInfo(
+	        motherBoard.getMSXCommandController().getMachineInfoCommand(), *this, true))
+	, outputPortInfo(new IOInfo(
+	        motherBoard.getMSXCommandController().getMachineInfoCommand(), *this, false))
 	, dummyDevice(motherBoard.getDummyDevice())
 	, msxcpu(motherBoard.getCPU())
-	, cliCommOutput(motherBoard.getCliComm())
+	, cliCommOutput(motherBoard.getMSXCliComm())
 {
 	for (int port = 0; port < 256; ++port) {
 		IO_In [port] = &dummyDevice;
@@ -175,15 +180,21 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard)
 
 MSXCPUInterface::~MSXCPUInterface()
 {
-	while (!watchPoints.empty()) {
-		removeWatchPoint(*watchPoints.front());
-	}
-	
+	removeAllWatchPoints();
 	msxcpu.setInterface(NULL);
 
+	#ifndef NDEBUG
 	for (int port = 0; port < 256; ++port) {
-		assert(IO_In [port] == &dummyDevice);
-		assert(IO_Out[port] == &dummyDevice);
+		if (IO_In[port] != &dummyDevice) {
+			std::cout << "In-port " << port << " still registered "
+			          << IO_In[port]->getName() << std::endl;
+			assert(false);
+		}
+		if (IO_Out[port] != &dummyDevice) {
+			std::cout << "Out-port " << port << " still registered "
+			          << IO_Out[port]->getName() << std::endl;
+			assert(false);
+		}
 	}
 	for (int primSlot = 0; primSlot < 4; ++primSlot) {
 		assert(!isExpanded(primSlot));
@@ -193,13 +204,22 @@ MSXCPUInterface::~MSXCPUInterface()
 			}
 		}
 	}
+	#endif
+}
+
+void MSXCPUInterface::removeAllWatchPoints()
+{
+	while (!watchPoints.empty()) {
+		removeWatchPoint(*watchPoints.back());
+	}
+	
 }
 
 byte MSXCPUInterface::readMemSlow(word address, const EmuTime& time)
 {
 	// execute read watches before actual read
-	if (unlikely(readWatchSet[address >> CPU::CACHE_LINE_BITS]
-	                         [address &  CPU::CACHE_LINE_LOW])) {
+	if (unlikely(readWatchSet[address >> CacheLine::BITS]
+	                         [address &  CacheLine::LOW])) {
 		executeMemWatch(address, WatchPoint::READ_MEM);
 	}
 	if (unlikely((address == 0xFFFF) && isExpanded(primarySlotState[3]))) {
@@ -217,8 +237,8 @@ void MSXCPUInterface::writeMemSlow(word address, byte value, const EmuTime& time
 		visibleDevices[address>>14]->writeMem(address, value, time);
 	}
 	// execute write watches after actual write
-	if (unlikely(writeWatchSet[address >> CPU::CACHE_LINE_BITS]
-	                          [address &  CPU::CACHE_LINE_LOW])) {
+	if (unlikely(writeWatchSet[address >> CacheLine::BITS]
+	                          [address &  CacheLine::LOW])) {
 		executeMemWatch(address, WatchPoint::WRITE_MEM);
 	}
 }
@@ -231,7 +251,7 @@ void MSXCPUInterface::setAllowedCache()
 		allowReadCache [255] = false;
 		allowWriteCache[255] = false;
 	}
-	for (int i = 0; i < CPU::CACHE_LINE_NUM; ++i) {
+	for (unsigned i = 0; i < CacheLine::NUM; ++i) {
 		if (readWatchSet [i].any()) allowReadCache [i] = false;
 		if (writeWatchSet[i].any()) allowWriteCache[i] = false;
 	}
@@ -251,7 +271,7 @@ void MSXCPUInterface::setExpanded(int ps)
 }
 
 void MSXCPUInterface::testUnsetExpanded(
-		int ps, std::vector<MSXDevice*>& alreadyRemoved) const
+		int ps, vector<MSXDevice*>& alreadyRemoved) const
 {
 	// TODO handle multi-devices
 	std::set<MSXDevice*> allowed(alreadyRemoved.begin(), alreadyRemoved.end());
@@ -562,7 +582,7 @@ void MSXCPUInterface::writeSlottedMem(unsigned address, byte value,
 }
 
 
-void MSXCPUInterface::setWatchPoint(std::auto_ptr<WatchPoint> watchPoint_)
+void MSXCPUInterface::setWatchPoint(auto_ptr<WatchPoint> watchPoint_)
 {
 	WatchPoint* watchPoint = watchPoint_.release();
 	watchPoints.push_back(watchPoint);
@@ -619,35 +639,54 @@ void MSXCPUInterface::registerIOWatch(WatchPoint& watchPoint, MSXDevice** device
 {
 	assert(dynamic_cast<MSXWatchIODevice*>(&watchPoint));
 	MSXWatchIODevice& ioWatch = static_cast<MSXWatchIODevice&>(watchPoint);
-	unsigned port = ioWatch.getAddress();
-	assert(port < 256);
-	ioWatch.getDevicePtr() = devices[port];
-	devices[port] = &ioWatch;
+	unsigned beginPort = ioWatch.getBeginAddress();
+	unsigned endPort   = ioWatch.getEndAddress();
+	assert(beginPort <= endPort);
+	assert(endPort < 0x100);
+	for (unsigned port = beginPort; port <= endPort; ++port) {
+		ioWatch.getDevicePtr() = devices[port];
+		devices[port] = &ioWatch;
+	}
 }
 
 void MSXCPUInterface::unregisterIOWatch(WatchPoint& watchPoint, MSXDevice** devices)
 {
-	assert(dynamic_cast<MSXWatchIODevice*>(&watchPoint));
-	MSXWatchIODevice& ioWatch = static_cast<MSXWatchIODevice&>(watchPoint);
-	unsigned port = ioWatch.getAddress();
-	assert(port < 256);
-	devices[port] = ioWatch.getDevicePtr();
+	const MSXWatchIODevice& ioWatch =
+		checked_cast<const MSXWatchIODevice&>(watchPoint);
+	unsigned beginPort = ioWatch.getBeginAddress();
+	unsigned endPort   = ioWatch.getEndAddress();
+	assert(beginPort <= endPort);
+	assert(endPort < 0x100);
+
+	for (unsigned port = beginPort; port <= endPort; ++port) {
+		// find pointer to watchpoint
+		MSXDevice** prev = &devices[port];
+		while (*prev != &ioWatch) {
+			prev = &checked_cast<MSXWatchIODevice*>(*prev)->getDevicePtr();
+		}
+		// remove watchpoint from chain
+		*prev = checked_cast<MSXWatchIODevice*>(*prev)->getDevicePtr();
+	}
 }
 
 void MSXCPUInterface::updateMemWatch(WatchPoint::Type type)
 {
-	std::bitset<CPU::CACHE_LINE_SIZE>* watchSet =
+	std::bitset<CacheLine::SIZE>* watchSet =
 		(type == WatchPoint::READ_MEM) ? readWatchSet : writeWatchSet;
-	for (int i = 0; i < CPU::CACHE_LINE_NUM; ++i) {
+	for (unsigned i = 0; i < CacheLine::NUM; ++i) {
 		watchSet[i].reset();
 	}
 	for (WatchPoints::const_iterator it = watchPoints.begin();
 	     it != watchPoints.end(); ++it) {
 		if ((*it)->getType() == type) {
-			unsigned addr = (*it)->getAddress();
-			assert(addr < 0x10000);
-			watchSet[addr >> CPU::CACHE_LINE_BITS].set(
-			         addr  & CPU::CACHE_LINE_LOW);
+			unsigned beginAddr = (*it)->getBeginAddress();
+			unsigned endAddr   = (*it)->getEndAddress();
+			assert(beginAddr <= endAddr);
+			assert(endAddr < 0x10000);
+			for (unsigned addr = beginAddr; addr <= endAddr; ++addr) {
+				watchSet[addr >> CacheLine::BITS].set(
+				         addr  & CacheLine::LOW);
+			}
 		}
 	}
 	setAllowedCache();
@@ -658,7 +697,9 @@ void MSXCPUInterface::executeMemWatch(word address, WatchPoint::Type type)
 {
 	for (WatchPoints::const_iterator it = watchPoints.begin();
 	     it != watchPoints.end(); ++it) {
-		if (((*it)->getAddress() == address) && ((*it)->getType() == type)) {
+		if (((*it)->getBeginAddress() <= address) &&
+		    ((*it)->getEndAddress()   >= address) &&
+		    ((*it)->getType()         == type)) {
 			(*it)->checkAndExecute();
 		}
 	}
@@ -719,14 +760,14 @@ static unsigned getSlot(TclObject* token, const string& itemName)
 	return slot;
 }
 
-SlotInfo::SlotInfo(CommandController& commandController,
+SlotInfo::SlotInfo(InfoCommand& machineInfoCommand,
                    MSXCPUInterface& interface_)
-	: InfoTopic(commandController, "slot")
+	: InfoTopic(machineInfoCommand, "slot")
 	, interface(interface_)
 {
 }
 
-void SlotInfo::execute(const std::vector<TclObject*>& tokens,
+void SlotInfo::execute(const vector<TclObject*>& tokens,
                        TclObject& result) const
 {
 	if (tokens.size() != 5) {
@@ -741,7 +782,7 @@ void SlotInfo::execute(const std::vector<TclObject*>& tokens,
 	result.setString(interface.slotLayout[ps][ss][page]->getName());
 }
 
-std::string SlotInfo::help(const std::vector<std::string>& /*tokens*/) const
+string SlotInfo::help(const vector<string>& /*tokens*/) const
 {
 	return "Retrieve name of the device inserted in given "
 	       "primary slot / secondary slot / page.";
@@ -750,14 +791,14 @@ std::string SlotInfo::help(const std::vector<std::string>& /*tokens*/) const
 
 // class SubSlottedInfo
 
-SubSlottedInfo::SubSlottedInfo(CommandController& commandController,
+SubSlottedInfo::SubSlottedInfo(InfoCommand& machineInfoCommand,
                                MSXCPUInterface& interface_)
-	: InfoTopic(commandController, "issubslotted")
+	: InfoTopic(machineInfoCommand, "issubslotted")
 	, interface(interface_)
 {
 }
 
-void SubSlottedInfo::execute(const std::vector<TclObject*>& tokens,
+void SubSlottedInfo::execute(const vector<TclObject*>& tokens,
                              TclObject& result) const
 {
 	if (tokens.size() != 3) {
@@ -766,8 +807,8 @@ void SubSlottedInfo::execute(const std::vector<TclObject*>& tokens,
 	result.setInt(interface.isExpanded(getSlot(tokens[2], "Slot")));
 }
 
-std::string SubSlottedInfo::help(
-	const std::vector<std::string>& /*tokens*/) const
+string SubSlottedInfo::help(
+	const vector<string>& /*tokens*/) const
 {
 	return "Indicates whether a certain primary slot is expanded.";
 }
@@ -775,14 +816,14 @@ std::string SubSlottedInfo::help(
 
 // class ExternalSlotInfo
 
-ExternalSlotInfo::ExternalSlotInfo(CommandController& commandController,
+ExternalSlotInfo::ExternalSlotInfo(InfoCommand& machineInfoCommand,
                                    CartridgeSlotManager& manager_)
-	: InfoTopic(commandController, "isexternalslot")
+	: InfoTopic(machineInfoCommand, "isexternalslot")
 	, manager(manager_)
 {
 }
 
-void ExternalSlotInfo::execute(const std::vector<TclObject*>& tokens,
+void ExternalSlotInfo::execute(const vector<TclObject*>& tokens,
                                TclObject& result) const
 {
 	int ps = 0;
@@ -800,8 +841,8 @@ void ExternalSlotInfo::execute(const std::vector<TclObject*>& tokens,
 	result.setInt(manager.isExternalSlot(ps, ss, true));
 }
 
-std::string ExternalSlotInfo::help(
-	const std::vector<std::string>& /*tokens*/) const
+string ExternalSlotInfo::help(
+	const vector<string>& /*tokens*/) const
 {
 	return "Indicates whether a certain slot is external or internal.";
 }
@@ -829,9 +870,9 @@ void IODebug::write(unsigned address, byte value, const EmuTime& time)
 
 // class IOInfo
 
-IOInfo::IOInfo(CommandController& commandController,
+IOInfo::IOInfo(InfoCommand& machineInfoCommand,
                MSXCPUInterface& interface_, bool input_)
-	: InfoTopic(commandController, input_ ? "input_port" : "output_port")
+	: InfoTopic(machineInfoCommand, input_ ? "input_port" : "output_port")
 	, interface(interface_), input(input_)
 {
 }
@@ -873,6 +914,7 @@ TurborCPUInterface::TurborCPUInterface(MSXMotherBoard& motherBoard)
 
 TurborCPUInterface::~TurborCPUInterface()
 {
+	removeAllWatchPoints();
 	for (int port = 0x98; port <= 0x9B; ++port) {
 		assert(IO_In [port] == delayDevice.get());
 		assert(IO_Out[port] == delayDevice.get());
