@@ -1,20 +1,20 @@
-// $Id: MSXWatchIODevice.cc,v 1.1 2006/04/20 18:13:15 m9710797 Exp $
+// $Id: MSXWatchIODevice.cc 5868 2006-11-08 19:25:30Z m9710797 $
 
 #include "MSXWatchIODevice.hh"
 #include "MSXMotherBoard.hh"
 #include "TclObject.hh"
 #include <cassert>
 
-#include <iostream>
-
 namespace openmsx {
 
 MSXWatchIODevice::MSXWatchIODevice(MSXMotherBoard& motherboard,
-                                   WatchPoint::Type type, unsigned address,
+                                   WatchPoint::Type type, 
+                                   unsigned beginAddr, unsigned endAddr,
                                    std::auto_ptr<TclObject> command,
                                    std::auto_ptr<TclObject> condition)
 	: MSXMultiDevice(motherboard)
-	, WatchPoint(motherboard.getCliComm(), command, condition, type, address)
+	, WatchPoint(motherboard.getMSXCliComm(), command, condition, type,
+	             beginAddr, endAddr)
 	, device(0)
 {
 }
@@ -43,7 +43,7 @@ byte MSXWatchIODevice::peekIO(word port, const EmuTime& time) const
 byte MSXWatchIODevice::readIO(word port, const EmuTime& time)
 {
 	assert(device);
-	std::cout << "Watch readIO " << port << std::endl;
+	//std::cout << "Watch readIO " << port << std::endl;
 	byte result = device->readIO(port, time);
 	checkAndExecute();
 	return result;
@@ -52,7 +52,7 @@ byte MSXWatchIODevice::readIO(word port, const EmuTime& time)
 void MSXWatchIODevice::writeIO(word port, byte value, const EmuTime& time)
 {
 	assert(device);
-	std::cout << "Watch writeIO " << port << " " << value << std::endl;
+	//std::cout << "Watch writeIO " << port << " " << value << std::endl;
 	device->writeIO(port, value, time);
 	checkAndExecute();
 }
