@@ -1,19 +1,21 @@
-// $Id: WatchPoint.cc,v 1.1 2006/04/20 18:13:15 m9710797 Exp $
+// $Id: WatchPoint.cc 5868 2006-11-08 19:25:30Z m9710797 $
 
 #include "WatchPoint.hh"
 #include "TclObject.hh"
+#include <cassert>
 
 namespace openmsx {
 
 unsigned WatchPoint::lastId = 0;
 
-WatchPoint::WatchPoint(CliComm& cliComm,
+WatchPoint::WatchPoint(MSXCliComm& cliComm,
                        std::auto_ptr<TclObject> command,
                        std::auto_ptr<TclObject> condition,
-                       Type type_, unsigned address_)
+                       Type type_, unsigned beginAddr_, unsigned endAddr_)
 	: BreakPointBase(cliComm, command, condition)
-	, type(type_), address(address_)
+	, type(type_), beginAddr(beginAddr_), endAddr(endAddr_)
 {
+	assert(beginAddr <= endAddr);
 	id = ++lastId;
 }
 
@@ -31,9 +33,14 @@ WatchPoint::Type WatchPoint::getType() const
 	return type;
 }
 
-unsigned WatchPoint::getAddress() const
+unsigned WatchPoint::getBeginAddress() const
 {
-	return address;
+	return beginAddr;
+}
+
+unsigned WatchPoint::getEndAddress() const
+{
+	return endAddr;
 }
 
 } // namespace openmsx
