@@ -1,10 +1,13 @@
-// $Id: AfterCommand.cc,v 1.30 2006/02/21 18:06:48 m9710797 Exp $
+// $Id: AfterCommand.cc 5993 2007-01-12 22:12:12Z m9710797 $
 
 #include "AfterCommand.hh"
 #include "CommandController.hh"
 #include "CliComm.hh"
 #include "Scheduler.hh"
+#include "Schedulable.hh"
 #include "EventDistributor.hh"
+#include "Reactor.hh"
+#include "MSXMotherBoard.hh"
 #include "CommandException.hh"
 #include <cstdlib>
 #include <sstream>
@@ -16,13 +19,79 @@ using std::set;
 
 namespace openmsx {
 
-AfterCommand::AfterCommand(Scheduler& scheduler_,
+class AfterCmd
+{
+public:
+	virtual ~AfterCmd();
+	const std::string& getCommand() const;
+	const std::string& getId() const;
+	virtual const std::string& getType() const = 0;
+	void execute();
+protected:
+	AfterCmd(AfterCommand& afterCommand,
+		 const std::string& command);
+private:
+	AfterCommand& afterCommand;
+	std::string command;
+	std::string id;
+	static unsigned lastAfterId;
+};
+
+class AfterTimedCmd : public AfterCmd, private Schedulable
+{
+public:
+	double getTime() const;
+	void reschedule();
+protected:
+	AfterTimedCmd(Scheduler& scheduler,
+		      AfterCommand& afterCommand,
+		      const std::string& command, double time);
+private:
+	virtual void executeUntil(const EmuTime& time, int userData);
+	virtual void schedulerDeleted();
+	virtual const std::string& schedName() const;
+
+	double time;
+};
+
+class AfterTimeCmd : public AfterTimedCmd
+{
+public:
+	AfterTimeCmd(Scheduler& scheduler,
+		     AfterCommand& afterCommand,
+		     const std::string& command, double time);
+	virtual const std::string& getType() const;
+};
+
+class AfterIdleCmd : public AfterTimedCmd
+{
+public:
+	AfterIdleCmd(Scheduler& scheduler,
+		     AfterCommand& afterCommand,
+		     const std::string& command, double time);
+	virtual const std::string& getType() const;
+};
+
+template<EventType T>
+class AfterEventCmd : public AfterCmd
+{
+public:
+	AfterEventCmd(AfterCommand& afterCommand,
+		      const std::string& type,
+		      const std::string& command);
+	virtual const std::string& getType() const;
+private:
+	const std::string type;
+};
+
+
+
+AfterCommand::AfterCommand(Reactor& reactor_,
                            EventDistributor& eventDistributor_,
-                           CommandController& commandController_)
-	: SimpleCommand(commandController_, "after")
-	, scheduler(scheduler_)
+                           CommandController& commandController)
+	: SimpleCommand(commandController, "after")
+	, reactor(reactor_)
 	, eventDistributor(eventDistributor_)
-	, commandController(commandController_)
 {
 	// TODO DETACHED <-> EMU types should be cleaned up
 	//      (moved to event iso listener?)
@@ -46,6 +115,10 @@ AfterCommand::AfterCommand(Scheduler& scheduler_,
 		OPENMSX_FINISH_FRAME_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_BREAK_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_BOOT_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_MACHINE_LOADED_EVENT, *this);
 }
 
 AfterCommand::~AfterCommand()
@@ -54,6 +127,10 @@ AfterCommand::~AfterCommand()
 		delete afterCmds.begin()->second; // removes itself from map
 	}
 
+	eventDistributor.unregisterEventListener(
+		OPENMSX_MACHINE_LOADED_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_BOOT_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_BREAK_EVENT, *this);
 	eventDistributor.unregisterEventListener(
@@ -76,11 +153,6 @@ AfterCommand::~AfterCommand()
 		OPENMSX_KEY_UP_EVENT, *this);
 }
 
-CommandController& AfterCommand::getCommandController() const
-{
-	return commandController;
-}
-
 string AfterCommand::execute(const vector<string>& tokens)
 {
 	if (tokens.size() < 2) {
@@ -94,6 +166,10 @@ string AfterCommand::execute(const vector<string>& tokens)
 		return afterEvent<OPENMSX_FINISH_FRAME_EVENT>(tokens);
 	} else if (tokens[1] == "break") {
 		return afterEvent<OPENMSX_BREAK_EVENT>(tokens);
+	} else if (tokens[1] == "boot") {
+		return afterEvent<OPENMSX_BOOT_EVENT>(tokens);
+	} else if (tokens[1] == "machine_switch") {
+		return afterEvent<OPENMSX_MACHINE_LOADED_EVENT>(tokens);
 	} else if (tokens[1] == "info") {
 		return afterInfo(tokens);
 	} else if (tokens[1] == "cancel") {
@@ -119,8 +195,13 @@ string AfterCommand::afterTime(const vector<string>& tokens)
 	if (tokens.size() != 4) {
 		throw SyntaxError();
 	}
+	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
+	if (!motherBoard) {
+		return "";
+	}
 	double time = getTime(tokens[2]);
-	AfterTimeCmd* cmd = new AfterTimeCmd(scheduler, *this, tokens[3], time);
+	AfterTimeCmd* cmd = new AfterTimeCmd(
+		motherBoard->getScheduler(), *this, tokens[3], time);
 	return cmd->getId();
 }
 
@@ -129,8 +210,13 @@ string AfterCommand::afterIdle(const vector<string>& tokens)
 	if (tokens.size() != 4) {
 		throw SyntaxError();
 	}
+	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
+	if (!motherBoard) {
+		return "";
+	}
 	double time = getTime(tokens[2]);
-	AfterIdleCmd* cmd = new AfterIdleCmd(scheduler, *this, tokens[3], time);
+	AfterIdleCmd* cmd = new AfterIdleCmd(
+		motherBoard->getScheduler(), *this, tokens[3], time);
 	return cmd->getId();
 }
 
@@ -140,7 +226,7 @@ string AfterCommand::afterEvent(const vector<string>& tokens)
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	AfterEventCmd<T>* cmd = new AfterEventCmd<T>(*this, tokens[2]);
+	AfterEventCmd<T>* cmd = new AfterEventCmd<T>(*this, tokens[1], tokens[2]);
 	return cmd->getId();
 }
 
@@ -195,6 +281,8 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 		cmds.insert("idle");
 		cmds.insert("frame");
 		cmds.insert("break");
+		cmds.insert("boot");
+		cmds.insert("machine_switch");
 		cmds.insert("info");
 		cmds.insert("cancel");
 		completeString(tokens, cmds);
@@ -203,12 +291,16 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 }
 
 
-void AfterCommand::signalEvent(const Event& event)
+bool AfterCommand::signalEvent(shared_ptr<const Event> event)
 {
-	if (event.getType() == OPENMSX_FINISH_FRAME_EVENT) {
+	if (event->getType() == OPENMSX_FINISH_FRAME_EVENT) {
 		executeEvents<OPENMSX_FINISH_FRAME_EVENT>();
-	} else if (event.getType() == OPENMSX_BREAK_EVENT) {
+	} else if (event->getType() == OPENMSX_BREAK_EVENT) {
 		executeEvents<OPENMSX_BREAK_EVENT>();
+	} else if (event->getType() == OPENMSX_BOOT_EVENT) {
+		executeEvents<OPENMSX_BOOT_EVENT>();
+	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
+		executeEvents<OPENMSX_MACHINE_LOADED_EVENT>();
 	} else {
 		for (AfterCmdMap::const_iterator it = afterCmds.begin();
 		     it != afterCmds.end(); ++it) {
@@ -217,6 +309,7 @@ void AfterCommand::signalEvent(const Event& event)
 			}
 		}
 	}
+	return true;
 }
 
 template<EventType T> void AfterCommand::executeEvents()
@@ -237,9 +330,9 @@ template<EventType T> void AfterCommand::executeEvents()
 
 // class AfterCmd
 
-unsigned AfterCommand::AfterCmd::lastAfterId = 0;
+unsigned AfterCmd::lastAfterId = 0;
 
-AfterCommand::AfterCmd::AfterCmd(AfterCommand& afterCommand_,
+AfterCmd::AfterCmd(AfterCommand& afterCommand_,
                                  const string& command_)
 	: afterCommand(afterCommand_), command(command_)
 {
@@ -250,22 +343,22 @@ AfterCommand::AfterCmd::AfterCmd(AfterCommand& afterCommand_,
 	afterCommand.afterCmds[id] = this;
 }
 
-AfterCommand::AfterCmd::~AfterCmd()
+AfterCmd::~AfterCmd()
 {
 	afterCommand.afterCmds.erase(id);
 }
 
-const string& AfterCommand::AfterCmd::getCommand() const
+const string& AfterCmd::getCommand() const
 {
 	return command;
 }
 
-const string& AfterCommand::AfterCmd::getId() const
+const string& AfterCmd::getId() const
 {
 	return id;
 }
 
-void AfterCommand::AfterCmd::execute()
+void AfterCmd::execute()
 {
 	try {
 		afterCommand.getCommandController().executeCommand(command);
@@ -279,7 +372,7 @@ void AfterCommand::AfterCmd::execute()
 
 // class  AfterTimedCmd
 
-AfterCommand::AfterTimedCmd::AfterTimedCmd(
+AfterTimedCmd::AfterTimedCmd(
 		Scheduler& scheduler,
 		AfterCommand& afterCommand,
 		const string& command, double time_)
@@ -291,25 +384,30 @@ AfterCommand::AfterTimedCmd::AfterTimedCmd(
 	reschedule();
 }
 
-double AfterCommand::AfterTimedCmd::getTime() const
+double AfterTimedCmd::getTime() const
 {
 	return time;
 }
 
-void AfterCommand::AfterTimedCmd::reschedule()
+void AfterTimedCmd::reschedule()
 {
 	removeSyncPoint();
 	EmuTime t = getScheduler().getCurrentTime() + EmuDuration(time);
 	setSyncPoint(t);
 }
 
-void AfterCommand::AfterTimedCmd::executeUntil(const EmuTime& /*time*/,
+void AfterTimedCmd::executeUntil(const EmuTime& /*time*/,
                                                int /*userData*/)
 {
 	execute();
 }
 
-const string& AfterCommand::AfterTimedCmd::schedName() const
+void AfterTimedCmd::schedulerDeleted()
+{
+	delete this;
+}
+
+const string& AfterTimedCmd::schedName() const
 {
 	static const string sched_name("AfterCmd");
 	return sched_name;
@@ -318,7 +416,7 @@ const string& AfterCommand::AfterTimedCmd::schedName() const
 
 // class AfterTimeCmd
 
-AfterCommand::AfterTimeCmd::AfterTimeCmd(
+AfterTimeCmd::AfterTimeCmd(
 		Scheduler& scheduler,
 		AfterCommand& afterCommand,
 		const string& command, double time)
@@ -326,7 +424,7 @@ AfterCommand::AfterTimeCmd::AfterTimeCmd(
 {
 }
 
-const string& AfterCommand::AfterTimeCmd::getType() const
+const string& AfterTimeCmd::getType() const
 {
 	static const string type("time");
 	return type;
@@ -335,7 +433,7 @@ const string& AfterCommand::AfterTimeCmd::getType() const
 
 // class AfterIdleCmd
 
-AfterCommand::AfterIdleCmd::AfterIdleCmd(
+AfterIdleCmd::AfterIdleCmd(
 		Scheduler& scheduler,
 		AfterCommand& afterCommand,
 		const string& command, double time)
@@ -343,7 +441,7 @@ AfterCommand::AfterIdleCmd::AfterIdleCmd(
 {
 }
 
-const string& AfterCommand::AfterIdleCmd::getType() const
+const string& AfterIdleCmd::getType() const
 {
 	static const string type("idle");
 	return type;
@@ -353,25 +451,17 @@ const string& AfterCommand::AfterIdleCmd::getType() const
 // class AfterEventCmd
 
 template<EventType T>
-AfterCommand::AfterEventCmd<T>::AfterEventCmd(
-		AfterCommand& afterCommand, const string& command)
-	: AfterCmd(afterCommand, command)
+AfterEventCmd<T>::AfterEventCmd(
+		AfterCommand& afterCommand, const string& type_,
+		const string& command)
+	: AfterCmd(afterCommand, command), type(type_)
 {
 }
 
-template<>
-const string& AfterCommand::AfterEventCmd<OPENMSX_FINISH_FRAME_EVENT>::getType() const
+template<EventType T>
+const string& AfterEventCmd<T>::getType() const
 {
-	static const string type("frame");
 	return type;
 }
 
-template<>
-const string& AfterCommand::AfterEventCmd<OPENMSX_BREAK_EVENT>::getType() const
-{
-	static const string type("break");
-	return type;
-}
-
-
 } // namespace openmsx
