@@ -1,4 +1,4 @@
-// $Id: CliConnection.cc,v 1.14 2006/06/05 16:33:30 m9710797 Exp $
+// $Id: CliConnection.cc 5731 2006-10-01 19:54:47Z m9710797 $
 
 // TODO:
 // - To avoid any possible conflicts, anything called from "run" should be
@@ -13,6 +13,7 @@
 #include "CommandController.hh"
 #include "CommandException.hh"
 #include "XMLElement.hh"
+#include "checked_cast.hh"
 #include <cstdio>
 #include <cassert>
 #include <iostream>
@@ -110,20 +111,21 @@ static string reply(const string& message, bool status)
 	       XMLElement::XMLEscape(message) + "</reply>\n";
 }
 
-void CliConnection::signalEvent(const Event& event)
+bool CliConnection::signalEvent(shared_ptr<const Event> event)
 {
-	assert(dynamic_cast<const CliCommandEvent*>(&event));
 	const CliCommandEvent& commandEvent =
-		static_cast<const CliCommandEvent&>(event);
-	if (commandEvent.getId() != this) return;
-	try {
-		string result = commandController.executeCommand(
-			commandEvent.getCommand(), this);
-		output(reply(result, true));
-	} catch (CommandException& e) {
-		string result = e.getMessage() + '\n';
-		output(reply(result, false));
+		checked_cast<const CliCommandEvent&>(*event);
+	if (commandEvent.getId() == this) {
+		try {
+			string result = commandController.executeCommand(
+				commandEvent.getCommand(), this);
+			output(reply(result, true));
+		} catch (CommandException& e) {
+			string result = e.getMessage() + '\n';
+			output(reply(result, false));
+		}
 	}
+	return true;
 }
 
 void CliConnection::cb_start_element(ParseState* user_data,
