@@ -1,17 +1,13 @@
-// $Id: CliServer.cc,v 1.15 2006/07/23 00:59:34 mthuurne Exp $
+// $Id: CliServer.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 #include "CliServer.hh"
-#include "CommandController.hh"
-#include "IntegerSetting.hh"
-#include "EnumSetting.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "CliConnection.hh"
 #include "StringOp.hh"
 #include "FileOperations.hh"
 #include "MSXException.hh"
 #include <string>
 #include <memory>
-#include <cassert>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -220,12 +216,13 @@ static void deleteSocket(const string& socket)
 
 
 CliServer::CliServer(CommandController& commandController_,
-                     EventDistributor& eventDistributor_)
+                     EventDistributor& eventDistributor_,
+                     GlobalCliComm& cliComm_)
 	: thread(this)
 	, listenSock(INVALID_SOCKET)
 	, commandController(commandController_)
 	, eventDistributor(eventDistributor_)
-	, cliComm(commandController.getCliComm())
+	, cliComm(cliComm_)
 {
 	exitLoop = false;
 	sock_startup();
