@@ -1,4 +1,4 @@
-// $Id: DiskChanger.cc,v 1.13 2006/04/11 20:40:19 manuelbi Exp $
+// $Id: DiskChanger.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 #include "DiskChanger.hh"
 #include "DummyDisk.hh"
@@ -8,13 +8,17 @@
 #include "DSKDiskImage.hh"
 #include "CommandController.hh"
 #include "Command.hh"
-#include "FileManipulator.hh"
+#include "MSXEventDistributor.hh"
+#include "InputEvents.hh"
+#include "Scheduler.hh"
+#include "DiskManipulator.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "CommandException.hh"
-#include "CliComm.hh"
-#include "GlobalSettings.hh"
+#include "MSXCliComm.hh"
 #include "TclObject.hh"
+#include "EmuTime.hh"
+#include "checked_cast.hh"
 
 using std::set;
 using std::string;
@@ -27,29 +31,40 @@ class DiskCommand : public Command
 public:
 	DiskCommand(CommandController& commandController,
 	            DiskChanger& diskChanger);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	virtual void execute(const vector<TclObject*>& tokens,
 	                     TclObject& result);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
-	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	DiskChanger& diskChanger;
 };
 
 DiskChanger::DiskChanger(const string& driveName_,
                          CommandController& commandController,
-                         FileManipulator& manipulator_)
-	: driveName(driveName_), manipulator(manipulator_)
+                         DiskManipulator& manipulator_,
+                         MSXEventDistributor* msxEventDistributor_,
+                         Scheduler* scheduler_)
+	: driveName(driveName_)
+	, manipulator(manipulator_)
 	, diskCommand(new DiskCommand(commandController, *this))
 	, cliComm(commandController.getCliComm())
 	, globalSettings(commandController.getGlobalSettings())
+	, msxEventDistributor(msxEventDistributor_)
+	, scheduler(scheduler_)
 {
 	ejectDisk();
-	manipulator.registerDrive(*this, driveName);
+	manipulator.registerDrive(*this);
+	if (msxEventDistributor) {
+		msxEventDistributor->registerEventListener(*this);
+	}
 }
 
 DiskChanger::~DiskChanger()
 {
-	manipulator.unregisterDrive(*this, driveName);
+	if (msxEventDistributor) {
+		msxEventDistributor->unregisterEventListener(*this);
+	}
+	manipulator.unregisterDrive(*this);
 }
 
 const string& DiskChanger::getDriveName() const
@@ -84,11 +99,45 @@ SectorAccessibleDisk* DiskChanger::getSectorAccessibleDisk()
 	return dynamic_cast<SectorAccessibleDisk*>(disk.get());
 }
 
-void DiskChanger::insertDisk(const string& diskImage,
-                             const vector<string>& patches)
+const std::string& DiskChanger::getContainerName() const
+{
+	return getDriveName();
+}
+
+void DiskChanger::sendChangeDiskEvent(const vector<string>& args)
+{
+	// note: might throw MSXException
+	MSXEventDistributor::EventPtr event(new MSXCommandEvent(args));
+	if (msxEventDistributor) {
+		msxEventDistributor->distributeEvent(
+			event, scheduler->getCurrentTime());
+	} else {
+		signalEvent(event, EmuTime::zero);
+	}
+}
+
+void DiskChanger::signalEvent(
+	shared_ptr<const Event> event, const EmuTime& /*time*/)
+{
+	if (event->getType() != OPENMSX_MSX_COMMAND_EVENT) return;
+
+	const MSXCommandEvent* commandEvent =
+		checked_cast<const MSXCommandEvent*>(event.get());
+	const vector<TclObject*>& tokens = commandEvent->getTokens();
+	if (tokens[0]->getString() == getDriveName()) {
+		if (tokens[1]->getString() == "eject") {
+			ejectDisk();
+		} else {
+			insertDisk(tokens);
+		}
+	}
+}
+
+void DiskChanger::insertDisk(const vector<TclObject*>& args)
 {
 	std::auto_ptr<Disk> newDisk;
-	if (diskImage == "-ramdsk") {
+	const string& diskImage = args[1]->getString();
+	if (diskImage == "ramdsk") {
 		newDisk.reset(new RamDSKDiskImage());
 	} else {
 		try {
@@ -109,9 +158,8 @@ void DiskChanger::insertDisk(const string& diskImage,
 			}
 		}
 	}
-	for (vector<string>::const_iterator it = patches.begin();
-	     it != patches.end(); ++it) {
-		disk->applyPatch(*it);
+	for (unsigned i = 2; i < args.size(); ++i) {
+		newDisk->applyPatch(args[i]->getString());
 	}
 
 	// no errors, only now replace original disk
@@ -142,10 +190,10 @@ DiskCommand::DiskCommand(CommandController& commandController,
 
 void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 {
+	int firstFileToken = 1;
 	if (tokens.size() == 1) {
-		const string& diskName = diskChanger.getDiskName();
 		result.addListElement(diskChanger.getDriveName() + ':');
-		result.addListElement(diskName);
+		result.addListElement(diskChanger.getDiskName());
 
 		TclObject options(result.getInterpreter());
 		if (dynamic_cast<DummyDisk*>(diskChanger.disk.get())) {
@@ -162,25 +210,47 @@ void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 			result.addListElement(options);
 		}
 
+	} else if (tokens[1]->getString() == "ramdsk") {
+		vector<string> args;
+		args.push_back(diskChanger.getDriveName());
+		args.push_back(tokens[1]->getString());
+		diskChanger.sendChangeDiskEvent(args);
 	} else if (tokens[1]->getString() == "-ramdsk") {
-		vector<string> nopatchfiles;
-		diskChanger.insertDisk(tokens[1]->getString(), nopatchfiles);
-	} else if (tokens[1]->getString() == "-eject") {
-		diskChanger.ejectDisk();
-	} else if (tokens[1]->getString() == "eject") {
-		diskChanger.ejectDisk();
+		vector<string> args;
+		args.push_back(diskChanger.getDriveName());
+		args.push_back("ramdsk");
+		diskChanger.sendChangeDiskEvent(args);
 		result.setString(
-			"Warning: use of 'eject' is deprecated, instead use '-eject'");
+			"Warning: use of '-ramdsk' is deprecated, instead use the 'ramdsk' subcommand");
+	} else if (tokens[1]->getString() == "-eject") {
+		vector<string> args;
+		args.push_back(diskChanger.getDriveName());
+		args.push_back("eject");
+		diskChanger.sendChangeDiskEvent(args);
+		result.setString(
+			"Warning: use of '-eject' is deprecated, instead use the 'eject' subcommand");
+	} else if (tokens[1]->getString() == "eject") {
+		vector<string> args;
+		args.push_back(diskChanger.getDriveName());
+		args.push_back("eject");
+		diskChanger.sendChangeDiskEvent(args);
 	} else {
+		if (tokens[1]->getString() == "insert") {
+			if (tokens.size() > 2) {
+				firstFileToken = 2; // skip this subcommand as filearg
+			} else {
+				throw CommandException("Missing argument to insert subcommand");
+			}
+		}
 		try {
 			UserFileContext context(getCommandController());
-			vector<string> patches;
-			for (unsigned i = 2; i < tokens.size(); ++i) {
-				patches.push_back(context.resolve(
+			vector<string> args;
+			args.push_back(diskChanger.getDriveName());
+			for (unsigned i = firstFileToken; i < tokens.size(); ++i) {
+				args.push_back(context.resolve(
 					tokens[i]->getString()));
 			}
-			diskChanger.insertDisk(
-				context.resolve(tokens[1]->getString()), patches);
+			diskChanger.sendChangeDiskEvent(args);
 		} catch (FileException& e) {
 			throw CommandException(e.getMessage());
 		}
@@ -190,17 +260,20 @@ void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 string DiskCommand::help(const vector<string>& /*tokens*/) const
 {
 	const string& name = diskChanger.getDriveName();
-	return name + " -eject      : remove disk from virtual drive\n" +
-	       name + " -ramdsk     : create a virtual disk in RAM\n" +
-	       name + " <filename> : change the disk file\n";
+	return name + " eject             : remove disk from virtual drive\n" +
+	       name + " ramdsk            : create a virtual disk in RAM\n" +
+	       name + " insert <filename> : change the disk file\n" +
+	       name + " <filename>        : change the disk file\n" +
+	       name + "                   : show which disk image is in drive";
 }
 
 void DiskCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() >= 2) {
 		set<string> extra;
-		extra.insert("-eject");
-		extra.insert("-ramdsk");
+		extra.insert("eject");
+		extra.insert("ramdsk");
+		extra.insert("insert");
 		UserFileContext context(getCommandController());
 		completeFileName(tokens, context, extra);
 	}
