@@ -1,8 +1,7 @@
-// $Id: FileBase.cc,v 1.12 2006/04/25 21:03:23 m9710797 Exp $
+// $Id: FileBase.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
-#include <cassert>
-#include <algorithm>
 #include "FileBase.hh"
+#include <algorithm>
 
 using std::string;
 
@@ -48,15 +47,16 @@ void FileBase::truncate(unsigned newSize)
 		PRT_DEBUG("Default truncate() can't shrink file!");
 		return;
 	}
-	unsigned grow = newSize - oldSize;
+	unsigned remaining = newSize - oldSize;
 	seek(oldSize);
 
 	const unsigned BUF_SIZE = 4096;
 	byte buf[BUF_SIZE];
 	memset(buf, 0, BUF_SIZE);
-	while (grow > 0) {
-		write(buf, std::min(BUF_SIZE, grow));
-		grow -= BUF_SIZE;
+	while (remaining) {
+		unsigned chunkSize = std::min(BUF_SIZE, remaining);
+		write(buf, chunkSize);
+		remaining -= chunkSize;
 	}
 }
 
