@@ -1,11 +1,10 @@
-// $Id: FileContext.cc,v 1.52 2005/09/22 11:27:28 m9710797 Exp $
+// $Id: FileContext.cc 5881 2006-11-15 17:08:34Z m9710797 $
 
 #include "FileContext.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
 #include "GlobalSettings.hh"
 #include "CommandController.hh"
-#include "Interpreter.hh"
 #include "CliComm.hh"
 #include "CommandException.hh"
 #include "StringSetting.hh"
@@ -190,14 +189,14 @@ UserFileContext::UserFileContext(CommandController& commandController,
 			vector<string> dirs;
 			const string& list = commandController.getGlobalSettings().
 				getUserDirSetting().getValue();
-			commandController.getInterpreter().splitList(list, dirs);
+			commandController.splitList(list, dirs);
 			for (vector<string>::const_iterator it = dirs.begin();
 			     it != dirs.end(); ++it) {
 				string path = *it;
 				if (path.empty()) {
 					continue;
 				}
-				if (path[path.length() - 1] != '/') {
+				if (*path.rbegin() != '/') {
 					path += '/';
 				}
 				path = FileOperations::expandTilde(path);
