@@ -1,16 +1,16 @@
-// $Id: FilePool.cc,v 1.12 2005/09/03 21:28:15 m9710797 Exp $
+// $Id: FilePool.cc 6012 2007-01-26 19:34:48Z m9710797 $
 
-#include <fstream>
-#include <cassert>
-#include "sha1.hh"
-#include "SettingsConfig.hh"
+#include "FilePool.hh"
 #include "File.hh"
 #include "FileException.hh"
 #include "FileContext.hh"
 #include "FileOperations.hh"
-#include "FilePool.hh"
 #include "ReadDir.hh"
 #include "Date.hh"
+#include "SettingsConfig.hh"
+#include "sha1.hh"
+#include <fstream>
+#include <cassert>
 
 using std::endl;
 using std::ifstream;
@@ -19,6 +19,7 @@ using std::ofstream;
 using std::pair;
 using std::string;
 using std::vector;
+using std::auto_ptr;
 
 namespace openmsx {
 
@@ -65,14 +66,13 @@ static bool parse(const string& line, string& sha1, time_t& time, string& filena
 	       (time != static_cast<time_t>(-1));
 }
 
-static void calcSha1sum(const string& filename, time_t& time, string& sum)
+static auto_ptr<File> calcSha1sum(const string& filename, string& sum)
 {
-	File file(filename);
-	time = file.getModificationDate();
-	byte* data = file.mmap();
+	auto_ptr<File> file(new File(filename));
 	SHA1 sha1;
-	sha1.update(data, file.getSize());
+	sha1.update(file->mmap(), file->getSize());
 	sum = sha1.hex_digest();
+	return file;
 }
 
 void FilePool::readSha1sums()
@@ -107,25 +107,26 @@ void FilePool::writeSha1sums()
 	}
 }
 
-string FilePool::getFile(const string& sha1sum)
+auto_ptr<File> FilePool::getFile(const string& sha1sum)
 {
-	string filename = getFromPool(sha1sum);
-	if (!filename.empty()) {
-		return filename;
+	auto_ptr<File> result;
+	result = getFromPool(sha1sum);
+	if (result.get()) {
+		return result;
 	}
 
 	for (Directories::const_iterator it = directories.begin();
 	     it != directories.end(); ++it) {
-		string filename = scanDirectory(sha1sum, *it);
-		if (!filename.empty()) {
-			return filename;
+		result = scanDirectory(sha1sum, *it);
+		if (result.get()) {
+			return result;
 		}
 	}
 
-	return string(); // not found
+	return result; // not found
 }
 
-string FilePool::getFromPool(const string& sha1sum)
+auto_ptr<File> FilePool::getFromPool(const string& sha1sum)
 {
 	pair<Pool::iterator, Pool::iterator> bound = pool.equal_range(sha1sum);
 	Pool::iterator it = bound.first;
@@ -133,12 +134,12 @@ string FilePool::getFromPool(const string& sha1sum)
 		time_t& time = it->second.first;
 		const string& filename = it->second.second;
 		try {
-			time_t newTime;
 			string newSum;
-			calcSha1sum(filename, newTime, newSum);
+			auto_ptr<File> file = calcSha1sum(filename, newSum);
+			time_t newTime = file->getModificationDate();
 			if (newSum == sha1sum) {
 				time = newTime;
-				return filename;
+				return file;
 			}
 			// did not match, update db with new sum
 			pool.erase(it++);
@@ -149,61 +150,75 @@ string FilePool::getFromPool(const string& sha1sum)
 			pool.erase(it++);
 		}
 	}
-	return string();
+	return auto_ptr<File>(); // not found
 }
 
-string FilePool::scanDirectory(const string& sha1sum, const string& directory)
+auto_ptr<File> FilePool::scanDirectory(const string& sha1sum, const string& directory)
 {
 	ReadDir dir(directory);
 	while (dirent* d = dir.getEntry()) {
-		string filename = directory + '/' + d->d_name;
-		if (!FileOperations::isRegularFile(filename)) {
-			continue;
+		string file = d->d_name;
+		string path = directory + '/' + file;
+		auto_ptr<File> result;
+		if (FileOperations::isRegularFile(path)) {
+			result = scanFile(sha1sum, path);
+		} else if (FileOperations::isDirectory(path)) {
+			if ((file != ".") && (file != "..")) {
+				result = scanDirectory(sha1sum, path);
+			}
 		}
-		Pool::iterator it = findInDatabase(filename);
-		if (it == pool.end()) {
-			// not in pool
-			try {
-				time_t time;
-				string sum;
-				calcSha1sum(filename, time, sum);
-				pool.insert(make_pair(sum,
-				                 make_pair(time, filename)));
-				if (sum == sha1sum) {
-					return filename;
-				}
-			} catch (FileException& e) {
-				// ignore
-			}
-		} else {
-			// already in pool
-			assert(filename == it->second.second);
-			try {
-				File file(filename);
-				if (file.getModificationDate() == it->second.first) {
-					// db is still up to date
-					if (it->first == sha1sum) {
-						return filename;
-					}
-				} else {
-					// db outdated
-					time_t time;
-					string sum;
-					calcSha1sum(filename, time, sum);
-					pool.erase(it);
-					pool.insert(make_pair(sum,
-					                 make_pair(time, filename)));
-					if (sum == sha1sum) {
-						return filename;
-					}
-				}
-			} catch (FileException& e) {
-				// error reading file, remove from db
-				pool.erase(it);
-			}
+		if (result.get()) {
+			return result;
 		}
 	}
-	return string();
+	return auto_ptr<File>(); // not found
+}
+
+auto_ptr<File> FilePool::scanFile(const string& sha1sum, const string& filename)
+{
+	Pool::iterator it = findInDatabase(filename);
+	if (it == pool.end()) {
+		// not in pool
+		try {
+			string sum;
+			auto_ptr<File> file = calcSha1sum(filename, sum);
+			time_t time = file->getModificationDate();
+			pool.insert(make_pair(sum, make_pair(time, filename)));
+			if (sum == sha1sum) {
+				return file;
+			}
+		} catch (FileException& e) {
+			// ignore
+		}
+	} else {
+		// already in pool
+		assert(filename == it->second.second);
+		try {
+			auto_ptr<File> file(new File(filename));
+			time_t time = file->getModificationDate();
+			if (time == it->second.first) {
+				// db is still up to date
+				if (it->first == sha1sum) {
+					return file;
+				}
+			} else {
+				// db outdated
+				SHA1 sha1;
+				sha1.update(file->mmap(), file->getSize());
+				string sum = sha1.hex_digest();
+				pool.erase(it);
+				pool.insert(make_pair(sum,
+				                      make_pair(time, filename)));
+				if (sum == sha1sum) {
+					return file;
+				}
+			}
+		} catch (FileException& e) {
+			// error reading file, remove from db
+			pool.erase(it);
+		}
+	}
+	return auto_ptr<File>(); // not valid
 }
 
 FilePool::Pool::iterator FilePool::findInDatabase(const string& filename)
