@@ -1,4 +1,4 @@
-// $Id: LocalFile.cc,v 1.41 2006/01/11 18:57:34 m9710797 Exp $
+// $Id: LocalFile.cc 5832 2006-11-02 22:13:09Z manuelbi $
 
 #include "probed_defs.hh"
 #include <unistd.h>
@@ -50,7 +50,7 @@ LocalFile::LocalFile(const string& filename_, File::OpenMode mode)
 		}
 	}
 	if (!file) {
-		throw FileException("Error opening file " + filename);
+		throw FileException("Error opening file \"" + filename + "\"");
 	}
 
 	if (mode == File::PRE_CACHE) {
@@ -150,8 +150,12 @@ void LocalFile::flush()
 
 const string LocalFile::getURL() const
 {
-	static const string prefix("file://");
-	return prefix + filename;
+	// we could use the official file:// prefix, but we
+	// are dropping it, because it is also allowed to omit
+	// it when specifying a file. Maybe we want to drop
+	// support for URLs anyway?
+//	static const string prefix("file://");
+	return /*prefix + */filename;
 }
 
 const string LocalFile::getLocalName()
