@@ -1,8 +1,9 @@
-// $Id: HDImageCLI.cc,v 1.2 2006/05/28 10:35:28 manuelbi Exp $
+// $Id: HDImageCLI.cc 5779 2006-10-14 16:17:32Z m9710797 $
 
 #include "HDImageCLI.hh"
 #include "Reactor.hh"
 #include "CommandController.hh"
+#include "CommandLineParser.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
 
@@ -14,7 +15,7 @@ namespace openmsx {
 HDImageCLI::HDImageCLI(CommandLineParser& commandLineParser)
 	: commandController(commandLineParser.getReactor().getCommandController())
 {
-	commandLineParser.registerOption("-hda", this);
+	commandLineParser.registerOption("-hda", *this);
 	// TODO: offer more options in case you want to specify 2 hard disk images?
 }
 
@@ -22,7 +23,7 @@ bool HDImageCLI::parseOption(const string& option, list<string>& cmdLine)
 {
 	string hd = option.substr(1); // hda
 	string filename = getArgument(option, cmdLine);
-	if (!commandController.hasCommand(hd)) {
+	if (!commandController.hasCommand(hd)) { // TODO WIP
 		throw MSXException("No hard disk named '" + hd + "'.");
 	}
 	TclObject command(commandController.getInterpreter());
