@@ -1,26 +1,32 @@
-// $Id: IDECDROM.cc,v 1.10 2006/07/20 19:54:23 m9710797 Exp $
+// $Id: IDECDROM.cc 5973 2006-12-29 19:11:50Z m9710797 $
 
 #include "IDECDROM.hh"
 #include "MSXMotherBoard.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
-#include "Command.hh"
+#include "RecordedCommand.hh"
 #include "CommandException.hh"
+#include "TclObject.hh"
+#include "MSXCliComm.hh"
 #include <algorithm>
 #include <bitset>
 #include <cassert>
 
 using std::string;
 using std::vector;
+using std::set;
 
 namespace openmsx {
 
-class CDXCommand : public SimpleCommand
+class CDXCommand : public RecordedCommand
 {
 public:
-	CDXCommand(CommandController& commandController, IDECDROM& cd);
-	virtual string execute(const vector<string>& tokens);
+	CDXCommand(MSXCommandController& msxCommandController,
+	           MSXEventDistributor& msxEventDistributor,
+	           Scheduler& scheduler, IDECDROM& cd);
+	virtual void execute(const std::vector<TclObject*>& tokens,
+		TclObject& result, const EmuTime& time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
@@ -29,10 +35,22 @@ private:
 
 
 static const unsigned MAX_CD = 26;
-static std::bitset<MAX_CD> cdInUse;
+typedef std::bitset<MAX_CD> CDInUse;
 
-static string calcName()
+IDECDROM::IDECDROM(MSXMotherBoard& motherBoard_, const XMLElement& /*config*/,
+                   const EmuTime& /*time*/)
+	: AbstractIDEDevice(motherBoard_)
+	, motherBoard(motherBoard_)
 {
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("cdInUse");
+	if (info.counter == 0) {
+		assert(info.stuff == NULL);
+		info.stuff = new CDInUse();
+	}
+	++info.counter;
+	CDInUse& cdInUse = *reinterpret_cast<CDInUse*>(info.stuff);
+
 	unsigned id = 0;
 	while (cdInUse[id]) {
 		++id;
@@ -40,26 +58,39 @@ static string calcName()
 			throw MSXException("Too many CDs");
 		}
 	}
-	return string("cd") + char('a' + id);
-}
-
-IDECDROM::IDECDROM(MSXMotherBoard& motherBoard, const XMLElement& /*config*/,
-                   const EmuTime& time)
-	: AbstractIDEDevice(motherBoard.getEventDistributor(), time)
-	, name(calcName())
-	, cdxCommand(new CDXCommand(motherBoard.getCommandController(), *this))
-{
-	cdInUse[name[2] - 'a'] = true;
+	name = string("cd") + char('a' + id);
+	cdInUse[id] = true;
+	cdxCommand.reset(new CDXCommand(motherBoard.getMSXCommandController(),
+	                                motherBoard.getMSXEventDistributor(),
+	                                motherBoard.getScheduler(), *this));
 
 	senseKey = 0;
 
 	remMedStatNotifEnabled = false;
 	mediaChanged = false;
+
+	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "add");	
 }
 
 IDECDROM::~IDECDROM()
 {
-	cdInUse[name[2] - 'a'] = false;
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("cdInUse");
+	assert(info.counter);
+	assert(info.stuff);
+	CDInUse& cdInUse = *reinterpret_cast<CDInUse*>(info.stuff);
+
+	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "remove");
+	unsigned id = name[2] - 'a';
+	assert(cdInUse[id]);
+	cdInUse[id] = false;
+
+	--info.counter;
+	if (info.counter == 0) {
+		assert(cdInUse.none());
+		delete &cdInUse;
+		info.stuff = NULL;
+	}
 }
 
 bool IDECDROM::isPacketDevice()
@@ -301,6 +332,7 @@ void IDECDROM::eject()
 	file.reset();
 	mediaChanged = true;
 	senseKey = 0x06 << 16; // unit attention (medium changed)
+	motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, "");
 }
 
 void IDECDROM::insert(const string& filename)
@@ -309,54 +341,75 @@ void IDECDROM::insert(const string& filename)
 	file = newFile;
 	mediaChanged = true;
 	senseKey = 0x06 << 16; // unit attention (medium changed)
+	motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, filename);
 }
 
 
 // class CDXCommand
 
-CDXCommand::CDXCommand(CommandController& commandController, IDECDROM& cd_)
-	: SimpleCommand(commandController, cd_.name)
+CDXCommand::CDXCommand(MSXCommandController& msxCommandController,
+                       MSXEventDistributor& msxEventDistributor,
+                       Scheduler& scheduler, IDECDROM& cd_)
+	: RecordedCommand(msxCommandController, msxEventDistributor,
+	                  scheduler, cd_.name)
 	, cd(cd_)
 {
 }
 
-string CDXCommand::execute(const vector<string>& tokens)
+void CDXCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result, 
+				const EmuTime& /*time*/)
 {
-	switch (tokens.size()) {
-	case 1: {
+	if (tokens.size() == 1) {
 		File* file = cd.file.get();
-		return file ? file->getOriginalName() : "";
-	}
-	case 2: {
+		result.addListElement(cd.name + ':');
+		result.addListElement(file ? file->getURL() : "");
+		if (!file) result.addListElement("empty");
+	} else if ( (tokens.size() == 2) && ( 
+		tokens[1]->getString() == "eject" || tokens[1]->getString() == "-eject" )) {
+		cd.eject();
 		// TODO check for locked tray
-		if (tokens[1] == "-eject") {
-			cd.eject();
-			return "";
-		} else {
-			try {
-				UserFileContext context(getCommandController());
-				string filename = context.resolve(tokens[1]);
-				cd.insert(filename);
-				return filename;
-			} catch (FileException& e) {
-				throw CommandException("Can't change cd image: " +
-				                       e.getMessage());
+		if ( tokens[1]->getString() == "-eject" ) { 
+			result.setString(
+			"Warning: use of '-eject' is deprecated, instead use the 'eject' subcommand");
+		}
+	} else if ( (tokens.size() == 2) || ( (tokens.size() == 3) && tokens[1]->getString() == "insert")) {
+		int fileToken = 1;
+		if (tokens[1]->getString() == "insert") {
+			if (tokens.size() > 2) {
+				fileToken = 2;
+			} else {
+				throw CommandException("Missing argument to insert subcommand");
 			}
 		}
-	}
-	default:
-		throw CommandException("Too many arguments.");
+		try {
+			UserFileContext context(getCommandController());
+			string filename = context.resolve(tokens[fileToken]->getString());
+			cd.insert(filename);
+			// return filename; // Note: the diskX command doesn't do this either, so this has not been converted to TclObject style here
+		} catch (FileException& e) {
+			throw CommandException("Can't change cd image: " +
+					e.getMessage());
+		}
+	} else {
+		throw CommandException("Too many or wrong arguments.");
 	}
 }
 
 string CDXCommand::help(const vector<string>& /*tokens*/) const
 {
-	return cd.name + ": change the cd image for this CDROM";
+	return cd.name + "                   : display the cd image for this CDROM drive\n" +
+	       cd.name + " eject             : eject the cd image from this CDROM drive\n" +
+	       cd.name + " insert <filename> : change the cd image for this CDROM drive\n" +
+	       cd.name + " <filename>        : change the cd image for this CDROM drive\n";
 }
 
 void CDXCommand::tabCompletion(vector<string>& tokens) const
 {
-	completeFileName(tokens);
+	set<string> extra;
+	extra.insert("eject");
+	extra.insert("insert");
+	UserFileContext context(getCommandController());
+	completeFileName(tokens, context, extra);
 }
 
 } // namespace openmsx
