@@ -1,42 +1,60 @@
-// $Id: IDEHD.cc,v 1.61 2006/06/21 17:46:51 m9710797 Exp $
+// $Id: IDEHD.cc 5973 2006-12-29 19:11:50Z m9710797 $
 
 #include "IDEHD.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "MSXMotherBoard.hh"
-#include "FileManipulator.hh"
+#include "DiskManipulator.hh"
 #include "XMLElement.hh"
-#include "Command.hh"
-#include "CommandController.hh"
+#include "RecordedCommand.hh"
 #include "CommandException.hh"
 #include "GlobalSettings.hh"
 #include "BooleanSetting.hh"
+#include "MSXCliComm.hh"
 #include <cassert>
 #include <bitset>
 
 using std::string;
 using std::vector;
+using std::set;
 
 namespace openmsx {
 
-class HDCommand : public SimpleCommand
+class HDCommand : public RecordedCommand
 {
 public:
-	HDCommand(CommandController& commandController, IDEHD& hd);
-	virtual string execute(const vector<string>& tokens);
+	HDCommand(MSXCommandController& msxCommandController,
+	          MSXEventDistributor& msxEventDistributor,
+	          Scheduler& scheduler, MSXCliComm& cliComm, IDEHD& hd);
+	virtual void execute(const std::vector<TclObject*>& tokens,
+		TclObject& result, const EmuTime& time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	IDEHD& hd;
+	MSXCliComm& cliComm;
 };
 
 
 static const unsigned MAX_HD = 26;
-static std::bitset<MAX_HD> hdInUse;
+typedef std::bitset<MAX_HD> HDInUse;
 
-static string calcName()
+IDEHD::IDEHD(MSXMotherBoard& motherBoard_, const XMLElement& config,
+             const EmuTime& /*time*/)
+	: AbstractIDEDevice(motherBoard_)
+	, motherBoard(motherBoard_)
+	, diskManipulator(motherBoard.getDiskManipulator())
 {
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("hdInUse");
+	if (info.counter == 0) {
+		assert(info.stuff == NULL);
+		info.stuff = new HDInUse();
+	}
+	++info.counter;
+	HDInUse& hdInUse = *reinterpret_cast<HDInUse*>(info.stuff);
+
 	unsigned id = 0;
 	while (hdInUse[id]) {
 		++id;
@@ -44,17 +62,9 @@ static string calcName()
 			throw MSXException("Too many HDs");
 		}
 	}
-	// for exception safety don't set hdInUse in this routine
-	return string("hd") + char('a' + id);
-}
+	// for exception safety, set hdInUse only at the end
+	name = string("hd") + char('a' + id);
 
-IDEHD::IDEHD(MSXMotherBoard& motherBoard, const XMLElement& config,
-             const EmuTime& time)
-	: AbstractIDEDevice(motherBoard.getEventDistributor(), time)
-	, fileManipulator(motherBoard.getFileManipulator())
-	, name(calcName())
-	, hdCommand(new HDCommand(motherBoard.getCommandController(), *this))
-{
 	string filename = config.getFileContext().resolveCreate(
 		config.getChildData("filename"));
 	try {
@@ -65,14 +75,36 @@ IDEHD::IDEHD(MSXMotherBoard& motherBoard, const XMLElement& config,
 		file->truncate(config.getChildDataAsInt("size") * 1024 * 1024);
 	}
 
-	fileManipulator.registerDrive(*this, name);
-	hdInUse[name[2] - 'a'] = true;
+	diskManipulator.registerDrive(*this);
+	hdInUse[id] = true;
+	hdCommand.reset(new HDCommand(motherBoard.getMSXCommandController(),
+	                              motherBoard.getMSXEventDistributor(),
+	                              motherBoard.getScheduler(),
+	                              motherBoard.getMSXCliComm(), *this));
+
+	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "add");
 }
 
 IDEHD::~IDEHD()
 {
-	hdInUse[name[2] - 'a'] = false;
-	fileManipulator.unregisterDrive(*this, name);
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("hdInUse");
+	assert(info.counter);
+	assert(info.stuff);
+	HDInUse& hdInUse = *reinterpret_cast<HDInUse*>(info.stuff);
+
+	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "remove");
+	unsigned id = name[2] - 'a';
+	assert(hdInUse[id]);
+	hdInUse[id] = false;
+	diskManipulator.unregisterDrive(*this);
+
+	--info.counter;
+	if (info.counter == 0) {
+		assert(hdInUse.none());
+		delete &hdInUse;
+		info.stuff = NULL;
+	}
 }
 
 bool IDEHD::isPacketDevice()
@@ -195,41 +227,60 @@ SectorAccessibleDisk* IDEHD::getSectorAccessibleDisk()
 	return this;
 }
 
+const std::string& IDEHD::getContainerName() const
+{
+	return name;
+}
+
 
 // class HDCommand
 
-HDCommand::HDCommand(CommandController& commandController, IDEHD& hd_)
-	: SimpleCommand(commandController, hd_.name)
+HDCommand::HDCommand(MSXCommandController& msxCommandController,
+                     MSXEventDistributor& msxEventDistributor,
+                     Scheduler& scheduler, MSXCliComm& cliComm_, IDEHD& hd_)
+	: RecordedCommand(msxCommandController, msxEventDistributor,
+	                  scheduler, hd_.name)
 	, hd(hd_)
+	, cliComm(cliComm_)
 {
 }
 
-string HDCommand::execute(const vector<string>& tokens)
+void HDCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result, 
+				const EmuTime& /*time*/)
 {
-	switch (tokens.size()) {
-	case 1:
-		return hd.file->getOriginalName();
-	case 2: {
-		try {
-			CommandController& controller = getCommandController();
-			if (controller.getGlobalSettings().
-			        getPowerSetting().getValue()) {
-				throw CommandException(
+	if (tokens.size() == 1) {
+		result.addListElement(hd.name + ':');
+		result.addListElement(hd.file->getURL());
+		// result.addListElement("readonly"); // TODO: add write protected flag when this is implemented
+	} else if ( (tokens.size() == 2) || ( (tokens.size() == 3) && tokens[1]->getString() == "insert")) {
+		CommandController& controller = getCommandController();
+		if (controller.getGlobalSettings().
+				getPowerSetting().getValue()) {
+			throw CommandException(
 					"Can only change hard disk image when MSX "
 					"is powered down.");
+		}
+		int fileToken = 1;
+		if (tokens[1]->getString() == "insert") {
+			if (tokens.size() > 2) {
+				fileToken = 2;
+			} else {
+				throw CommandException("Missing argument to insert subcommand");
 			}
+		}
+		try {
 			UserFileContext context(controller);
-			string filename = context.resolve(tokens[1]);
+			string filename = context.resolve(tokens[fileToken]->getString());
 			std::auto_ptr<File> newFile(new File(filename));
 			hd.file = newFile;
-			return filename;
+			cliComm.update(CliComm::MEDIA, hd.name, filename);
+			// return filename; // Note: the diskX command doesn't do this either, so this has not been converted to TclObject style here
 		} catch (FileException& e) {
 			throw CommandException("Can't change hard disk image: " +
 			                       e.getMessage());
 		}
-	}
-	default:
-		throw CommandException("Too many arguments.");
+	} else {
+		throw CommandException("Too many or wrong arguments.");
 	}
 }
 
@@ -240,7 +291,12 @@ string HDCommand::help(const vector<string>& /*tokens*/) const
 
 void HDCommand::tabCompletion(vector<string>& tokens) const
 {
-	completeFileName(tokens);
+	set<string> extra;
+	if (tokens.size() < 3) {
+		extra.insert("insert");	
+	}
+	UserFileContext context(getCommandController());
+	completeFileName(tokens, context, extra);
 }
 
 } // namespace openmsx
