@@ -1,16 +1,17 @@
-// $Id: Joystick.cc,v 1.27 2006/02/21 18:06:49 m9710797 Exp $
+// $Id: Joystick.cc 6088 2007-02-16 20:38:53Z m9710797 $
 
 #include "Joystick.hh"
 #include "PluggingController.hh"
-#include "UserInputEventDistributor.hh"
+#include "MSXEventDistributor.hh"
 #include "InputEvents.hh"
+#include "checked_cast.hh"
 #include <cassert>
 
 using std::string;
 
 namespace openmsx {
 
-void Joystick::registerAll(UserInputEventDistributor& eventDistributor,
+void Joystick::registerAll(MSXEventDistributor& eventDistributor,
                            PluggingController& controller)
 {
 	if (!SDL_WasInit(SDL_INIT_JOYSTICK)) {
@@ -24,9 +25,10 @@ void Joystick::registerAll(UserInputEventDistributor& eventDistributor,
 	}
 }
 
-Joystick::Joystick(UserInputEventDistributor& eventDistributor_, unsigned joyNum_)
+Joystick::Joystick(MSXEventDistributor& eventDistributor_, unsigned joyNum_)
 	: eventDistributor(eventDistributor_)
 	, joyNum(joyNum_)
+	, joystick(0)
 {
 	PRT_DEBUG("Creating a Joystick object for joystick " << joyNum);
 	assert(joyNum < (unsigned)SDL_NumJoysticks());
@@ -36,6 +38,9 @@ Joystick::Joystick(UserInputEventDistributor& eventDistributor_, unsigned joyNum
 
 Joystick::~Joystick()
 {
+	if (joystick) {
+		closeJoystick();
+	}
 }
 
 //Pluggable
@@ -64,9 +69,15 @@ void Joystick::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
 }
 
 void Joystick::unplugHelper(const EmuTime& /*time*/)
+{
+	closeJoystick();
+}
+
+void Joystick::closeJoystick()
 {
 	eventDistributor.unregisterEventListener(*this);
 	SDL_JoystickClose(joystick);
+	joystick = 0;
 }
 
 
@@ -82,9 +93,10 @@ void Joystick::write(byte /*value*/, const EmuTime& /*time*/)
 }
 
 //EventListener
-void Joystick::signalEvent(const Event& event)
+void Joystick::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
 {
-	const JoystickEvent* joyEvent = dynamic_cast<const JoystickEvent*>(&event);
+	const JoystickEvent* joyEvent =
+		dynamic_cast<const JoystickEvent*>(event.get());
 	if (!joyEvent) {
 		return;
 	}
@@ -95,11 +107,10 @@ void Joystick::signalEvent(const Event& event)
 		return;
 	}
 
-	switch (event.getType()) {
+	switch (event->getType()) {
 	case OPENMSX_JOY_AXIS_MOTION_EVENT: {
-		assert(dynamic_cast<const JoystickAxisMotionEvent*>(&event));
 		const JoystickAxisMotionEvent& motionEvent =
-			static_cast<const JoystickAxisMotionEvent&>(event);
+			checked_cast<const JoystickAxisMotionEvent&>(*event);
 		short value = motionEvent.getValue();
 		switch (motionEvent.getAxis()) {
 		case JoystickAxisMotionEvent::X_AXIS: // Horizontal
@@ -133,9 +144,8 @@ void Joystick::signalEvent(const Event& event)
 		break;
 	}
 	case OPENMSX_JOY_BUTTON_DOWN_EVENT: {
-		assert(dynamic_cast<const JoystickButtonEvent*>(&event));
 		const JoystickButtonEvent& buttonEvent =
-			static_cast<const JoystickButtonEvent&>(event);
+			checked_cast<const JoystickButtonEvent&>(*event);
 		switch (buttonEvent.getButton() % 2) {
 		case 0:
 			status &= ~JOY_BUTTONA;
@@ -150,9 +160,8 @@ void Joystick::signalEvent(const Event& event)
 		break;
 	}
 	case OPENMSX_JOY_BUTTON_UP_EVENT: {
-		assert(dynamic_cast<const JoystickButtonEvent*>(&event));
 		const JoystickButtonEvent& buttonEvent =
-			static_cast<const JoystickButtonEvent&>(event);
+			checked_cast<const JoystickButtonEvent&>(*event);
 		switch (buttonEvent.getButton() % 2) {
 		case 0:
 			status |= JOY_BUTTONA;
