@@ -1,4 +1,4 @@
-// $Id: Keyboard.cc,v 1.49 2006/02/21 18:06:49 m9710797 Exp $
+// $Id: Keyboard.cc 5779 2006-10-14 16:17:32Z m9710797 $
 
 #include <cstdio>
 #include <cstdlib>
@@ -6,68 +6,71 @@
 #include <cassert>
 #include "Clock.hh"
 #include "Keyboard.hh"
-#include "UserInputEventDistributor.hh"
+#include "MSXEventDistributor.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
-#include "CommandController.hh"
-#include "Command.hh"
+#include "MSXCommandController.hh"
+#include "RecordedCommand.hh"
 #include "CommandException.hh"
 #include "InputEvents.hh"
 #include "Scheduler.hh"
 #include "Unicode.hh"
+#include "checked_cast.hh"
 
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-class KeyMatrixUpCmd : public SimpleCommand
+class KeyMatrixUpCmd : public RecordedCommand
 {
 public:
-	KeyMatrixUpCmd(CommandController& commandController,
-		       Keyboard& keyboard);
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	KeyMatrixUpCmd(MSXCommandController& msxCommandController,
+	               MSXEventDistributor& msxEventDistributor,
+	               Scheduler& scheduler, Keyboard& keyboard);
+	virtual string execute(const vector<string>& tokens, const EmuTime& time);
+	virtual string help(const vector<string>& tokens) const;
 private:
 	Keyboard& keyboard;
 };
 
-class KeyMatrixDownCmd : public SimpleCommand
+class KeyMatrixDownCmd : public RecordedCommand
 {
 public:
-	KeyMatrixDownCmd(CommandController& commandController,
-			 Keyboard& keyboard);
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	KeyMatrixDownCmd(MSXCommandController& msxCommandController,
+	                 MSXEventDistributor& msxEventDistributor,
+	                 Scheduler& scheduler, Keyboard& keyboard);
+	virtual string execute(const vector<string>& tokens, const EmuTime& time);
+	virtual string help(const vector<string>& tokens) const;
 private:
 	Keyboard& keyboard;
 };
 
-class KeyInserter : public SimpleCommand, private Schedulable
+class KeyInserter : public RecordedCommand, private Schedulable
 {
 public:
-	KeyInserter(Scheduler& scheduler,
-		    CommandController& commandController,
-		    Keyboard& keyboard);
+	KeyInserter(MSXCommandController& msxCommandController,
+	            MSXEventDistributor& msxEventDistributor,
+	            Scheduler& scheduler, Keyboard& keyboard);
 	virtual ~KeyInserter();
 
 private:
-	void type(const std::string& str);
+	void type(const string& str);
 	void reschedule(const EmuTime& time);
 
 	// Command
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string execute(const vector<string>& tokens, const EmuTime& time);
+	virtual string help(const vector<string>& tokens) const;
 
 	// Schedulable
 	virtual void executeUntil(const EmuTime& time, int userData);
-	virtual const std::string& schedName() const;
+	virtual const string& schedName() const;
 
 	Keyboard& keyboard;
 	char last;
-	std::string text;
+	string text;
 };
 
 
@@ -123,13 +126,17 @@ void Keyboard::loadKeymapfile(const string& filename)
 	}
 }
 
-Keyboard::Keyboard(Scheduler& scheduler, CommandController& commandController,
-                   UserInputEventDistributor& eventDistributor_, bool keyG)
+Keyboard::Keyboard(Scheduler& scheduler,
+                   MSXCommandController& msxCommandController,
+                   MSXEventDistributor& eventDistributor_, bool keyG)
 	: Schedulable(scheduler)
-	, keyMatrixUpCmd  (new KeyMatrixUpCmd  (commandController, *this))
-	, keyMatrixDownCmd(new KeyMatrixDownCmd(commandController, *this))
-	, keyTypeCmd(new KeyInserter(scheduler, commandController, *this))
 	, eventDistributor(eventDistributor_)
+	, keyMatrixUpCmd  (new KeyMatrixUpCmd  (
+		msxCommandController, eventDistributor, scheduler, *this))
+	, keyMatrixDownCmd(new KeyMatrixDownCmd(
+		msxCommandController, eventDistributor, scheduler, *this))
+	, keyTypeCmd(new KeyInserter(
+		msxCommandController, eventDistributor, scheduler, *this))
 {
 	keyGhosting = keyG;
 	keysChanged = false;
@@ -137,8 +144,8 @@ Keyboard::Keyboard(Scheduler& scheduler, CommandController& commandController,
 	memset(cmdKeyMatrix,  255, sizeof(cmdKeyMatrix));
 	memset(userKeyMatrix, 255, sizeof(userKeyMatrix));
 
-	const XMLElement* config = commandController.getSettingsConfig().
-	                                                 findChild("KeyMap");
+	const XMLElement* config =
+		msxCommandController.getSettingsConfig().findChild("KeyMap");
 	if (config) {
 		string filename = config->getData();
 		loadKeymapfile(config->getFileContext().resolve(filename));
@@ -172,19 +179,19 @@ const byte* Keyboard::getKeys()
 	return keyMatrix;
 }
 
-void Keyboard::signalEvent(const Event& event)
+void Keyboard::signalEvent(shared_ptr<const Event> event,
+                           const EmuTime& /*time*/)
 {
-	EventType type = event.getType();
+	EventType type = event->getType();
 	if ((type == OPENMSX_KEY_DOWN_EVENT) ||
 	    (type == OPENMSX_KEY_UP_EVENT)) {
 		// Ignore possible console on/off events:
 		// we do not rescan the keyboard since this may lead to
 		// an unwanted pressing of <return> in MSX after typing
 		// "set console off" in the console.
-		assert(dynamic_cast<const KeyEvent*>(&event));
+		const KeyEvent& keyEvent = checked_cast<const KeyEvent&>(*event);
 		Keys::KeyCode key = (Keys::KeyCode)
-			((int)((KeyEvent&)event).getKeyCode() &
-			 (int)Keys::K_MASK);
+			((int)keyEvent.getKeyCode() & (int)Keys::K_MASK);
 		if (key != Keys::K_CAPSLOCK) {
 			processKey(type == OPENMSX_KEY_DOWN_EVENT, key);
 		} else {
@@ -204,7 +211,7 @@ void Keyboard::executeUntil(const EmuTime& /*time*/, int /*userData*/)
 	processKey(false, Keys::K_CAPSLOCK);
 }
 
-const std::string& Keyboard::schedName() const
+const string& Keyboard::schedName() const
 {
 	static const string schedName = "Keyboard";
 	return schedName;
@@ -320,14 +327,16 @@ bool Keyboard::commonKeys(char asciiCode1, char asciiCode2)
 
 // class KeyMatrixUpCmd
 
-KeyMatrixUpCmd::KeyMatrixUpCmd(CommandController& commandController,
-                                         Keyboard& keyboard_)
-	: SimpleCommand(commandController, "keymatrixup")
+KeyMatrixUpCmd::KeyMatrixUpCmd(MSXCommandController& msxCommandController,
+                               MSXEventDistributor& msxEventDistributor,
+                               Scheduler& scheduler, Keyboard& keyboard_)
+	: RecordedCommand(msxCommandController, msxEventDistributor,
+	                  scheduler, "keymatrixup")
 	, keyboard(keyboard_)
 {
 }
 
-string KeyMatrixUpCmd::execute(const vector<string>& tokens)
+string KeyMatrixUpCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 {
 	return keyboard.processCmd(tokens, true);
 }
@@ -342,14 +351,16 @@ string KeyMatrixUpCmd::help(const vector<string>& /*tokens*/) const
 
 // class KeyMatrixDownCmd
 
-KeyMatrixDownCmd::KeyMatrixDownCmd(CommandController& commandController,
-                                             Keyboard& keyboard_)
-	: SimpleCommand(commandController, "keymatrixdown")
+KeyMatrixDownCmd::KeyMatrixDownCmd(MSXCommandController& msxCommandController,
+                                   MSXEventDistributor& msxEventDistributor,
+                                   Scheduler& scheduler, Keyboard& keyboard_)
+	: RecordedCommand(msxCommandController, msxEventDistributor,
+	                  scheduler, "keymatrixdown")
 	, keyboard(keyboard_)
 {
 }
 
-string KeyMatrixDownCmd::execute(const vector<string>& tokens)
+string KeyMatrixDownCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 {
 	return keyboard.processCmd(tokens, false);
 }
@@ -364,9 +375,11 @@ string KeyMatrixDownCmd::help(const vector<string>& /*tokens*/) const
 
 // class KeyInserter
 
-KeyInserter::KeyInserter(Scheduler& scheduler,
-		CommandController& commandController, Keyboard& keyboard_)
-	: SimpleCommand(commandController, "type")
+KeyInserter::KeyInserter(MSXCommandController& msxCommandController,
+                         MSXEventDistributor& msxEventDistributor,
+                         Scheduler& scheduler, Keyboard& keyboard_)
+	: RecordedCommand(msxCommandController, msxEventDistributor,
+	                  scheduler, "type")
 	, Schedulable(scheduler)
 	, keyboard(keyboard_), last(-1)
 {
@@ -376,7 +389,7 @@ KeyInserter::~KeyInserter()
 {
 }
 
-string KeyInserter::execute(const vector<string>& tokens)
+string KeyInserter::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 {
 	if (tokens.size() != 2) {
 		throw SyntaxError();
