@@ -1,4 +1,4 @@
-// $Id: main.cc,v 1.98 2006/07/17 18:58:14 m9710797 Exp $
+// $Id: main.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 /*
  *  openmsx - the MSX emulator that aims for perfection
@@ -6,8 +6,9 @@
  */
 
 #include "Reactor.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "CommandLineParser.hh"
+#include "AfterCommand.hh"
 #include "SettingsConfig.hh"
 #include "CliServer.hh"
 #include "Interpreter.hh"
@@ -53,7 +54,11 @@ static int main(int argc, char **argv)
 	try {
 		Thread::setMainThread();
 		Reactor reactor;
-		reactor.getCommandController().getInterpreter().init(argv[0]);
+		reactor.getGlobalCommandController().getInterpreter().init(argv[0]);
+		AfterCommand afterCommand(reactor,
+		                          reactor.getEventDistributor(),
+		                          reactor.getGlobalCommandController());
+
 
 		CommandLineParser parser(reactor);
 		parser.parse(argc, argv);
@@ -67,8 +72,9 @@ static int main(int argc, char **argv)
 					getRenderer().restoreDefault();
 			}
 			if (parseStatus != CommandLineParser::TEST) {
-				CliServer cliServer(reactor.getCommandController(),
-				                    reactor.getEventDistributor());
+				CliServer cliServer(reactor.getGlobalCommandController(),
+				                    reactor.getEventDistributor(),
+				                    reactor.getGlobalCliComm());
 				reactor.run(parser);
 			}
 		}
