@@ -1,4 +1,4 @@
-// $Id: RomDatabase.cc,v 1.1 2006/07/12 17:32:21 m9710797 Exp $
+// $Id: RomDatabase.cc 5895 2006-11-21 18:03:18Z m9710797 $
 
 #include "RomDatabase.hh"
 #include "RomInfo.hh"
@@ -6,7 +6,7 @@
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "File.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "StringOp.hh"
 #include "XMLLoader.hh"
 #include "XMLElement.hh"
@@ -66,7 +66,7 @@ static string parseRemarks(const XMLElement& elem)
 	return result;
 }
 
-static void addEntry(CliComm& cliComm, auto_ptr<RomInfo> romInfo,
+static void addEntry(GlobalCliComm& cliComm, auto_ptr<RomInfo> romInfo,
                      const string& sha1, DBMap& result)
 {
 	assert(romInfo.get());
@@ -78,10 +78,11 @@ static void addEntry(CliComm& cliComm, auto_ptr<RomInfo> romInfo,
 	}
 }
 
-static void parseEntry(CliComm& cliComm,
+static void parseEntry(GlobalCliComm& cliComm,
 	const XMLElement& rom, DBMap& result,
 	const string& title,   const string& year,
 	const string& company, const string& country,
+	bool original,         const string& origType,
 	const string& remark,  const string& type)
 {
 	XMLElement::Children hashTags;
@@ -92,8 +93,8 @@ static void parseEntry(CliComm& cliComm,
 			continue;
 		}
 		auto_ptr<RomInfo> romInfo(new RomInfo(
-			title, year, company, country, remark,
-			RomInfo::nameToRomType(type)));
+			title, year, company, country, original, origType,
+			remark, RomInfo::nameToRomType(type)));
 		string sha1 = (*it2)->getData();
 		addEntry(cliComm, romInfo, sha1, result);
 	}
@@ -109,18 +110,41 @@ static string parseStart(const XMLElement& rom)
 	else return "";
 }
 
-static void parseDB(CliComm& cliComm, const XMLElement& doc, DBMap& result)
+static void parseDump(GlobalCliComm& cliComm,
+	const XMLElement& dump, DBMap& result,
+	const string& title,   const string& year,
+	const string& company, const string& country,
+	const string& remark)
 {
-	const XMLElement::Children& children = doc.getChildren();
-	for (XMLElement::Children::const_iterator it1 = children.begin();
-	     it1 != children.end(); ++it1) {
-		// Parse all <software> tags
-		const XMLElement& soft = **it1;
+	// TODO make <original> tag not required?
+	const XMLElement& originalTag = dump.getChild("original");
+	bool original = originalTag.getAttributeAsBool("value");
+	string origType = originalTag.getData();
 
+	if (const XMLElement* megarom = dump.findChild("megarom")) {
+		parseEntry(cliComm, *megarom, result, title, year,
+			   company, country, original, origType,
+			   remark, megarom->getChildData("type"));
+	} else if (const XMLElement* rom = dump.findChild("rom")) {
+		string type = rom->getChildData("type", "Mirrored");
+		if (type == "Normal") {
+			type += parseStart(*rom);
+		} else if (type == "Mirrored") {
+			type += parseStart(*rom);
+		}
+		parseEntry(cliComm, *rom, result, title, year, company,
+		           country, original, origType, remark, type);
+	}
+}
+
+static void parseSoftware(GlobalCliComm& cliComm, const string& filename,
+                          const XMLElement& soft, DBMap& result)
+{
+	try {
 		const XMLElement* system = soft.findChild("system");
 		if (system && (system->getData() != "MSX")) {
 			// skip non-MSX entries
-			continue;
+			return;
 		}
 
 		// TODO there can be multiple title tags
@@ -132,28 +156,30 @@ static void parseDB(CliComm& cliComm, const XMLElement& doc, DBMap& result)
 
 		XMLElement::Children dumps;
 		soft.getChildren("dump", dumps);
-		for (XMLElement::Children::const_iterator it2 = dumps.begin();
-		     it2 != dumps.end(); ++it2) {
-			const XMLElement& dump = **it2;
-			if (const XMLElement* megarom = dump.findChild("megarom")) {
-				parseEntry(cliComm, *megarom, result, title, year,
-				           company, country, remark,
-				           megarom->getChildData("type"));
-			} else if (const XMLElement* rom = dump.findChild("rom")) {
-				string type = rom->getChildData("type", "Mirrored");
-				if (type == "Normal") {
-					type += parseStart(*rom);
-				} else if (type == "Mirrored") {
-					type += parseStart(*rom);
-				}
-				parseEntry(cliComm, *rom, result, title, year,
-				           company, country, remark, type);
-			}
+		for (XMLElement::Children::const_iterator it = dumps.begin();
+		     it != dumps.end(); ++it) {
+			parseDump(cliComm, **it, result, title, year,
+			          company, country, remark);
 		}
+	} catch (MSXException& e) {
+		string title = soft.getChildData("title", "<missing-title>");
+		cliComm.printWarning("Wrong entry with title '" + title +
+		                     "' in " + filename + ": " + e.getMessage());
 	}
 }
 
-static auto_ptr<XMLElement> openDB(CliComm& cliComm, const string& filename,
+static void parseDB(GlobalCliComm& cliComm, const string& filename,
+                    const XMLElement& doc, DBMap& result)
+{
+	const XMLElement::Children& children = doc.getChildren();
+	for (XMLElement::Children::const_iterator it = children.begin();
+	     it != children.end(); ++it) {
+		// Parse all <software> tags
+		parseSoftware(cliComm, filename, **it, result);
+	}
+}
+
+static auto_ptr<XMLElement> openDB(GlobalCliComm& cliComm, const string& filename,
                                    const string& type)
 {
 	auto_ptr<XMLElement> doc;
@@ -170,7 +196,7 @@ static auto_ptr<XMLElement> openDB(CliComm& cliComm, const string& filename,
 	return doc;
 }
 
-static void initDatabase(CliComm& cliComm)
+static void initDatabase(GlobalCliComm& cliComm)
 {
 	static bool init = false;
 	if (init) return;
@@ -180,12 +206,12 @@ static void initDatabase(CliComm& cliComm)
 	const vector<string>& paths = context.getPaths();
 	for (vector<string>::const_iterator it = paths.begin();
 	     it != paths.end(); ++it) {
+		string filename = *it + "softwaredb.xml";
 		auto_ptr<XMLElement> doc(
-			openDB(cliComm, *it + "softwaredb.xml",
-			       "softwaredb1.dtd"));
+			openDB(cliComm, filename, "softwaredb1.dtd"));
 		if (doc.get()) {
 			DBMap tmp;
-			parseDB(cliComm, *doc, tmp);
+			parseDB(cliComm, filename, *doc, tmp);
 			for (DBMap::const_iterator it = tmp.begin();
 			     it != tmp.end(); ++it) {
 				if (romDBSHA1.find(it->first) == romDBSHA1.end()) {
@@ -205,7 +231,7 @@ static void initDatabase(CliComm& cliComm)
 	}
 }
 
-auto_ptr<RomInfo> RomDatabase::fetchRomInfo(CliComm& cliComm, const Rom& rom)
+auto_ptr<RomInfo> RomDatabase::fetchRomInfo(GlobalCliComm& cliComm, const Rom& rom)
 {
 	// Note: RomInfo is copied only to make ownership managment easier
 
@@ -213,7 +239,8 @@ auto_ptr<RomInfo> RomDatabase::fetchRomInfo(CliComm& cliComm, const Rom& rom)
 
 	if (rom.getSize() == 0) {
 		return auto_ptr<RomInfo>(
-			new RomInfo("", "", "", "", "Empty ROM", ROM_UNKNOWN));
+			new RomInfo("", "", "", "", false, "", "Empty ROM",
+			            ROM_UNKNOWN));
 	}
 
 	const string& sha1sum = rom.getSHA1Sum();
@@ -224,7 +251,8 @@ auto_ptr<RomInfo> RomDatabase::fetchRomInfo(CliComm& cliComm, const Rom& rom)
 	}
 
 	// no match found
-	return auto_ptr<RomInfo>(new RomInfo("", "", "", "", "", ROM_UNKNOWN));
+	return auto_ptr<RomInfo>(new RomInfo("", "", "", "", false, "", "", 
+	                                     ROM_UNKNOWN)); 
 }
 
 } // namespace openmsx
