@@ -1,4 +1,4 @@
-// $Id: RomFSA1FM.cc,v 1.25 2006/05/26 10:45:54 m9710797 Exp $
+// $Id: RomFSA1FM.cc 5862 2006-11-07 11:27:59Z m9710797 $
 
 // FS-A1FM mappers
 //   based on info from:
@@ -36,7 +36,7 @@
 
 #include "RomFSA1FM.hh"
 #include "MSXCPU.hh"
-#include "CPU.hh"
+#include "CacheLine.hh"
 #include "Rom.hh"
 #include "SRAM.hh"
 #include "MSXMotherBoard.hh"
@@ -47,23 +47,30 @@ namespace openmsx {
 
 // 8kb shared sram //
 
-static SRAM* sram = 0;
-static int sramRefCounter = 0;
-
-static void allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
+static SRAM* allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
 {
-	if (sramRefCounter == 0) {
-		sram = new SRAM(motherBoard, config.getId() + " SRAM", 0x2000,
-		                config);
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("FSA1FM-sram");
+	if (info.counter == 0) {
+		assert(info.stuff == NULL);
+		info.stuff = new SRAM(motherBoard, config.getId() + " SRAM", 
+		                      0x2000, config);
 	}
-	++sramRefCounter;
+	++info.counter;
+	return reinterpret_cast<SRAM*>(info.stuff);
 }
 
-static void releaseSRAM()
+static void releaseSRAM(MSXMotherBoard& motherBoard)
 {
-	-- sramRefCounter;
-	if (sramRefCounter == 0) {
+	MSXMotherBoard::SharedStuff& info =
+		motherBoard.getSharedStuff("FSA1FM-sram");
+	assert(info.counter);
+	SRAM* sram = reinterpret_cast<SRAM*>(info.stuff);
+	assert(sram);
+	--info.counter;
+	if (info.counter == 0) {
 		delete sram;
+		info.stuff = NULL;
 	}
 }
 
@@ -73,15 +80,16 @@ static void releaseSRAM()
 RomFSA1FM1::RomFSA1FM1(MSXMotherBoard& motherBoard, const XMLElement& config,
                        const EmuTime& time, std::auto_ptr<Rom> rom)
 	: MSXRom(motherBoard, config, time, rom)
-	, firmwareSwitch(new FirmwareSwitch(motherBoard.getCommandController()))
+	, firmwareSwitch(
+	      new FirmwareSwitch(motherBoard.getCommandController(), config))
 {
-	allocSRAM(motherBoard, config);
+	sram = allocSRAM(motherBoard, config);
 	reset(time);
 }
 
 RomFSA1FM1::~RomFSA1FM1()
 {
-	releaseSRAM();
+	releaseSRAM(getMotherBoard());
 }
 
 void RomFSA1FM1::reset(const EmuTime& /*time*/)
@@ -120,7 +128,7 @@ byte RomFSA1FM1::readMem(word address, const EmuTime& time)
 
 const byte* RomFSA1FM1::getReadCacheLine(word address) const
 {
-	if (address == (0x7FC0 & CPU::CACHE_LINE_HIGH)) {
+	if (address == (0x7FC0 & CacheLine::HIGH)) {
 		// dont't cache IO area
 		return NULL;
 	} else if ((0x4000 <= address) && (address < 0x6000)) {
@@ -151,7 +159,7 @@ void RomFSA1FM1::writeMem(word address, byte value, const EmuTime& /*time*/)
 
 byte* RomFSA1FM1::getWriteCacheLine(word address) const
 {
-	if (address == (0x7FC0 & CPU::CACHE_LINE_HIGH)) {
+	if (address == (0x7FC0 & CacheLine::HIGH)) {
 		// dont't cache IO area
 		return NULL;
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
@@ -169,13 +177,13 @@ RomFSA1FM2::RomFSA1FM2(MSXMotherBoard& motherBoard, const XMLElement& config,
                        const EmuTime& time, std::auto_ptr<Rom> rom)
 	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
-	allocSRAM(motherBoard, config);
+	sram = allocSRAM(motherBoard, config);
 	reset(time);
 }
 
 RomFSA1FM2::~RomFSA1FM2()
 {
-	releaseSRAM();
+	releaseSRAM(getMotherBoard());
 }
 
 void RomFSA1FM2::reset(const EmuTime& /*time*/)
@@ -216,7 +224,7 @@ const byte* RomFSA1FM2::getReadCacheLine(word address) const
 {
 	if (0xC000 <= address) {
 		return unmappedRead;
-	} else if ((0x7FF0 & CPU::CACHE_LINE_HIGH) == address) {
+	} else if ((0x7FF0 & CacheLine::HIGH) == address) {
 		return NULL;
 	} else if (isRam[address >> 13]) {
 		return &(*sram)[address & 0x1FFF];
