@@ -1,4 +1,4 @@
-// $Id: RomFactory.cc,v 1.35 2006/01/25 19:26:32 m9710797 Exp $
+// $Id: RomFactory.cc 5818 2006-10-29 19:08:39Z m9710797 $
 
 #include "RomFactory.hh"
 #include "RomTypes.hh"
@@ -13,6 +13,8 @@
 #include "RomAscii8kB.hh"
 #include "RomAscii8_8.hh"
 #include "RomAscii16kB.hh"
+#include "RomPadial8kB.hh"
+#include "RomPadial16kB.hh"
 #include "RomHydlide2.hh"
 #include "RomRType.hh"
 #include "RomCrossBlaim.hh"
@@ -22,6 +24,7 @@
 #include "RomNational.hh"
 #include "RomMajutsushi.hh"
 #include "RomSynthesizer.hh"
+#include "RomPlayBall.hh"
 #include "RomGameMaster2.hh"
 #include "RomHalnote.hh"
 #include "RomKorean80in1.hh"
@@ -220,6 +223,12 @@ auto_ptr<MSXDevice> RomFactory::create(
 	case ROM_ASCII16:
 		return auto_ptr<MSXDevice>(
 			new RomAscii16kB(motherBoard, config, time, rom));
+	case ROM_PADIAL8:
+		return auto_ptr<MSXDevice>(
+			new RomPadial8kB(motherBoard, config, time, rom));
+	case ROM_PADIAL16:
+		return auto_ptr<MSXDevice>(
+			new RomPadial16kB(motherBoard, config, time, rom));
 	case ROM_R_TYPE:
 		return auto_ptr<MSXDevice>(
 			new RomRType(motherBoard, config, time, rom));
@@ -266,6 +275,9 @@ auto_ptr<MSXDevice> RomFactory::create(
 	case ROM_SYNTHESIZER:
 		return auto_ptr<MSXDevice>(
 			new RomSynthesizer(motherBoard, config, time, rom));
+	case ROM_PLAYBALL:
+		return auto_ptr<MSXDevice>(
+			new RomPlayBall(motherBoard, config, time, rom));
 	case ROM_HALNOTE:
 		return auto_ptr<MSXDevice>(
 			new RomHalnote(motherBoard, config, time, rom));
