@@ -1,4 +1,4 @@
-// $Id: RomInfo.cc,v 1.66 2006/07/12 17:32:21 m9710797 Exp $
+// $Id: RomInfo.cc 5962 2006-12-24 19:34:23Z Vampier $
 
 #include "RomInfo.hh"
 #include "CliComm.hh"
@@ -46,7 +46,10 @@ static const RomTypeMap& getRomTypeMap()
 		romTypeMap["GameMaster2"] = ROM_GAME_MASTER2;
 		romTypeMap["Majutsushi"]  = ROM_MAJUTSUSHI;
 		romTypeMap["Synthesizer"] = ROM_SYNTHESIZER;
+		romTypeMap["PlayBall"]    = ROM_PLAYBALL;
 		romTypeMap["HolyQuran"]   = ROM_HOLY_QURAN;
+		romTypeMap["Padial8"]     = ROM_PADIAL8;
+		romTypeMap["Padial16"]    = ROM_PADIAL16;
 
 		// ROM mapper types used for system ROMs in machines
 		romTypeMap["Panasonic"]   = ROM_PANASONIC;
@@ -85,14 +88,17 @@ static const RomTypeMap& getRomTypeMap()
 
 RomInfo::RomInfo(const string& ntitle,   const string& nyear,
                  const string& ncompany, const string& ncountry,
+                 bool noriginal,         const string& norigType,
                  const string& nremark,  const RomType& nromType)
+	: title(ntitle)
+	, year(nyear)
+	, company(ncompany)
+	, country(ncountry)
+	, original(noriginal)
+	, origType(norigType)
+	, remark(nremark)
+	, romType(nromType)
 {
-	title = ntitle;
-	year = nyear;
-	company = ncompany;
-	country = ncountry;
-	remark = nremark;
-	romType = nromType;
 }
 
 RomType RomInfo::nameToRomType(string name)
@@ -151,7 +157,7 @@ void RomInfo::getAllRomTypes(set<string>& result)
 	}
 }
 
-void RomInfo::print(CliComm& cliComm)
+void RomInfo::print(CliComm& cliComm) const
 {
 	string year(getYear());
 	if (year.empty()) {
@@ -161,11 +167,28 @@ void RomInfo::print(CliComm& cliComm)
 	if (company.empty()) {
 		company = "(info not available)";
 	}
+	string status;
+	if (getOriginal()) {
+		// this is an unmodified original dump
+		status = "Unmodified dump (confirmed by " + getOrigType() + ")";
+	} else {
+		// not original or unknown
+		if (getOrigType() == "broken" ) {
+			status = "Bad dump (game is broken)";
+		} else if (getOrigType() == "translated") {
+			status = "Translated from original";
+		} else if (getOrigType() == "working") {
+			status = "Modified but confirmed working";
+		} else {
+			status = "Unknown";
+		}
+	}
 	string info = "Found this ROM in the database:\n"
 	              "  Title:    " + getTitle() + "\n"
 	              "  Year:     " + year + "\n"
 	              "  Company:  " + company + "\n"
-	              "  Country:  " + country;
+	              "  Country:  " + country + "\n"
+	              "  Status:   " + status;
 	if (!getRemark().empty()) {
 		info += "\n  Remark:   " + getRemark();
 	}
