@@ -1,4 +1,4 @@
-// $Id: RomInfoTopic.cc,v 1.10 2005/11/12 21:53:21 m9710797 Exp $
+// $Id: RomInfoTopic.cc 5818 2006-10-29 19:08:39Z m9710797 $
 
 #include "RomInfoTopic.hh"
 #include "RomInfo.hh"
@@ -16,8 +16,8 @@ namespace openmsx {
 typedef std::map<RomType, string> Description;
 static Description description;
 
-RomInfoTopic::RomInfoTopic(CommandController& commandController)
-	: InfoTopic(commandController, "romtype")
+RomInfoTopic::RomInfoTopic(InfoCommand& openMSXInfoCommand)
+	: InfoTopic(openMSXInfoCommand, "romtype")
 {
 	description[ROM_GENERIC_8KB] = "Generic 8kB";
 	description[ROM_GENERIC_16KB] = "Generic 16kB";
@@ -25,6 +25,8 @@ RomInfoTopic::RomInfoTopic(CommandController& commandController)
 	description[ROM_KONAMI] = "Konami MegaROM";
 	description[ROM_ASCII8] = "ASCII 8kB";
 	description[ROM_ASCII16] = "ASCII 16kB";
+	description[ROM_PADIAL8] = "Padial 8kB";
+	description[ROM_PADIAL16] = "Padial 16kB";
 	description[ROM_R_TYPE] = "R-Type";
 	description[ROM_CROSS_BLAIM] = "Cross Blaim";
 	description[ROM_MSX_AUDIO] = "Panasonic MSX-AUDIO";
@@ -73,6 +75,7 @@ RomInfoTopic::RomInfoTopic(CommandController& commandController)
 	// with DAC
 	description[ROM_MAJUTSUSHI] = "Hai no Majutsushi";
 	description[ROM_SYNTHESIZER] = "Konami's Synthesizer";
+	description[ROM_PLAYBALL] = "Sony's PlayBall";
 }
 
 void RomInfoTopic::execute(const vector<TclObject*>& tokens,
@@ -82,10 +85,7 @@ void RomInfoTopic::execute(const vector<TclObject*>& tokens,
 	case 2: {
 		set<string> romTypes;
 		RomInfo::getAllRomTypes(romTypes);
-		for (set<string>::const_iterator it = romTypes.begin();
-		     it != romTypes.end(); ++it) {
-			result.addListElement(*it);
-		}
+		result.addListElements(romTypes.begin(), romTypes.end());
 		break;
 	}
 	case 3: {
