@@ -1,4 +1,4 @@
-// $Id: RomMajutsushi.cc,v 1.18 2006/02/04 16:59:36 mthuurne Exp $
+// $Id: RomMajutsushi.cc 5785 2006-10-18 15:57:18Z m9710797 $
 
 // Mapper for "Hai no Majutsushi" from Konami.
 // It's a Konami4 mapper with a DAC.
@@ -14,7 +14,7 @@ RomMajutsushi::RomMajutsushi(
 	MSXMotherBoard& motherBoard, const XMLElement& config,
 	const EmuTime& time, std::auto_ptr<Rom> rom)
 	: RomKonami4(motherBoard, config, time, rom)
-	, dac(new DACSound8U(motherBoard.getMixer(), getName(),
+	, dac(new DACSound8U(motherBoard.getMSXMixer(), getName(),
 		"Majutsushi DAC", config, time))
 {
 }
