@@ -1,4 +1,4 @@
-// $Id: RomSynthesizer.cc,v 1.17 2005/06/26 19:17:46 m9710797 Exp $
+// $Id: RomSynthesizer.cc 5785 2006-10-18 15:57:18Z m9710797 $
 
 /* On Sat, 3 Apr 2004, Manuel Pazos wrote:
  *
@@ -21,7 +21,7 @@
 #include "RomSynthesizer.hh"
 #include "MSXMotherBoard.hh"
 #include "DACSound8U.hh"
-#include "CPU.hh"
+#include "CacheLine.hh"
 #include "Rom.hh"
 
 namespace openmsx {
@@ -36,7 +36,7 @@ RomSynthesizer::RomSynthesizer(
 	setRom (2, 1);
 	setBank(3, unmappedRead);
 
-	dac.reset(new DACSound8U(motherBoard.getMixer(), getName(),
+	dac.reset(new DACSound8U(motherBoard.getMSXMixer(), getName(),
 	                         "Konami Synthesizer DAC", config, time));
 
 	reset(time);
@@ -60,7 +60,7 @@ void RomSynthesizer::writeMem(word address, byte value, const EmuTime& time)
 
 byte* RomSynthesizer::getWriteCacheLine(word address) const
 {
-	if ((address & 0xC010 & CPU::CACHE_LINE_HIGH) == 0x4000) {
+	if ((address & 0xC010 & CacheLine::HIGH) == 0x4000) {
 		return NULL;
 	} else {
 		return unmappedWrite;
