@@ -1,4 +1,4 @@
-// $Id: SRAM.cc,v 1.20 2006/02/24 20:14:22 m9710797 Exp $
+// $Id: SRAM.cc 5854 2006-11-06 18:24:21Z m9710797 $
 
 #include "SRAM.hh"
 #include "XMLElement.hh"
@@ -6,18 +6,33 @@
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "MSXMotherBoard.hh"
-#include "CliComm.hh"
+#include "MSXCliComm.hh"
+#include "Alarm.hh"
 
 using std::string;
 
 namespace openmsx {
 
+class SRAMSync : public Alarm
+{
+public:
+	explicit SRAMSync(SRAM& sram);
+	~SRAMSync();
+private:
+	virtual bool alarm();
+	SRAM& sram;
+};
+
+
+// class SRAM
+
 SRAM::SRAM(MSXMotherBoard& motherBoard, const string& name, int size,
            const XMLElement& config_, const char* header_)
 	: ram(motherBoard, name, "sram", size)
 	, config(config_)
 	, header(header_)
-	, cliComm(motherBoard.getCliComm())
+	, cliComm(motherBoard.getMSXCliComm())
+	, sramSync(new SRAMSync(*this))
 {
 	load();
 }
@@ -28,17 +43,25 @@ SRAM::SRAM(MSXMotherBoard& motherBoard, const string& name,
 	: ram(motherBoard, name, description, size)
 	, config(config_)
 	, header(header_)
-	, cliComm(motherBoard.getCliComm())
+	, cliComm(motherBoard.getMSXCliComm())
 {
 	load();
 }
 
 SRAM::~SRAM()
 {
-	cancel(); // cancel pending alarm
 	save();
 }
 
+void SRAM::write(unsigned addr, byte value)
+{
+	if (!sramSync->pending()) {
+		sramSync->schedule(5000000); // sync to disk after 5s
+	}
+	assert(addr < getSize());
+	ram[addr] = value;
+}
+
 void SRAM::load()
 {
 	const string& filename = config.getChildData("sramname");
@@ -84,9 +107,22 @@ void SRAM::save()
 	}
 }
 
-bool SRAM::alarm()
+
+// class SRAMSync
+
+SRAMSync::SRAMSync(SRAM& sram_)
+	: sram(sram_)
 {
-	save();
+}
+
+SRAMSync::~SRAMSync()
+{
+	cancel(); // cancel pending alarm
+}
+
+bool SRAMSync::alarm()
+{
+	sram.save();
 	return false; // don't reschedule
 }
 
