@@ -1,4 +1,4 @@
-// $Id: MidiInReader.cc,v 1.29 2006/05/25 12:37:12 m9710797 Exp $
+// $Id: MidiInReader.cc 5731 2006-10-01 19:54:47Z m9710797 $
 
 #include "MidiInReader.hh"
 #include "MidiInConnector.hh"
@@ -110,7 +110,7 @@ void MidiInReader::signal(const EmuTime& time)
 }
 
 // EventListener
-void MidiInReader::signalEvent(const Event& /*event*/)
+bool MidiInReader::signalEvent(shared_ptr<const Event> /*event*/)
 {
 	if (getConnector()) {
 		signal(scheduler.getCurrentTime());
@@ -118,6 +118,7 @@ void MidiInReader::signalEvent(const Event& /*event*/)
 		ScopedLock l(lock);
 		queue.empty();
 	}
+	return true;
 }
 
 } // namespace openmsx
