@@ -1,10 +1,9 @@
-// $Id: SettingsManager.cc,v 1.30 2006/07/14 20:48:23 m9710797 Exp $
+// $Id: SettingsManager.cc 5830 2006-11-02 19:44:55Z m9710797 $
 
 #include "SettingsManager.hh"
-#include "CommandController.hh"
+#include "GlobalCommandController.hh"
 #include "Command.hh"
 #include "InfoTopic.hh"
-#include "Interpreter.hh"
 #include "TclObject.hh"
 #include "Setting.hh"
 #include "CommandException.hh"
@@ -23,9 +22,8 @@ namespace openmsx {
 class SettingInfo : public InfoTopic
 {
 public:
-	SettingInfo(CommandController& commandController,
-	            SettingsManager& manager);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	SettingInfo(InfoCommand& openMSXInfoCommand, SettingsManager& manager);
+	virtual void execute(const vector<TclObject*>& tokens,
 	                     TclObject& result) const;
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
@@ -38,8 +36,8 @@ class SetCompleter : public CommandCompleter
 public:
 	SetCompleter(CommandController& commandController,
 	             SettingsManager& manager);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
-	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	SettingsManager& manager;
 };
@@ -49,20 +47,20 @@ class SettingCompleter : public CommandCompleter
 public:
 	SettingCompleter(CommandController& commandController,
 	                 SettingsManager& manager,
-	                 const std::string& name);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
-	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+	                 const string& name);
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
 	SettingsManager& manager;
 };
 
 
-SettingsManager::SettingsManager(CommandController& commandController_)
-	: settingInfo   (new SettingInfo     (commandController_, *this))
-	, setCompleter  (new SetCompleter    (commandController_, *this))
-	, incrCompleter (new SettingCompleter(commandController_, *this, "incr"))
-	, unsetCompleter(new SettingCompleter(commandController_, *this, "unset"))
-	, commandController(commandController_)
+SettingsManager::SettingsManager(GlobalCommandController& commandController)
+	: settingInfo   (new SettingInfo(
+	                    commandController.getOpenMSXInfoCommand(), *this))
+	, setCompleter  (new SetCompleter    (commandController, *this))
+	, incrCompleter (new SettingCompleter(commandController, *this, "incr"))
+	, unsetCompleter(new SettingCompleter(commandController, *this, "unset"))
 {
 }
 
@@ -71,73 +69,45 @@ SettingsManager::~SettingsManager()
 	assert(settingsMap.empty());
 }
 
-void SettingsManager::registerSetting(Setting& setting)
+void SettingsManager::registerSetting(Setting& setting, const string& name)
 {
-	const string& name = setting.getName();
 	assert(settingsMap.find(name) == settingsMap.end());
 	settingsMap[name] = &setting;
-
-	commandController.getInterpreter().registerSetting(setting);
 }
 
-void SettingsManager::unregisterSetting(Setting& setting)
+void SettingsManager::unregisterSetting(Setting& /*setting*/, const string& name)
 {
-	commandController.getInterpreter().unregisterSetting(setting);
-
-	const string& name = setting.getName();
 	assert(settingsMap.find(name) != settingsMap.end());
 	settingsMap.erase(name);
 }
 
 // Helper functions for setting commands
 
-template <typename T>
-void SettingsManager::getSettingNames(string& result) const
-{
-	for (SettingsMap::const_iterator it = settingsMap.begin();
-	     it != settingsMap.end(); ++it) {
-		if (dynamic_cast<T*>(it->second)) {
-			result += it->first + '\n';
-		}
-	}
-}
-
-template <typename T>
 void SettingsManager::getSettingNames(set<string>& result) const
 {
 	for (SettingsMap::const_iterator it = settingsMap.begin();
 	     it != settingsMap.end(); ++it) {
-		if (dynamic_cast<T*>(it->second)) {
-			result.insert(it->first);
-		}
+		result.insert(it->first);
 	}
 }
 
-template <typename T>
-T& SettingsManager::getByName(const string& cmd, const string& name) const
+Setting& SettingsManager::getByName(const string& cmd, const string& name) const
 {
 	Setting* setting = getByName(name);
 	if (!setting) {
 		throw CommandException(cmd + ": " + name +
 		                       ": no such setting");
 	}
-	T* typeSetting = dynamic_cast<T*>(setting);
-	if (!typeSetting) {
-		throw CommandException(cmd + ": " + name +
-		                       ": setting has wrong type");
-	}
-	return *typeSetting;
+	return *setting;
 }
 
-Setting* SettingsManager::getByName(const std::string& name) const
+Setting* SettingsManager::getByName(const string& name) const
 {
 	SettingsMap::const_iterator it = settingsMap.find(name);
-	// TODO: The cast is valid because currently all nodes are leaves.
-	//       In the future this will no longer be the case.
 	return it != settingsMap.end() ? it->second : NULL;
 }
 
-string SettingsManager::makeUnique(const std::string& name) const
+string SettingsManager::makeUnique(const string& name) const
 {
 	string result = name;
 	unsigned n = 0;
@@ -188,9 +158,9 @@ void SettingsManager::saveSettings(XMLElement& config) const
 
 // SettingInfo implementation
 
-SettingInfo::SettingInfo(CommandController& commandController,
-                                          SettingsManager& manager_)
-	: InfoTopic(commandController, "setting")
+SettingInfo::SettingInfo(InfoCommand& openMSXInfoCommand,
+                         SettingsManager& manager_)
+	: InfoTopic(openMSXInfoCommand, "setting")
 	, manager(manager_)
 {
 }
@@ -234,7 +204,7 @@ void SettingInfo::tabCompletion(vector<string>& tokens) const
 	switch (tokens.size()) {
 		case 3: { // complete setting name
 			set<string> settings;
-			manager.getSettingNames<Setting>(settings);
+			manager.getSettingNames(settings);
 			completeString(tokens, settings);
 			break;
 		}
@@ -253,8 +223,7 @@ SetCompleter::SetCompleter(CommandController& commandController,
 string SetCompleter::help(const vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		return manager.getByName<Setting>("set", tokens[1])
-		                                          .getDescription();
+		return manager.getByName("set", tokens[1]).getDescription();
 	}
 	return "Set or query the value of a openMSX setting or TCL variable\n"
 	       "  set <setting>          shows current value\n"
@@ -269,7 +238,7 @@ void SetCompleter::tabCompletion(vector<string>& tokens) const
 		case 2: {
 			// complete setting name
 			set<string> settings;
-			manager.getSettingNames<Setting>(settings);
+			manager.getSettingNames(settings);
 			completeString(tokens, settings);
 			break;
 		}
@@ -307,7 +276,7 @@ void SettingCompleter::tabCompletion(vector<string>& tokens) const
 		case 2: {
 			// complete setting name
 			set<string> settings;
-			manager.getSettingNames<Setting>(settings);
+			manager.getSettingNames(settings);
 			completeString(tokens, settings);
 			break;
 		}
