@@ -1,4 +1,4 @@
-// $Id: sha1.cc,v 1.12 2005/06/12 14:12:16 mthuurne Exp $
+// $Id: sha1.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 /*
 Based on:
@@ -20,11 +20,11 @@ A million repetitions of "a"
 34AA973C D4C4DAA4 F61EEB2B DBAD2731 6534016F
 */
 
+#include "sha1.hh"
+#include "build-info.hh"
 #include <cassert>
 #include <cstdio>
 #include <memory.h>
-#include "build-info.hh"
-#include "sha1.hh"
 
 using std::string;
 
@@ -58,7 +58,7 @@ private:
 	}
 
 public:
-	WorkspaceBlock(const byte buffer[64]);
+	explicit WorkspaceBlock(const byte buffer[64]);
 
 	// SHA-1 rounds
 	void r0(uint32 v, uint32& w, uint32 x, uint32 y, uint32& z, int i)
