@@ -1,4 +1,4 @@
-// $Id: AY8910.cc,v 1.39 2006/05/22 21:14:15 m9710797 Exp $
+// $Id: AY8910.cc 6004 2007-01-19 20:48:18Z m9710797 $
 
 /*
  * Emulation of the AY-3-8910
@@ -12,8 +12,8 @@
 
 #include "AY8910.hh"
 #include "AY8910Periphery.hh"
-#include "Mixer.hh"
 #include "MSXMotherBoard.hh"
+#include "MSXCliComm.hh"
 #include "SimpleDebuggable.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
@@ -215,7 +215,9 @@ AY8910::Amplitude::Amplitude(const XMLElement& config)
 	}
 
 	vol[0] = vol[1] = vol[2] = 0;
-	envChan[0] = envChan[1] = envChan[2] = false;
+	envChan[0] = false;
+	envChan[1] = false;
+	envChan[2] = false;
 	envVolume = 0;
 	setMasterVolume(0); // avoid UMR
 }
@@ -281,8 +283,13 @@ inline bool AY8910::Amplitude::anyEnvelope()
 inline AY8910::Envelope::Envelope(Amplitude& amplitude)
 	: amplitude(amplitude)
 {
-	period = count = step = attack = 0;
-	hold = alternate = holding = false;
+	period = 0;
+	count  = 0;
+	step   = 0;
+	attack = 0;
+	hold      = false;
+	alternate = false;
+	holding   = false;
 }
 
 inline void AY8910::Envelope::reset()
@@ -381,11 +388,13 @@ inline void AY8910::Envelope::advance(int duration)
 
 AY8910::AY8910(MSXMotherBoard& motherBoard, AY8910Periphery& periphery_,
                const XMLElement& config, const EmuTime& time)
-	: SoundDevice(motherBoard.getMixer(), "PSG", "PSG")
+	: SoundDevice(motherBoard.getMSXMixer(), "PSG", "PSG")
+	, cliComm(motherBoard.getMSXCliComm())
 	, periphery(periphery_)
 	, amplitude(config)
 	, envelope(amplitude)
 	, debuggable(new AY8910Debuggable(motherBoard, *this))
+	, warningPrinted(false)
 {
 	// make valgrind happy
 	memset(regs, 0, sizeof(regs));
@@ -458,16 +467,23 @@ void AY8910::writeRegister(byte reg, byte value, const EmuTime& time)
 	assert(reg <= 15);
 	if ((reg < AY_PORTA) && (reg == AY_ESHAPE || regs[reg] != value)) {
 		// Update the output buffer before changing the register.
-		getMixer().updateStream(time);
+		updateStream(time);
 	}
-	getMixer().lock();
 	wrtReg(reg, value, time);
-	getMixer().unlock();
 }
 void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 {
-	// Force port directions
+	// Warn/force port directions
 	if (reg == AY_ENABLE) {
+		if ((value & PORT_A_DIRECTION) && !warningPrinted) {
+			warningPrinted = true;
+			cliComm.printWarning(
+				"The running MSX software has set unsafe PSG "
+				"port directions (port A is set as output). "
+				"This is not allowed by the MSX standard. "
+				"Some MSX models (mostly MSX1) can get damaged "
+				"by this.");
+		}
 		// portA -> input
 		// portB -> output
 		value = (value & ~PORT_A_DIRECTION) | PORT_B_DIRECTION;
@@ -484,10 +500,11 @@ void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 	case AY_CFINE:
 	case AY_CCOARSE:
 		tone[reg / 2].setPeriod(
-			regs[reg & ~1] + 256 * (regs[reg | 1] & 0x0F), updateStep );
+			regs[reg & ~1] + 256 * (regs[reg | 1] & 0x0F), updateStep);
 		break;
 	case AY_NOISEPER:
-		noise.setPeriod(value & 0x1F, updateStep);
+		// half the frequency of tone generation
+		noise.setPeriod(value & 0x1F, updateStep * 2);
 		break;
 	case AY_AVOL:
 	case AY_BVOL:
@@ -497,8 +514,10 @@ void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 		break;
 	case AY_EFINE:
 	case AY_ECOARSE:
+		// also half the frequency of tone generation, but handled
+		// inside Envelope::setPeriod()
 		envelope.setPeriod(
-			regs[AY_EFINE] + 256 * regs[AY_ECOARSE], updateStep );
+			regs[AY_EFINE] + 256 * regs[AY_ECOARSE], updateStep);
 		break;
 	case AY_ESHAPE:
 		envelope.setShape(value);
@@ -538,6 +557,10 @@ void AY8910::checkMute()
 void AY8910::setVolume(int volume)
 {
 	amplitude.setMasterVolume(volume);
+	// recalculate volume for all channels
+	for (int i = 0; i < 3; ++i) {
+		amplitude.setChannelVolume(i, regs[AY_AVOL + i]);
+	}
 }
 
 
