@@ -1,10 +1,10 @@
-// $Id: DACSound16S.cc,v 1.22 2005/11/11 22:00:07 m9710797 Exp $
+// $Id: DACSound16S.cc 5785 2006-10-18 15:57:18Z m9710797 $
 
 #include "DACSound16S.hh"
 
 namespace openmsx {
 
-DACSound16S::DACSound16S(Mixer& mixer, const std::string& name,
+DACSound16S::DACSound16S(MSXMixer& mixer, const std::string& name,
                          const std::string& desc, const XMLElement& config,
                          const EmuTime& /*time*/)
 	: SoundDevice(mixer, name, desc)
