@@ -1,10 +1,10 @@
-// $Id: DACSound8U.cc,v 1.9 2005/11/11 22:00:07 m9710797 Exp $
+// $Id: DACSound8U.cc 5785 2006-10-18 15:57:18Z m9710797 $
 
 #include "DACSound8U.hh"
 
 namespace openmsx {
 
-DACSound8U::DACSound8U(Mixer& mixer, const std::string& name,
+DACSound8U::DACSound8U(MSXMixer& mixer, const std::string& name,
                        const std::string& desc, const XMLElement& config,
                        const EmuTime& time)
 	: DACSound16S(mixer, name, desc, config, time)
