@@ -1,4 +1,4 @@
-// $Id: MSXAudio.cc,v 1.42 2006/07/14 20:48:23 m9710797 Exp $
+// $Id: MSXAudio.cc 5904 2006-11-28 21:41:22Z m9710797 $
 
 #include "MSXAudio.hh"
 #include "Y8950.hh"
@@ -16,7 +16,7 @@ namespace openmsx {
 class MusicModulePeriphery : public Y8950Periphery
 {
 public:
-	MusicModulePeriphery(MSXAudio& audio);
+	explicit MusicModulePeriphery(MSXAudio& audio);
 	virtual void write(nibble outputs, nibble values, const EmuTime& time);
 	virtual nibble read(const EmuTime& time);
 private:
@@ -26,7 +26,7 @@ private:
 class PanasonicAudioPeriphery : public Y8950Periphery
 {
 public:
-	PanasonicAudioPeriphery(CommandController& commandController);
+	explicit PanasonicAudioPeriphery(CommandController& commandController);
 	virtual void write(nibble outputs, nibble values, const EmuTime& time);
 	virtual nibble read(const EmuTime& time);
 private:
@@ -51,7 +51,7 @@ MSXAudio::MSXAudio(MSXMotherBoard& motherBoard, const XMLElement& config,
 	string type = StringOp::toLower(config.getChildData("type", "philips"));
 	if (type == "philips") {
 		periphery.reset(new MusicModulePeriphery(*this));
-		dac.reset(new DACSound8U(motherBoard.getMixer(),
+		dac.reset(new DACSound8U(motherBoard.getMSXMixer(),
 		               getName() + " 8-bit DAC", "MSX-AUDIO 8-bit DAC",
 		               config, time));
 	} else if (type == "panasonic") {
@@ -80,18 +80,21 @@ void MSXAudio::reset(const EmuTime& time)
 
 byte MSXAudio::readIO(word port, const EmuTime& time)
 {
-	if (port == 0x0A) {
+	byte result;
+	if ((port & 0xFF) == 0x0A) {
 		// read DAC
-		return 255;
+		result = 255;
 	} else {
-		return (port & 1) ? y8950->readReg(registerLatch, time)
-		                  : y8950->readStatus();
+		result = (port & 1) ? y8950->readReg(registerLatch, time)
+		                    : y8950->readStatus();
 	}
+	//std::cout << "read:  " << (int)(port& 0xff) << " " << (int)result << std::endl;
+	return result;
 }
 
 byte MSXAudio::peekIO(word port, const EmuTime& time) const
 {
-	if (port == 0x0A) {
+	if ((port & 0xFF) == 0x0A) {
 		// read DAC
 		return 255; // read always returns 255
 	} else {
@@ -102,8 +105,8 @@ byte MSXAudio::peekIO(word port, const EmuTime& time) const
 
 void MSXAudio::writeIO(word port, byte value, const EmuTime& time)
 {
-	//PRT_DEBUG("Audio: write "<<hex<<(int)port<<" "<<(int)value<<dec);
-	if (port == 0x0A) {
+	//std::cout << "write: " << (int)(port& 0xff) << " " << (int)value << std::endl;
+	if ((port & 0xFF) == 0x0A) {
 		dacValue = value;
 		if (dacEnabled && dac.get()) {
 			dac->writeDAC(dacValue, time);
@@ -171,8 +174,7 @@ nibble MusicModulePeriphery::read(const EmuTime& /*time*/)
 static string generateName(CommandController& controller)
 {
 	// TODO better name?
-	return controller.getSettingsConfig().getSettingsManager().
-	                                 makeUnique("PanasonicAudioSwitch");
+	return controller.makeUniqueSettingName("PanasonicAudioSwitch");
 }
 
 PanasonicAudioPeriphery::PanasonicAudioPeriphery(
