@@ -1,11 +1,11 @@
-// $Id: MSXMoonSound.cc,v 1.21 2005/12/03 18:58:10 arnoldmnl Exp $
+// $Id: MSXMoonSound.cc 6187 2007-03-21 23:06:21Z m9710797 $
 
 #include "MSXMoonSound.hh"
 #include "YMF262.hh"
 #include "YMF278.hh"
 #include "XMLElement.hh"
 #include "MSXMotherBoard.hh"
-#include "Mixer.hh"
+#include "MSXMixer.hh"
 
 namespace openmsx {
 
@@ -36,9 +36,8 @@ void MSXMoonSound::reset(const EmuTime& time)
 
 byte MSXMoonSound::readIO(word port, const EmuTime& time)
 {
-	getMotherBoard().getMixer().updateStream(time); // TODO optimize
 	byte result;
-	if ((port&0xFF) < 0xC0) {
+	if ((port & 0xFF) < 0xC0) {
 		// WAVE part  0x7E-0x7F
 		switch (port & 0x01) {
 		case 0: // read latch, not supported
@@ -75,7 +74,7 @@ byte MSXMoonSound::readIO(word port, const EmuTime& time)
 byte MSXMoonSound::peekIO(word port, const EmuTime& time) const
 {
 	byte result;
-	if ((port&0xFF) < 0xC0) {
+	if ((port & 0xFF) < 0xC0) {
 		// WAVE part  0x7E-0x7F
 		switch (port & 0x01) {
 		case 0: // read latch, not supported
