@@ -1,4 +1,4 @@
-// $Id: MSXSCCPlusCart.cc,v 1.56 2006/01/25 19:26:32 m9710797 Exp $
+// $Id: MSXSCCPlusCart.cc 5975 2007-01-05 13:44:57Z m9710797 $
 
 // Note: this device is actually called SCC-II. But this would take a lot of
 // renaming, which isn't worth it right now. TODO rename this :)
@@ -11,7 +11,7 @@
 #include "FileException.hh"
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
-#include "CPU.hh"
+#include "CacheLine.hh"
 #include "XMLElement.hh"
 
 namespace openmsx {
@@ -44,11 +44,13 @@ MSXSCCPlusCart::MSXSCCPlusCart(MSXMotherBoard& motherBoard,
 		highRAM = true;
 	} else if (subtype == "mirrored") {
 		mapperMask = 0x07;
-		lowRAM = highRAM = true;
+		lowRAM  = true;
+		highRAM = true;
 	} else {
 		// subtype "expanded", and all others
 		mapperMask = 0x0F;
-		lowRAM = highRAM = true;
+		lowRAM  = true;
+		highRAM = true;
 	}
 
 	// make valgrind happy
@@ -179,7 +181,7 @@ byte* MSXSCCPlusCart::getWriteCacheLine(word start) const
 {
 	//return NULL;
 	if ((0x4000 <= start) && (start < 0xC000)) {
-		if (start == (0xBFFF & CPU::CACHE_LINE_HIGH)) {
+		if (start == (0xBFFF & CacheLine::HIGH)) {
 			return NULL;
 		}
 		int regio = (start >> 13) - 2;
