@@ -1,8 +1,9 @@
-// $Id: MSXTurboRPCM.cc,v 1.29 2006/01/23 19:35:25 m9710797 Exp $
+// $Id: MSXTurboRPCM.cc 5785 2006-10-18 15:57:18Z m9710797 $
 
 #include "MSXTurboRPCM.hh"
 #include "MSXMotherBoard.hh"
 #include "DACSound8U.hh"
+#include "MSXMixer.hh"
 
 namespace openmsx {
 
@@ -10,7 +11,7 @@ MSXTurboRPCM::MSXTurboRPCM(MSXMotherBoard& motherBoard,
                            const XMLElement& config, const EmuTime& time)
 	: MSXDevice(motherBoard, config, time)
 	, AudioInputConnector(motherBoard.getPluggingController(), "pcminput")
-	, mixer(motherBoard.getMixer())
+	, mixer(motherBoard.getMSXMixer())
 	, dac(new DACSound8U(mixer, "PCM", "Turbo-R PCM", config, time))
 	, reference(time)
 	, hwMute(false)
