@@ -1,4 +1,4 @@
-// $Id: SCC.cc,v 1.50 2006/01/23 19:35:25 m9710797 Exp $
+// $Id: SCC.cc 6075 2007-02-12 20:59:56Z m9710797 $
 
 //-----------------------------------------------------------------------------
 //
@@ -96,9 +96,9 @@
 //-----------------------------------------------------------------------------
 
 #include "SCC.hh"
-#include "Mixer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include <cmath>
 
 using std::string;
 
@@ -122,7 +122,7 @@ static string calcDescription(SCC::ChipMode mode)
 
 SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
          const XMLElement& config, const EmuTime& time, ChipMode mode)
-	: SoundDevice(motherBoard.getMixer(), name, calcDescription(mode))
+	: SoundDevice(motherBoard.getMSXMixer(), name, calcDescription(mode))
 	, currentChipMode(mode)
 	, deformTimer(time)
 	, debuggable(new SCCDebuggable(motherBoard, *this))
@@ -137,9 +137,9 @@ SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
 
 	nbSamples = 9999;
 
-	in[0] = in[1] = in[2] = in[3] = in[4] = 0;
-	inHp [0] = inHp [1] = inHp [2] = 0;
-	outHp[0] = outHp[1] = outHp[2] = 0;
+	for (unsigned i = 0; i < sizeof(in)/sizeof(int); ++i) {
+		in[i] = 0;
+	}
 
 	reset(time);
 	registerSound(config);
@@ -176,12 +176,12 @@ void SCC::reset(const EmuTime& /*time*/)
 
 void SCC::setSampleRate(int sampleRate)
 {
-	baseStep = (1ULL << 28) * 3579545ULL / 32 / sampleRate;
+	setResampleRatio(3579545.0 / 32, sampleRate);
 }
 
 void SCC::setVolume(int maxVolume)
 {
-	masterVolume = maxVolume;
+	masterVolume = 2 * maxVolume;
 }
 
 void SCC::setChipMode(ChipMode newMode)
@@ -293,7 +293,7 @@ byte SCC::getFreqVol(byte address)
 
 void SCC::writeMemInterface(byte address, byte value, const EmuTime& time)
 {
-	getMixer().updateStream(time);
+	updateStream(time);
 
 	switch (currentChipMode) {
 	case SCC_Real:
@@ -352,6 +352,10 @@ static inline int adjust(signed char wav, byte vol)
 
 void SCC::writeWave(byte channel, byte address, byte value)
 {
+	// write to channel 5 only possible in SCC+ mode
+	assert(channel < 5);
+	assert((channel != 4) || (currentChipMode == SCC_plusmode));
+
 	if (!readOnly[channel]) {
 		byte pos = address & 0x1F;
 		wave[channel][pos] = value;
@@ -359,7 +363,7 @@ void SCC::writeWave(byte channel, byte address, byte value)
 		if ((currentChipMode != SCC_plusmode) && (channel == 3)) {
 			// copy waveform 4 -> waveform 5
 			wave[4][pos] = wave[3][pos];
-			volAdjustedWave[4][pos] = volAdjustedWave[3][pos];
+			volAdjustedWave[4][pos] = adjust(value, volume[4]);
 		}
 	}
 }
@@ -388,7 +392,7 @@ void SCC::setFreqVol(byte address, byte value)
 			// 4 bit frequency
 			frq >>= 8;
 		}
-		incr[channel] = (frq <= 8) ? 0 : baseStep / (frq + 1);
+		incr[channel] = (frq <= 8) ? 0 : (1 << 28) / (frq + 1);
 		count[channel] &= 0x0F800000; // reset to begin of byte
 		pos[channel] = (unsigned)-1;
 
@@ -457,36 +461,13 @@ void SCC::setDeformReg(byte value, const EmuTime& time)
 	}
 }
 
-int SCC::filter(int input)
+void SCC::generateInput(float* buffer, unsigned num)
 {
-	// filter code copied from BlueMSX
-	in[4] = in[3];
-	in[3] = in[2];
-	in[2] = in[1];
-	in[1] = in[0];
-	in[0] = input;
-
-	inHp[2] = inHp[1];
-	inHp[1] = inHp[0];
-	inHp[0] = (1 * (in[0] + in[4]) + 12 * (in[1] + in[3]) + 45 * in[2]) / 100;
-
-	outHp[2] = outHp[1];
-	outHp[1] = outHp[0];
-	outHp[0] = ( 997 * ( inHp[0] + inHp[2]) +
-	            1994 * (outHp[1] - inHp[1]) -
-	             994 *  outHp[2]) / 1000;
-
-	return outHp[0];
-}
-
-void SCC::updateBuffer(unsigned length, int* buffer,
-     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
-{
-	nbSamples += length;
+	nbSamples += num;
 	if ((deformValue & 0xC0) == 0x00) {
 		// No rotation stuff, this is almost always true. So it makes
 		// sense to have a special optimized routine for this case
-		while (length--) {
+		for (unsigned j = 0; j < num; ++j) {
 			int mixed = 0;
 			byte enable = ch_enable;
 			for (int i = 0; i < 5; ++i, enable >>= 1) {
@@ -500,12 +481,12 @@ void SCC::updateBuffer(unsigned length, int* buffer,
 					mixed += out[i];
 				}
 			}
-			*buffer++ = filter((masterVolume * mixed) / 256);
+			buffer[j] = (masterVolume * mixed) / 256;
 		}
 	} else {
 		// Rotation mode
 		//  TODO not completely correct
-		while (length--) {
+		for (unsigned j = 0; j < num; ++j) {
 			int mixed = 0;
 			byte enable = ch_enable;
 			for (int i = 0; i < 5; ++i, enable >>= 1) {
@@ -523,11 +504,21 @@ void SCC::updateBuffer(unsigned length, int* buffer,
 					mixed += out[i];
 				}
 			}
-			*buffer++ = filter((masterVolume * mixed) / 256);
+			buffer[j] = (masterVolume * mixed) / 256;
 		}
 	}
 }
 
+void SCC::updateBuffer(unsigned length, int* buffer,
+     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
+{
+	float tmpBuf[length];
+	generateOutput(tmpBuf, length);
+	for (unsigned i = 0; i < length; ++i) {
+		buffer[i] = lrintf(tmpBuf[i]);
+	}
+}
+
 void SCC::checkMute()
 {
 	// SCC is muted unless an enabled channel with non-zero volume exists.
