@@ -1,6 +1,7 @@
-// $Id: SoundDevice.cc,v 1.15 2006/07/07 21:40:54 m9710797 Exp $
+// $Id: SoundDevice.cc 5924 2006-12-14 20:50:54Z m9710797 $
 
 #include "SoundDevice.hh"
+#include "MSXMixer.hh"
 #include "XMLElement.hh"
 #include <cassert>
 
@@ -8,7 +9,7 @@ using std::string;
 
 namespace openmsx {
 
-SoundDevice::SoundDevice(Mixer& mixer_, const string& name_,
+SoundDevice::SoundDevice(MSXMixer& mixer_, const string& name_,
                          const string& description_)
 	: mixer(mixer_), name(name_), description(description_)
 	, muteCount(1), muted(true)
@@ -58,18 +59,18 @@ bool SoundDevice::isMuted() const
 }
 
 void SoundDevice::registerSound(const XMLElement& config,
-                                Mixer::ChannelMode mode)
+                                ChannelMode::Mode mode)
 {
 	const XMLElement& soundConfig = config.getChild("sound");
 	short volume = soundConfig.getChildDataAsInt("volume");
-	if (mode != Mixer::STEREO) {
+	if (mode != ChannelMode::STEREO) {
 		string modeStr = soundConfig.getChildData("mode", "mono");
 		if (modeStr == "left") {
-			mode = Mixer::MONO_LEFT;
+			mode = ChannelMode::MONO_LEFT;
 		} else if (modeStr == "right") {
-			mode = Mixer::MONO_RIGHT;
+			mode = ChannelMode::MONO_RIGHT;
 		} else {
-			mode = Mixer::MONO;
+			mode = ChannelMode::MONO;
 		}
 	}
 	mixer.registerSound(*this, volume, mode);
@@ -80,4 +81,9 @@ void SoundDevice::unregisterSound()
 	mixer.unregisterSound(*this);
 }
 
+void SoundDevice::updateStream(const EmuTime& time)
+{
+	mixer.updateStream(time);
+}
+
 } // namespace openmsx
