@@ -1,4 +1,4 @@
-// $Id: WavWriter.cc,v 1.4 2005/09/20 22:32:10 m9710797 Exp $
+// $Id: WavWriter.cc 5989 2007-01-11 15:27:35Z m9710797 $
 
 #include "WavWriter.hh"
 #include "MSXException.hh"
@@ -77,18 +77,25 @@ void WavWriter::write8mono(unsigned char val)
 	write8mono(&val, 1);
 }
 
-void WavWriter::write8mono(unsigned char* val, size_t len)
+void WavWriter::write8mono(unsigned char* val, unsigned len)
 {
 	bytes += fwrite(val, 1, len, wavfp);
 }
 
-void WavWriter::write16stereo(short left, short right)
+void WavWriter::write16stereo(short* buffer, unsigned samples)
 {
-	short buf[2];
-	buf[0] = litEnd_16(left);
-	buf[1] = litEnd_16(right);
-	fwrite(buf, 4, 1, wavfp);
-	bytes += 4;
+	unsigned size = 4 * samples;
+	if (OPENMSX_BIGENDIAN) {
+		short buf[2 * samples];
+		for (unsigned i = 0; i < samples; ++i) {
+			buf[2 * i + 0] = litEnd_16(buffer[2 * i + 0]);
+			buf[2 * i + 1] = litEnd_16(buffer[2 * i + 1]);
+		}
+		fwrite(buf, 1, size, wavfp);
+	} else {
+		fwrite(buffer, 1, size, wavfp);
+	}
+	bytes += size;
 }
 
 void WavWriter::flush()
