@@ -1,4 +1,4 @@
-// $Id: Y8950.cc,v 1.79 2006/07/07 21:40:55 m9710797 Exp $
+// $Id: Y8950.cc 5993 2007-01-12 22:12:12Z m9710797 $
 
 /*
   * Based on:
@@ -9,6 +9,7 @@
 #include "Y8950.hh"
 #include "Y8950Adpcm.hh"
 #include "Y8950KeyboardConnector.hh"
+#include "Y8950Periphery.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "DACSound16S.hh"
@@ -27,8 +28,6 @@ private:
 };
 
 
-static const double PI = 3.14159265358979;
-
 static const double EG_STEP = 0.1875;
 static const double SL_STEP = 3.0;
 static const double TL_STEP = 0.75;
@@ -142,7 +141,7 @@ int Y8950::Slot::lin2db(double d)
 void Y8950::Slot::makeSinTable()
 {
 	for (int i=0; i < PG_WIDTH/4; i++)
-		sintable[i] = lin2db(sin(2.0*PI*i/PG_WIDTH));
+		sintable[i] = lin2db(sin(2.0*M_PI*i/PG_WIDTH));
 	for (int i=0; i < PG_WIDTH/4; i++)
 		sintable[PG_WIDTH/2 - 1 - i] = sintable[i];
 	for (int i=0; i < PG_WIDTH/2; i++)
@@ -160,18 +159,18 @@ void Y8950::makeDphaseNoiseTable(int sampleRate)
 void Y8950::makePmTable()
 {
 	for (int i=0; i<PM_PG_WIDTH; i++)
-		pmtable[0][i] = (int)((double)PM_AMP * pow(2,(double)PM_DEPTH*sin(2.0*PI*i/PM_PG_WIDTH)/1200));
+		pmtable[0][i] = (int)((double)PM_AMP * pow(2,(double)PM_DEPTH*sin(2.0*M_PI*i/PM_PG_WIDTH)/1200));
 	for (int i=0; i < PM_PG_WIDTH; i++)
-		pmtable[1][i] = (int)((double)PM_AMP * pow(2,(double)PM_DEPTH2*sin(2.0*PI*i/PM_PG_WIDTH)/1200));
+		pmtable[1][i] = (int)((double)PM_AMP * pow(2,(double)PM_DEPTH2*sin(2.0*M_PI*i/PM_PG_WIDTH)/1200));
 }
 
 // Table for Amp Modulator
 void Y8950::makeAmTable()
 {
 	for (int i=0; i<AM_PG_WIDTH; i++)
-		amtable[0][i] = (int)((double)AM_DEPTH/2/DB_STEP * (1.0 + sin(2.0*PI*i/PM_PG_WIDTH)));
+		amtable[0][i] = (int)((double)AM_DEPTH/2/DB_STEP * (1.0 + sin(2.0*M_PI*i/PM_PG_WIDTH)));
 	for (int i=0; i<AM_PG_WIDTH; i++)
-		amtable[1][i] = (int)((double)AM_DEPTH2/2/DB_STEP * (1.0 + sin(2.0*PI*i/PM_PG_WIDTH)));
+		amtable[1][i] = (int)((double)AM_DEPTH2/2/DB_STEP * (1.0 + sin(2.0*M_PI*i/PM_PG_WIDTH)));
 }
 
 // Phase increment counter table
@@ -280,8 +279,18 @@ Y8950::Patch::Patch()
 
 void Y8950::Patch::reset()
 {
-	AM = PM = EG = false;
-	KR = ML = KL = TL = FB = AR = DR = SL = RR = 0;
+	AM = false;
+	PM = false;
+	EG = false;
+	KR = 0;
+	ML = 0;
+	KL = 0;
+	TL = 0;
+	FB = 0;
+	AR = 0;
+	DR = 0;
+	SL = 0;
+	RR = 0;
 }
 
 
@@ -450,14 +459,14 @@ void Y8950::Channel::keyOff()
 Y8950::Y8950(MSXMotherBoard& motherBoard, const std::string& name,
              const XMLElement& config, unsigned sampleRam, const EmuTime& time,
              Y8950Periphery& perihery_)
-	: SoundDevice(motherBoard.getMixer(), name, "MSX-AUDIO")
+	: SoundDevice(motherBoard.getMSXMixer(), name, "MSX-AUDIO")
 	, irq(motherBoard.getCPU())
 	, perihery(perihery_)
 	, timer1(motherBoard.getScheduler(), *this)
 	, timer2(motherBoard.getScheduler(), *this)
 	, adpcm(new Y8950Adpcm(*this, motherBoard, name, sampleRam))
 	, connector(new Y8950KeyboardConnector(motherBoard.getPluggingController()))
-	, dac13(new DACSound16S(motherBoard.getMixer(), name + " DAC",
+	, dac13(new DACSound16S(motherBoard.getMSXMixer(), name + " DAC",
 	                        "MSX-AUDIO 13-bit DAC", config, time))
 	, debuggable(new Y8950Debuggable(motherBoard, *this))
 {
@@ -521,11 +530,12 @@ void Y8950::reset(const EmuTime &time)
 	noiseB_dphase = 0;
 
 	// update the output buffer before changing the register
-	getMixer().updateStream(time);
+	updateStream(time);
 	for (int i = 0; i < 0x100; ++i)
 		reg[i] = 0x00;
 
 	reg[0x04] = 0x18;
+	reg[0x19] = 0x0F; // fixes 'Thunderbirds are Go'
 	status = 0x00;
 	statusMask = 0;
 	irq.reset();
@@ -891,10 +901,10 @@ void Y8950::writeReg(byte rg, byte data, const EmuTime& time)
 	//TODO also ADPCM
 	//if (rg>=0x20) {
 		// update the output buffer before changing the register
-		getMixer().updateStream(time);
+		updateStream(time);
 	//}
-	getMixer().lock();
 
+	//std::cout << "write: " << (int)rg << " " << (int)data << std::endl;
 	switch (rg & 0xe0) {
 	case 0x00: {
 		switch (rg) {
@@ -1120,25 +1130,27 @@ void Y8950::writeReg(byte rg, byte data, const EmuTime& time)
 		reg[rg] = data;
 	}
 	}
-	getMixer().unlock();
 	//TODO only for registers that influence sound
 	checkMute();
 }
 
 byte Y8950::readReg(byte rg, const EmuTime &time)
 {
-	getMixer().updateStream(time); // TODO only when necessary
+	updateStream(time); // TODO only when necessary
 
+	byte result;
 	switch (rg) {
 		case 0x0F: // ADPCM-DATA
 		case 0x13: //  ???
 		case 0x14: //  ???
 		case 0x1A: // PCM-DATA
-			return adpcm->readReg(rg);
+			result = adpcm->readReg(rg);
 
 		default:
-			return peekReg(rg, time);
+			result = peekReg(rg, time);
 	}
+	//std::cout << "read: " << (int)rg << " " << (int)result << std::endl;
+	return result;
 }
 
 byte Y8950::peekReg(byte rg, const EmuTime &time) const
@@ -1167,7 +1179,9 @@ byte Y8950::peekReg(byte rg, const EmuTime &time) const
 byte Y8950::readStatus()
 {
 	setStatus(STATUS_BUF_RDY);	// temp hack
-	return peekStatus();
+	byte result = peekStatus();
+	//std::cout << "status: " << (int)result << std::endl;
+	return result;
 }
 
 byte Y8950::peekStatus() const
