@@ -1,4 +1,4 @@
-// $Id: YM2413.cc,v 1.62 2006/01/21 10:51:59 m9710797 Exp $
+// $Id: YM2413.cc 5975 2007-01-05 13:44:57Z m9710797 $
 
 /*
  * Based on:
@@ -7,7 +7,6 @@
  */
 
 #include "YM2413.hh"
-#include "Mixer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include <cmath>
@@ -28,7 +27,6 @@ private:
 
 
 static const int CLOCK_FREQ = 3579545;
-static const double PI = 3.14159265358979323846;
 
 static const double DB_STEP = 48.0 / (1 << 8); //   48 / (1 << DB_BITS)
 static const double EG_STEP = 0.375;
@@ -154,7 +152,7 @@ int YM2413::lin2db(double d)
 void YM2413::makeSinTable()
 {
 	for (int i = 0; i < PG_WIDTH / 4; ++i)
-		fullsintable[i] = lin2db(sin(2.0 * PI * i / PG_WIDTH));
+		fullsintable[i] = lin2db(sin(2.0 * M_PI * i / PG_WIDTH));
 	for (int i = 0; i < PG_WIDTH / 4; ++i)
 		fullsintable[PG_WIDTH / 2 - 1 - i] = fullsintable[i];
 	for (int i = 0; i < PG_WIDTH / 2; ++i)
@@ -168,12 +166,12 @@ void YM2413::makeSinTable()
 
 static inline double saw(double phase)
 {
-  if (phase <= (PI / 2)) {
-    return phase * 2 / PI;
-  } else if (phase <= (PI * 3 / 2)) {
-    return 2.0 - (phase * 2 / PI);
+  if (phase <= (M_PI / 2)) {
+    return phase * 2 / M_PI;
+  } else if (phase <= (M_PI * 3 / 2)) {
+    return 2.0 - (phase * 2 / M_PI);
   } else {
-    return -4.0 + phase * 2 / PI;
+    return -4.0 + phase * 2 / M_PI;
   }
 }
 
@@ -183,7 +181,7 @@ void YM2413::makePmTable()
 	for (int i = 0; i < PM_PG_WIDTH; ++i) {
 		 pmtable[i] = (int)((double)PM_AMP *
 		     pow(2, (double)PM_DEPTH *
-		            saw(2.0 * PI * i / PM_PG_WIDTH) / 1200));
+		            saw(2.0 * M_PI * i / PM_PG_WIDTH) / 1200));
 	}
 }
 
@@ -192,7 +190,7 @@ void YM2413::makeAmTable()
 {
 	for (int i = 0; i < AM_PG_WIDTH; ++i) {
 		amtable[i] = (int)((double)AM_DEPTH / 2 / DB_STEP *
-		                   (1.0 + saw(2.0 * PI * i / PM_PG_WIDTH)));
+		                   (1.0 + saw(2.0 * M_PI * i / PM_PG_WIDTH)));
 	}
 }
 
@@ -593,7 +591,7 @@ static byte inst_data[16 + 3][8] = {
 
 YM2413::YM2413(MSXMotherBoard& motherBoard, const std::string& name,
                const XMLElement& config, const EmuTime& time)
-	: SoundDevice(motherBoard.getMixer(), name, "MSX-MUSIC")
+	: SoundDevice(motherBoard.getMSXMixer(), name, "MSX-MUSIC")
 	, debuggable(new YM2413Debuggable(motherBoard, *this))
 {
 	for (int i = 0; i < 16 + 3; ++i) {
@@ -715,8 +713,9 @@ void YM2413::update_rhythm_mode()
 void YM2413::update_key_status()
 {
 	for (int i = 0; i < 9; ++i) {
-		ch[i].mod.slot_on_flag = ch[i].car.slot_on_flag =
-			reg[0x20 + i] & 0x10;
+		bool slot_on = reg[0x20 + i] & 0x10;
+		ch[i].mod.slot_on_flag = slot_on;
+		ch[i].car.slot_on_flag = slot_on;
 	}
 	if (reg[0x0e] & 0x20) {
 		ch[6].mod.slot_on_flag |= reg[0x0e] & 0x10; // BD1
@@ -1050,8 +1049,7 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 	//PRT_DEBUG("YM2413: write reg "<<(int)regis<<" "<<(int)data);
 
 	// update the output buffer before changing the register
-	getMixer().updateStream(time);
-	getMixer().lock();
+	updateStream(time);
 
 	assert (regis < 0x40);
 	reg[regis] = data;
@@ -1218,7 +1216,6 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 	default:
 		break;
 	}
-	getMixer().unlock();
 	checkMute();
 }
 
