@@ -1,4 +1,4 @@
-// $Id: YMF262.cc,v 1.38 2006/01/21 10:51:59 m9710797 Exp $
+// $Id: YMF262.cc 6187 2007-03-21 23:06:21Z m9710797 $
 
 /*
  *
@@ -7,7 +7,7 @@
  *
  * Copyright (C) 2003 Jarek Burczynski
  *
- * Version 0.1
+ * Version 0.2
  *
  *
  * Revision History:
@@ -40,7 +40,6 @@
  */
 
 #include "YMF262.hh"
-#include "Mixer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include <cmath>
@@ -58,8 +57,6 @@ private:
 };
 
 
-const double PI = 3.14159265358979323846;
-
 const int FREQ_SH   = 16;  // 16.16 fixed point (frequency calculations)
 const int EG_SH     = 16;  // 16.16 fixed point (EG timing)
 const int LFO_SH    = 24;  //  8.24 fixed point (LFO calculations)
@@ -894,7 +891,7 @@ void YMF262::init_tables(void)
 	const double LOG2 = ::log(2);
 	for (int i = 0; i < SIN_LEN; i++) {
 		// non-standard sinus
-		double m = sin(((i * 2) + 1) * PI / SIN_LEN); // checked against the real chip
+		double m = sin(((i * 2) + 1) * M_PI / SIN_LEN); // checked against the real chip
 		// we never reach zero here due to ((i * 2) + 1)
 		double o = (m > 0.0) ?
 			8 * ::log( 1.0 / m) / LOG2:	// convert to 'decibels'
@@ -1223,6 +1220,7 @@ void YMF262::update_channels(YMF262Channel &ch)
 
 byte YMF262::readReg(int r)
 {
+	// no need to call updateStream(time)
 	return peekReg(r);
 }
 
@@ -1242,6 +1240,8 @@ void YMF262::writeReg(int r, byte v, const EmuTime& time)
 }
 void YMF262::writeRegForce(int r, byte v, const EmuTime& time)
 {
+	updateStream(time); // TODO optimize only for regs that directly influence sound
+
 	reg[r] = v;
 
 	byte ch_offset = 0;
@@ -1541,7 +1541,6 @@ void YMF262::writeRegForce(int r, byte v, const EmuTime& time)
 				ch.kcode |= (ch.block_fnum & 0x200) >> 9;	// notesel == 0
 			}
 			if (OPL3_mode) {
-				int chan_no = (r & 0x0F) + ch_offset;
 				// in OPL3 mode
 				//DO THIS:
 				//if this is 1st channel forming up a 4-op channel
@@ -1550,7 +1549,7 @@ void YMF262::writeRegForce(int r, byte v, const EmuTime& time)
 				//OR THIS:
 				//if this is 2nd channel forming up 4-op channel just do nothing
 				//else normal 2 operator function keyon/off
-				switch(chan_no) {
+				switch (chan_no) {
 				case 0: case 1: case 2:
 				case 9: case 10: case 11:
 					if (ch.extended) {
@@ -1819,7 +1818,7 @@ void YMF262::reset(const EmuTime& time)
 
 YMF262::YMF262(MSXMotherBoard& motherBoard, const std::string& name,
                const XMLElement& config, const EmuTime& time)
-	: SoundDevice(motherBoard.getMixer(), name, "MoonSound FM-part")
+	: SoundDevice(motherBoard.getMSXMixer(), name, "MoonSound FM-part")
 	, irq(motherBoard.getCPU())
 	, timer1(motherBoard.getScheduler(), *this)
 	, timer2(motherBoard.getScheduler(), *this)
@@ -1835,7 +1834,7 @@ YMF262::YMF262(MSXMotherBoard& motherBoard, const std::string& name,
 	init_tables();
 
 	reset(time);
-	registerSound(config, Mixer::STEREO);
+	registerSound(config, ChannelMode::STEREO);
 }
 
 YMF262::~YMF262()
@@ -1845,6 +1844,7 @@ YMF262::~YMF262()
 
 byte YMF262::readStatus()
 {
+	// no need to call updateStream(time)
 	byte result = status | status2;
 	status2 = 0;
 	return result;
