@@ -1,4 +1,4 @@
-// $Id: YMF278.cc,v 1.60 2006/01/23 19:35:25 m9710797 Exp $
+// $Id: YMF278.cc 6187 2007-03-21 23:06:21Z m9710797 $
 
 #include "YMF278.hh"
 #include "Rom.hh"
@@ -545,6 +545,7 @@ void YMF278::writeRegOPL4(byte reg, byte data, const EmuTime& time)
 
 void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 {
+	updateStream(time); // TODO optimize only for regs that directly influence sound
 	// Handle slot registers specifically
 	if (reg >= 0x08 && reg <= 0xF7) {
 		int snum = (reg - 8) % 24;
@@ -721,6 +722,7 @@ void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 
 byte YMF278::readReg(byte reg, const EmuTime& time)
 {
+	// no need to call updateStream(time)
 	byte result;
 	switch(reg) {
 		case 2: // 3 upper bits are device ID
@@ -761,6 +763,7 @@ byte YMF278::peekReg(byte reg) const
 
 byte YMF278::readStatus(const EmuTime& time)
 {
+	// no need to call updateStream(time)
 	return peekStatus(time);
 }
 
@@ -774,7 +777,7 @@ byte YMF278::peekStatus(const EmuTime& time) const
 
 YMF278::YMF278(MSXMotherBoard& motherBoard, const std::string& name, int ramSize,
                const XMLElement& config, const EmuTime& time)
-	: SoundDevice(motherBoard.getMixer(), name, "MoonSound wave-part")
+	: SoundDevice(motherBoard.getMSXMixer(), name, "MoonSound wave-part")
 	, rom(new Rom(motherBoard, name + " ROM", "rom", config))
 	, loadTime(time), busyTime(time)
 	, debugRegisters(new DebugRegisters(*this, motherBoard))
@@ -789,7 +792,7 @@ YMF278::YMF278(MSXMotherBoard& motherBoard, const std::string& name, int ramSize
 	endRam = endRom + ramSize;
 
 	reset(time);
-	registerSound(config, Mixer::STEREO);
+	registerSound(config, ChannelMode::STEREO);
 }
 
 YMF278::~YMF278()
