@@ -1,15 +1,16 @@
-// $Id: Display.cc,v 1.59 2006/07/23 23:22:24 mthuurne Exp $
+// $Id: Display.cc 6086 2007-02-14 19:22:06Z m9710797 $
 
 #include "Display.hh"
+#include "Layer.hh"
 #include "VideoSystem.hh"
 #include "EventDistributor.hh"
 #include "FinishFrameEvent.hh"
 #include "FileOperations.hh"
 #include "Alarm.hh"
-#include "CommandConsole.hh"
 #include "Command.hh"
+#include "GlobalCommandController.hh"
 #include "InfoTopic.hh"
-#include "CliComm.hh"
+#include "GlobalCliComm.hh"
 #include "Timer.hh"
 #include "RenderSettings.hh"
 #include "BooleanSetting.hh"
@@ -20,6 +21,7 @@
 #include "MachineConfig.hh"
 #include "VideoSystemChangeListener.hh"
 #include "Version.hh"
+#include "checked_cast.hh"
 #include <algorithm>
 #include <cassert>
 #include <iostream>
@@ -32,7 +34,7 @@ namespace openmsx {
 class RepaintAlarm : public Alarm
 {
 public:
-	RepaintAlarm(EventDistributor& eventDistributor);
+	explicit RepaintAlarm(EventDistributor& eventDistributor);
 	virtual bool alarm();
 private:
 	EventDistributor& eventDistributor;
@@ -42,8 +44,8 @@ class ScreenShotCmd : public SimpleCommand
 {
 public:
 	ScreenShotCmd(CommandController& commandController, Display& display);
-	virtual std::string execute(const std::vector<std::string>& tokens);
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string execute(const vector<string>& tokens);
+	virtual string help(const vector<string>& tokens) const;
 private:
 	Display& display;
 };
@@ -51,10 +53,10 @@ private:
 class FpsInfoTopic : public InfoTopic
 {
 public:
-	FpsInfoTopic(CommandController& commandController, Display& display);
-	virtual void execute(const std::vector<TclObject*>& tokens,
+	FpsInfoTopic(InfoCommand& openMSXInfoCommand, Display& display);
+	virtual void execute(const vector<TclObject*>& tokens,
 			     TclObject& result) const;
-	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual string help(const vector<string>& tokens) const;
 private:
 	Display& display;
 };
@@ -65,14 +67,12 @@ Display::Display(Reactor& reactor_)
 	, alarm(new RepaintAlarm(reactor_.getEventDistributor()))
 	, screenShotCmd(new ScreenShotCmd(
 		reactor_.getCommandController(), *this))
-	, fpsInfo(new FpsInfoTopic(reactor_.getCommandController(), *this))
+	, fpsInfo(new FpsInfoTopic(
+	      reactor_.getGlobalCommandController().getOpenMSXInfoCommand(), *this))
 	, reactor(reactor_)
 	, renderSettings(new RenderSettings(reactor.getCommandController()))
 	, switchInProgress(false)
 {
-	// TODO clean up
-	reactor.getCommandConsole().setDisplay(this);
-
 	frameDurationSum = 0;
 	for (unsigned i = 0; i < NUM_FRAME_DURATIONS; ++i) {
 		frameDurations.addFront(20);
@@ -116,7 +116,6 @@ Display::~Display()
 	resetVideoSystem();
 
 	alarm->cancel();
-	reactor.getCommandConsole().setDisplay(0);
 	assert(listeners.empty());
 }
 
@@ -165,6 +164,17 @@ void Display::detach(VideoSystemChangeListener& listener)
 	listeners.erase(it);
 }
 
+Layer* Display::findLayer(const string& name) const
+{
+	for (Layers::const_iterator it = layers.begin();
+	     it != layers.end(); ++it) {
+		if ((*it)->getName() == name) {
+			return *it;
+		}
+	}
+	return 0;
+}
+
 Display::Layers::iterator Display::baseLayer()
 {
 	// Note: It is possible to cache this, but since the number of layers is
@@ -184,23 +194,25 @@ Display::Layers::iterator Display::baseLayer()
 	}
 }
 
-void Display::signalEvent(const Event& event)
+bool Display::signalEvent(shared_ptr<const Event> event)
 {
-	if (event.getType() == OPENMSX_FINISH_FRAME_EVENT) {
-		const FinishFrameEvent& ffe = static_cast<const FinishFrameEvent&>(event);
+	if (event->getType() == OPENMSX_FINISH_FRAME_EVENT) {
+		const FinishFrameEvent& ffe =
+			checked_cast<const FinishFrameEvent&>(*event);
 		if (!ffe.isSkipped() &&
 		    (renderSettings->getVideoSource().getValue() == ffe.getSource())) {
 			repaint();
 			reactor.getEventDistributor().distributeEvent(
 				new SimpleEvent<OPENMSX_FRAME_DRAWN_EVENT>());
 		}
-	} else if (event.getType() == OPENMSX_DELAYED_REPAINT_EVENT) {
+	} else if (event->getType() == OPENMSX_DELAYED_REPAINT_EVENT) {
 		repaint();
-	} else if (event.getType() == OPENMSX_SWITCH_RENDERER_EVENT) {
+	} else if (event->getType() == OPENMSX_SWITCH_RENDERER_EVENT) {
 		doRendererSwitch();
-	} else if (event.getType() == OPENMSX_MACHINE_LOADED_EVENT) {
+	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
 		setWindowTitle();
 	}
+	return true;
 }
 
 void Display::setWindowTitle()
@@ -252,7 +264,7 @@ void Display::checkRendererSwitch()
 		// causes problems???
 		switchInProgress = true;
 		reactor.getEventDistributor().distributeEvent(
-		      new SimpleEvent<OPENMSX_SWITCH_RENDERER_EVENT>());
+			new SimpleEvent<OPENMSX_SWITCH_RENDERER_EVENT>());
 	}
 }
 
@@ -263,11 +275,12 @@ void Display::doRendererSwitch()
 	try {
 		doRendererSwitch2();
 	} catch (MSXException& e) {
-		reactor.getCliComm().printWarning(
+		reactor.getGlobalCliComm().printWarning(
 			"Couldn't activate renderer " +
 			renderSettings->getRenderer().getValueString() +
 			": " + e.getMessage());
 		renderSettings->getRenderer().setValue(RendererFactory::SDL);
+		currentRenderer = RendererFactory::SDL;
 		doRendererSwitch2();
 	}
 
@@ -385,7 +398,7 @@ bool RepaintAlarm::alarm()
 // ScreenShotCmd
 
 ScreenShotCmd::ScreenShotCmd(CommandController& commandController,
-                                      Display& display_)
+                             Display& display_)
 	: SimpleCommand(commandController, "screenshot")
 	, display(display_)
 {
@@ -431,9 +444,9 @@ string ScreenShotCmd::help(const vector<string>& /*tokens*/) const
 
 // FpsInfoTopic
 
-FpsInfoTopic::FpsInfoTopic(CommandController& commandController,
+FpsInfoTopic::FpsInfoTopic(InfoCommand& openMSXInfoCommand,
                            Display& display_)
-	: InfoTopic(commandController, "fps")
+	: InfoTopic(openMSXInfoCommand, "fps")
 	, display(display_)
 {
 }
