@@ -1,16 +1,12 @@
-// $Id: FBPostProcessor.cc,v 1.4 2006/04/05 08:13:25 m9710797 Exp $
+// $Id: FBPostProcessor.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 #include "FBPostProcessor.hh"
 #include "RenderSettings.hh"
 #include "Scaler.hh"
 #include "ScalerFactory.hh"
-#include "BooleanSetting.hh"
 #include "IntegerSetting.hh"
 #include "FloatSetting.hh"
 #include "VisibleSurface.hh"
-#include "DeinterlacedFrame.hh"
-#include "DoubledFrame.hh"
-#include "RawFrame.hh"
 #include "HostCPU.hh"
 #include "Math.hh"
 #include <algorithm>
@@ -47,7 +43,7 @@ void FBPostProcessor<Pixel>::preCalcNoise(double factor)
 
 template <class Pixel>
 void FBPostProcessor<Pixel>::drawNoiseLine(
-		Pixel* in, Pixel* out, signed char* noise, unsigned width)
+		Pixel* in, Pixel* out, signed char* noise, unsigned long width)
 {
 	#ifdef ASM_X86
 	const HostCPU& cpu = HostCPU::getInstance();
@@ -81,7 +77,7 @@ void FBPostProcessor<Pixel>::drawNoiseLine(
 			"movq     %%mm1,  8(%1, %3);"
 			"movq     %%mm2, 16(%1, %3);"
 			"movq     %%mm3, 24(%1, %3);"
-			"addl     $32, %3;"
+			"add      $32, %3;"
 			"jnz      0b;"
 			"emms;"
 
@@ -125,7 +121,7 @@ void FBPostProcessor<Pixel>::drawNoiseLine(
 			"movq     %%mm1,  8(%1, %3);"
 			"movq     %%mm2, 16(%1, %3);"
 			"movq     %%mm3, 24(%1, %3);"
-			"addl     $32, %3;"
+			"add      $32, %3;"
 			"jnz      0b;"
 			"emms;"
 
@@ -212,8 +208,7 @@ FBPostProcessor<Pixel>::FBPostProcessor(CommandController& commandController,
 	Display& display, VisibleSurface& screen_, VideoSource videoSource,
 	unsigned maxWidth, unsigned height)
 	: PostProcessor(
-		commandController, display, screen_, videoSource, maxWidth, height
-		)
+		commandController, display, screen_, videoSource, maxWidth, height)
 	, noiseShift(screen.getHeight())
 	, pixelOps(screen.getFormat())
 {
@@ -236,32 +231,7 @@ FBPostProcessor<Pixel>::~FBPostProcessor()
 template <class Pixel>
 void FBPostProcessor<Pixel>::paint()
 {
-	// TODO: When frames are being skipped or if (de)interlace was just
-	//       turned on, it's not guaranteed that prevFrame is a
-	//       different field from currFrame.
-	//       Or in the case of frame skip, it might be the right field,
-	//       but from several frames ago.
-	FrameSource* frame;
-	FrameSource::FieldType field = currFrame->getField();
-	if (field != FrameSource::FIELD_NONINTERLACED) {
-		if (renderSettings.getDeinterlace().getValue()) {
-			// deinterlaced
-			if (field == FrameSource::FIELD_ODD) {
-				deinterlacedFrame->init(prevFrame, currFrame);
-			} else {
-				deinterlacedFrame->init(currFrame, prevFrame);
-			}
-			frame = deinterlacedFrame;
-		} else {
-			// interlaced
-			interlacedFrame->init(currFrame,
-				(field == FrameSource::FIELD_ODD) ? 1 : 0);
-			frame = interlacedFrame;
-		}
-	} else {
-		// non interlaced
-		frame = currFrame;
-	}
+	if (!paintFrame) return;
 
 	// New scaler algorithm selected?
 	RenderSettings::ScaleAlgorithm algo =
@@ -275,7 +245,7 @@ void FBPostProcessor<Pixel>::paint()
 	}
 
 	// Scale image.
-	const unsigned srcHeight = frame->getHeight();
+	const unsigned srcHeight = paintFrame->getHeight();
 	const unsigned dstHeight = screen.getHeight();
 
 	unsigned g = Math::gcd(srcHeight, dstHeight);
@@ -292,11 +262,11 @@ void FBPostProcessor<Pixel>::paint()
 		assert(srcStartY < srcHeight);
 
 		// get region with equal lineWidth
-		unsigned lineWidth = getLineWidth(frame, srcStartY, srcStep);
+		unsigned lineWidth = getLineWidth(paintFrame, srcStartY, srcStep);
 		unsigned srcEndY = srcStartY + srcStep;
 		unsigned dstEndY = dstStartY + dstStep;
 		while ((srcEndY < srcHeight) && (dstEndY < dstHeight) &&
-		       (getLineWidth(frame, srcEndY, srcStep) == lineWidth)) {
+		       (getLineWidth(paintFrame, srcEndY, srcStep) == lineWidth)) {
 			srcEndY += srcStep;
 			dstEndY += dstStep;
 		}
@@ -305,9 +275,9 @@ void FBPostProcessor<Pixel>::paint()
 		//fprintf(stderr, "post processing lines %d-%d: %d\n",
 		//	srcStartY, srcEndY, lineWidth );
 		currScaler->scaleImage(
-			*frame, srcStartY, srcEndY, lineWidth, // source
+			*paintFrame, srcStartY, srcEndY, lineWidth, // source
 			screen, dstStartY, dstEndY); // dest
-		frame->freeLineBuffers();
+		paintFrame->freeLineBuffers();
 
 		// next region
 		srcStartY = srcEndY;
@@ -323,13 +293,14 @@ void FBPostProcessor<Pixel>::paint()
 
 template <class Pixel>
 RawFrame* FBPostProcessor<Pixel>::rotateFrames(
-	RawFrame* finishedFrame, FrameSource::FieldType field)
+	RawFrame* finishedFrame, FrameSource::FieldType field,
+	const EmuTime& time)
 {
 	for (unsigned y = 0; y < screen.getHeight(); ++y) {
 		noiseShift[y] = rand() & (NOISE_SHIFT - 1) & ~7;
 	}
 
-	return PostProcessor::rotateFrames(finishedFrame, field);
+	return PostProcessor::rotateFrames(finishedFrame, field, time);
 }
 
 
