@@ -1,4 +1,4 @@
-// $Id: GLRasterizer.cc,v 1.42 2006/04/05 18:43:14 m9710797 Exp $
+// $Id: GLRasterizer.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 /*
 TODO:
@@ -24,7 +24,6 @@ TODO:
 #include "OutputSurface.hh"
 #include "Renderer.hh"
 #include "RenderSettings.hh"
-#include "BooleanSetting.hh"
 #include "FloatSetting.hh"
 #include "IntegerSetting.hh"
 #include "StringSetting.hh"
@@ -196,8 +195,8 @@ inline static int translateX(int absoluteX)
 inline void GLRasterizer::renderBitmapLine(byte mode, int vramLine)
 {
 	if (lineValidInMode[vramLine] != mode) {
-		const byte *vramPtr =
-			vram.bitmapCacheWindow.readArea(vramLine << 7);
+		const byte* vramPtr =
+			vram.bitmapCacheWindow.getReadArea(vramLine * 128, 128);
 		bitmapConverter->convertLine(lineBuffer, vramPtr);
 		bitmapTexture->update(vramLine, lineBuffer, lineWidth);
 		lineValidInMode[vramLine] = mode;
@@ -223,16 +222,16 @@ inline void GLRasterizer::renderBitmapLines(byte line, int count)
 
 inline void GLRasterizer::renderPlanarBitmapLine(byte mode, int vramLine)
 {
-	if ((lineValidInMode[vramLine      ] != mode) ||
+	if ((lineValidInMode[vramLine |   0] != mode) ||
 	    (lineValidInMode[vramLine | 512] != mode)) {
-		int addr0 = vramLine << 7;
-		int addr1 = addr0 | 0x10000;
-		const byte* vramPtr0 = vram.bitmapCacheWindow.readArea(addr0);
-		const byte* vramPtr1 = vram.bitmapCacheWindow.readArea(addr1);
+		const byte* vramPtr0;
+		const byte* vramPtr1;
+		vram.bitmapCacheWindow.getReadAreaPlanar(
+			vramLine * 256, 256, vramPtr0, vramPtr1);
 		bitmapConverter->convertLinePlanar(lineBuffer, vramPtr0, vramPtr1);
 		bitmapTexture->update(vramLine, lineBuffer, lineWidth);
-		lineValidInMode[vramLine] =
-			lineValidInMode[vramLine | 512] = mode;
+		lineValidInMode[vramLine |   0] = mode;
+		lineValidInMode[vramLine | 512] = mode;
 	}
 }
 
@@ -255,8 +254,7 @@ inline void GLRasterizer::renderPlanarBitmapLines(byte line, int count)
 
 GLRasterizer::GLRasterizer(
 		CommandController& commandController,
-		VDP& vdp_, Display& display, OutputSurface& screen_
-		)
+		VDP& vdp_, Display& display, OutputSurface& screen_)
 	: VideoLayer(VIDEO_MSX, commandController, display)
 	, renderSettings(display.getRenderSettings())
 	, vdp(vdp_), vram(vdp.getVRAM())
@@ -452,7 +450,7 @@ void GLRasterizer::resetPalette()
 	}
 }
 
-void GLRasterizer::frameStart()
+void GLRasterizer::frameStart(const EmuTime& /*time*/)
 {
 	// Calculate line to render at top of screen.
 	// Make sure the display area is centered.
@@ -822,8 +820,7 @@ void GLRasterizer::renderText1(
 				byte charPixels[8 * 8];
 				characterConverter->convertMonoBlock(
 					charPixels,
-					vram.patternTable.readArea((-1 << 11) | (charcode * 8))
-					);
+					vram.patternTable.getReadArea(charcode * 8, 8));
 				GLBindMonoBlock(textureId, charPixels);
 			}
 			// Plot current character.
@@ -881,7 +878,7 @@ void GLRasterizer::renderText2(
 				byte charPixels[8 * 8];
 				characterConverter->convertMonoBlock(
 					charPixels,
-					vram.patternTable.readArea((-1 << 11) | (charcode * 8)));
+					vram.patternTable.getReadArea(charcode * 8, 8));
 				GLBindMonoBlock(textureId, charPixels);
 			}
 			// Plot current character.
@@ -943,7 +940,7 @@ void GLRasterizer::renderGraphic1Row(
 			byte charPixels[8 * 8];
 			characterConverter->convertMonoBlock(
 				charPixels,
-				vram.patternTable.readArea((-1 << 11) | (charNr * 8)));
+				vram.patternTable.getReadArea(charNr * 8, 8));
 			GLBindMonoBlock(textureId, charPixels);
 		}
 		int colour = vram.colourTable.readNP((-1 << 6) | (charNr / 8));
@@ -1024,11 +1021,11 @@ void GLRasterizer::renderGraphic2Row(
 
 		if (!valid) {
 			Pixel charPixels[8 * 8];
-			int index = (-1 << 13) | (charNr * 8);
+			int char8 = charNr * 8;
 			characterConverter->convertColourBlock(
 				charPixels,
-				vram.patternTable.readArea(index),
-				vram.colourTable.readArea(index));
+				vram.patternTable.getReadArea(char8, 8),
+				vram.colourTable. getReadArea(char8, 8));
 			GLBindColourBlock(textureId, charPixels);
 		}
 		GLDrawColourBlock(textureId, x, screenLine);
@@ -1056,8 +1053,8 @@ void GLRasterizer::renderMultiColour(
 	int rowEnd = (vramLine + count + 3) / 4;
 	screenLine -= (vramLine & 3) * 2;
 	for (int row = rowStart; row < rowEnd; row++) {
-		const byte *namePtr = vram.nameTable.readArea(
-			(-1 << 10) | (row / 2) * 32 );
+		const byte* namePtr =
+			vram.nameTable.getReadArea((row / 2) * 32, 32);
 		for (int col = colStart; col < colEnd; col++) {
 			int charcode = namePtr[col];
 			int colour = vram.patternTable.readNP(
@@ -1294,6 +1291,11 @@ void GLRasterizer::drawSprites(
 	glDisable(GL_TEXTURE_2D);
 }
 
+bool GLRasterizer::isRecording() const
+{
+	return false;
+}
+
 void GLRasterizer::update(const Setting& setting)
 {
 	VideoLayer::update(setting);
