@@ -1,9 +1,11 @@
-// $Id: GLScalerFactory.cc,v 1.2 2006/03/26 00:41:57 mthuurne Exp $
+// $Id: GLScalerFactory.cc 5577 2006-08-12 16:48:02Z m9710797 $
 
 #include "GLScalerFactory.hh"
 #include "GLSimpleScaler.hh"
 #include "GLScaleNxScaler.hh"
 #include "GLTVScaler.hh"
+#include "GLHQScaler.hh"
+#include "GLHQLiteScaler.hh"
 #include "RenderSettings.hh"
 #include "EnumSetting.hh"
 #include <cassert>
@@ -20,8 +22,6 @@ auto_ptr<GLScaler> GLScalerFactory::createScaler(
 	case RenderSettings::SCALER_SIMPLE:
 	// TODO: Until we have GL versions of these, map them to "simple".
 	case RenderSettings::SCALER_SAI:
-	case RenderSettings::SCALER_HQ:
-	case RenderSettings::SCALER_HQLITE:
 	case RenderSettings::SCALER_RGBTRIPLET:
 		return auto_ptr<GLScaler>(
 			new GLSimpleScaler(renderSettings)
@@ -30,6 +30,10 @@ auto_ptr<GLScaler> GLScalerFactory::createScaler(
 		return auto_ptr<GLScaler>(new GLScaleNxScaler());
 	case RenderSettings::SCALER_TV:
 		return auto_ptr<GLScaler>(new GLTVScaler());
+	case RenderSettings::SCALER_HQ:
+		return auto_ptr<GLScaler>(new GLHQScaler());
+	case RenderSettings::SCALER_HQLITE:
+		return auto_ptr<GLScaler>(new GLHQLiteScaler());
 	default:
 		assert(false);
 	}
