@@ -1,4 +1,4 @@
-// $Id: RenderSettings.cc,v 1.76 2006/06/29 17:39:19 m9710797 Exp $
+// $Id: RenderSettings.cc 5740 2006-10-03 16:51:29Z m9710797 $
 
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
@@ -13,7 +13,7 @@ namespace openmsx {
 class ColorMatrixChecker : public SettingChecker<StringSettingPolicy>
 {
 public:
-	ColorMatrixChecker(RenderSettings& renderSettings);
+	explicit ColorMatrixChecker(RenderSettings& renderSettings);
 	virtual void check(SettingImpl<StringSettingPolicy>& setting,
 	                   std::string& newValue);
 private:
