@@ -1,4 +1,4 @@
-// $Id: SDLGLVisibleSurface.cc,v 1.8 2006/06/29 20:21:13 m9710797 Exp $
+// $Id: SDLGLVisibleSurface.cc 5901 2006-11-25 22:45:00Z m9710797 $
 
 #include "SDLGLVisibleSurface.hh"
 #include "ScreenShotSaver.hh"
@@ -7,17 +7,12 @@
 #include "InitException.hh"
 #include "IconLayer.hh"
 #include "build-info.hh"
+#include "Math.hh"
+#include <vector>
 #include <cstdlib>
 
 namespace openmsx {
 
-static unsigned roundUpPow2(unsigned x)
-{
-	unsigned result = 1;
-	while (result < x) result <<= 1;
-	return result;
-}
-
 SDLGLVisibleSurface::SDLGLVisibleSurface(
 		unsigned width, unsigned height, bool fullscreen,
 		FrameBuffer frameBuffer_)
@@ -31,6 +26,19 @@ SDLGLVisibleSurface::SDLGLVisibleSurface(
 	//flags |= SDL_RESIZABLE;
 	createSurface(width, height, flags);
 
+	// From the glew documentation:
+	//   GLEW obtains information on the supported extensions from the
+	//   graphics driver. Experimental or pre-release drivers, however,
+	//   might not report every available extension through the standard
+	//   mechanism, in which case GLEW will report it unsupported. To
+	//   circumvent this situation, the glewExperimental global switch can
+	//   be turned on by setting it to GL_TRUE before calling glewInit(),
+	//   which ensures that all extensions with valid entry points will be
+	//   exposed.
+	// The 'glewinfo' utility also sets this flag before reporting results,
+	// so I believe it would cause less confusion to do the same here.
+	glewExperimental = GL_TRUE;
+
 	// Initialise GLEW library.
 	// This must happen after GL itself is initialised, which is done by
 	// the SDL_SetVideoMode() call in createSurface().
@@ -99,8 +107,8 @@ SDLGLVisibleSurface::SDLGLVisibleSurface(
 		data = 0;
 	} else {
 		// TODO 64 byte aligned (see RawFrame)
-		unsigned texW = roundUpPow2(width);
-		unsigned texH = roundUpPow2(height);
+		unsigned texW = Math::powerOfTwo(width);
+		unsigned texH = Math::powerOfTwo(height);
 		data = (char*)malloc(format.BytesPerPixel * texW * texH);
 		pitch = width * format.BytesPerPixel;
 
@@ -136,10 +144,11 @@ unsigned SDLGLVisibleSurface::mapRGB(double dr, double dg, double db)
 		int r = static_cast<int>(dr * 255.0);
 		int g = static_cast<int>(dg * 255.0);
 		int b = static_cast<int>(db * 255.0);
+		// convert to BGRA (is faster then RGBA for texture upload)
 		if (OPENMSX_BIGENDIAN) {
-			return (r << 24) | (g << 16) | (b <<  8) | 0x000000FF;
+			return (b << 24) | (g << 16) | (r <<  8) | 0x000000FF;
 		} else {
-			return (r <<  0) | (g <<  8) | (b << 16) | 0xFF000000;
+			return (b <<  0) | (g <<  8) | (r << 16) | 0xFF000000;
 		}
 	} else {
 		return OutputSurface::mapRGB(dr, dg, db);
@@ -180,24 +189,13 @@ void SDLGLVisibleSurface::takeScreenShot(const std::string& filename)
 {
 	unsigned width  = getWidth();
 	unsigned height = getHeight();
-	byte** row_pointers = static_cast<byte**>(
-		malloc(height * sizeof(byte*)));
-	byte* buffer = static_cast<byte*>(
-		malloc(width * height * 3 * sizeof(byte)));
+	std::vector<byte*> row_pointers(height);
+	std::vector<byte> buffer(width * height * 3);
 	for (unsigned i = 0; i < height; ++i) {
 		row_pointers[height - 1 - i] = &buffer[width * 3 * i];
 	}
-	glReadPixels(0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, buffer);
-	try {
-		ScreenShotSaver::save(width, height, row_pointers, filename);
-	} catch(...) {
-		// TODO: Use "finally" instead?
-		free(row_pointers);
-		free(buffer);
-		throw;
-	}
-	free(row_pointers);
-	free(buffer);
+	glReadPixels(0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, &buffer[0]);
+	ScreenShotSaver::save(width, height, &row_pointers[0], filename);
 }
 
 std::auto_ptr<Layer> SDLGLVisibleSurface::createSnowLayer()
