@@ -1,4 +1,4 @@
-// $Id: SDLRasterizer.cc,v 1.51 2006/05/28 21:21:45 m9710797 Exp $
+// $Id: SDLRasterizer.cc 5984 2007-01-08 22:03:21Z m9710797 $
 
 #include "SDLRasterizer.hh"
 #include "VDP.hh"
@@ -50,89 +50,18 @@ inline int SDLRasterizer<Pixel>::translateX(int absoluteX, bool narrow)
 }
 
 template <class Pixel>
-inline void SDLRasterizer<Pixel>::renderBitmapLine(
-	byte mode, int vramLine)
+inline void SDLRasterizer<Pixel>::renderBitmapLine(Pixel* buf, unsigned vramLine)
 {
-	if (lineValidInMode[vramLine] != mode) {
+	if (vdp.getDisplayMode().isPlanar()) {
+		const byte* vramPtr0;
+		const byte* vramPtr1;
+		vram.bitmapCacheWindow.getReadAreaPlanar(
+			vramLine * 256, 256, vramPtr0, vramPtr1);
+		bitmapConverter->convertLinePlanar(buf, vramPtr0, vramPtr1);
+	} else {
 		const byte* vramPtr =
-			vram.bitmapCacheWindow.readArea(vramLine << 7);
-		bitmapConverter->convertLine(
-			bitmapDisplayCache->getLinePtr(vramLine, (Pixel*)0),
-			vramPtr);
-		lineValidInMode[vramLine] = mode;
-	}
-}
-
-template <class Pixel>
-inline void SDLRasterizer<Pixel>::renderBitmapLines(
-	byte line, int count)
-{
-	byte mode = vdp.getDisplayMode().getByte();
-	// Which bits in the name mask determine the page?
-	int pageMask = 0x200 | vdp.getEvenOddMask();
-	int nameMask = vram.nameTable.getMask() >> 7;
-
-	while (count--) {
-		// TODO: Optimise addr and line; too many conversions right now.
-		int vramLine = nameMask & (pageMask | line);
-		renderBitmapLine(mode, vramLine);
-		if (vdp.isMultiPageScrolling()) {
-			vramLine &= ~0x100;
-			renderBitmapLine(mode, vramLine);
-		}
-		line++; // is a byte, so wraps at 256
-	}
-}
-
-template <class Pixel>
-inline void SDLRasterizer<Pixel>::renderPlanarBitmapLine(
-	byte mode, int vramLine)
-{
-	if ( lineValidInMode[vramLine] != mode
-	|| lineValidInMode[vramLine | 512] != mode ) {
-		int addr0 = vramLine << 7;
-		int addr1 = addr0 | 0x10000;
-		const byte* vramPtr0 =
-			vram.bitmapCacheWindow.readArea(addr0);
-		const byte* vramPtr1 =
-			vram.bitmapCacheWindow.readArea(addr1);
-		bitmapConverter->convertLinePlanar(
-			bitmapDisplayCache->getLinePtr(vramLine, (Pixel*)0),
-			vramPtr0, vramPtr1
-			);
-		lineValidInMode[vramLine] =
-			lineValidInMode[vramLine | 512] = mode;
-	}
-}
-
-template <class Pixel>
-inline void SDLRasterizer<Pixel>::renderPlanarBitmapLines(
-	byte line, int count)
-{
-	byte mode = vdp.getDisplayMode().getByte();
-	// Which bits in the name mask determine the page?
-	int pageMask = vdp.getEvenOddMask();
-	while (count--) {
-		// TODO: Optimise addr and line; too many conversions right now.
-		int vramLine = (vram.nameTable.getMask() >> 7) & (pageMask | line);
-		renderPlanarBitmapLine(mode, vramLine);
-		if (vdp.isMultiPageScrolling()) {
-			vramLine &= ~0x100;
-			renderPlanarBitmapLine(mode, vramLine);
-		}
-		line++; // is a byte, so wraps at 256
-	}
-}
-
-template <class Pixel>
-inline void SDLRasterizer<Pixel>::renderCharacterLines(
-	byte line, int count)
-{
-	while (count--) {
-		// Render this line.
-		characterConverter->convertLine(
-			charDisplayCache->getLinePtr(line, (Pixel*)0), line);
-		line++; // is a byte, so wraps at 256
+			vram.bitmapCacheWindow.getReadArea(vramLine * 128, 128);
+		bitmapConverter->convertLine(buf, vramPtr);
 	}
 }
 
@@ -152,14 +81,6 @@ SDLRasterizer<Pixel>::SDLRasterizer(
 {
 	workFrame = new RawFrame(screen.getFormat(), 640, 240);
 
-	// Create display caches.
-	charDisplayCache = new RawFrame(
-		screen.getFormat(), 512, vdp.isMSX1VDP() ? 192 : 256
-		);
-	bitmapDisplayCache = vdp.isMSX1VDP()
-		? NULL
-		: new RawFrame(screen.getFormat(), 512, 256 * 4);
-
 	// Init the palette.
 	precalcPalette();
 
@@ -185,8 +106,6 @@ SDLRasterizer<Pixel>::~SDLRasterizer()
 	renderSettings.getBrightness() .detach(*this);
 	renderSettings.getContrast()   .detach(*this);
 
-	delete bitmapDisplayCache;
-	delete charDisplayCache;
 	delete workFrame;
 }
 
@@ -203,9 +122,6 @@ void SDLRasterizer<Pixel>::reset()
 	setDisplayMode(vdp.getDisplayMode());
 	spriteConverter->setTransparency(vdp.getTransparency());
 
-	// Invalidate bitmap cache.
-	memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
-
 	resetPalette();
 }
 
@@ -221,12 +137,13 @@ void SDLRasterizer<Pixel>::resetPalette()
 }
 
 template <class Pixel>
-void SDLRasterizer<Pixel>::frameStart()
+void SDLRasterizer<Pixel>::frameStart(const EmuTime& time)
 {
 	workFrame = postProcessor->rotateFrames(workFrame,
 	    vdp.isInterlaced()
 	    ? (vdp.getEvenOdd() ? FrameSource::FIELD_ODD : FrameSource::FIELD_EVEN)
-	    : FrameSource::FIELD_NONINTERLACED);
+	    : FrameSource::FIELD_NONINTERLACED,
+	    time);
 
 	// Calculate line to render at top of screen.
 	// Make sure the display area is centered.
@@ -258,22 +175,13 @@ void SDLRasterizer<Pixel>::setDisplayMode(DisplayMode mode)
 }
 
 template <class Pixel>
-void SDLRasterizer<Pixel>::setPalette(
-	int index, int grb)
+void SDLRasterizer<Pixel>::setPalette(int index, int grb)
 {
 	// Update SDL colours in palette.
 	Pixel newColor = V9938_COLOURS[(grb >> 4) & 7][grb >> 8][grb & 7];
-	if ((palFg[index     ] != newColor) ||
-	    (palFg[index + 16] != newColor) ||
-	    (palBg[index     ] != newColor)) {
-		palFg[index     ] = newColor;
-		palFg[index + 16] = newColor;
-		palBg[index     ] = newColor;
-		// Any line containing pixels of this colour must be repainted.
-		// We don't know which lines contain which colours,
-		// so we have to repaint them all.
-		memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
-	}
+	palFg[index     ] = newColor;
+	palFg[index + 16] = newColor;
+	palBg[index     ] = newColor;
 
 	precalcColourIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
 	                    vdp.getBackgroundColour());
@@ -362,32 +270,17 @@ void SDLRasterizer<Pixel>::precalcColourIndex0(
 
 	int tpIndex = transparency ? bgcolorIndex : 0;
 	if (mode.getBase() != DisplayMode::GRAPHIC5) {
-		if (palFg[0] != palBg[tpIndex]) {
-			palFg[0] = palBg[tpIndex];
-
-			// Any line containing pixels of colour 0 must be repainted.
-			// We don't know which lines contain such pixels,
-			// so we have to repaint them all.
-			memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
-		}
+		palFg[0] = palBg[tpIndex];
 	} else {
-		if ((palFg[ 0] != palBg[tpIndex >> 2]) ||
-		    (palFg[16] != palBg[tpIndex &  3])) {
-			palFg[ 0] = palBg[tpIndex >> 2];
-			palFg[16] = palBg[tpIndex &  3];
-
-			// Any line containing pixels of colour 0 must be repainted.
-			// We don't know which lines contain such pixels,
-			// so we have to repaint them all.
-			memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
-		}
+		palFg[ 0] = palBg[tpIndex >> 2];
+		palFg[16] = palBg[tpIndex &  3];
 	}
 }
 
 template <class Pixel>
-void SDLRasterizer<Pixel>::updateVRAMCache(int address)
+void SDLRasterizer<Pixel>::updateVRAMCache(int /*address*/)
 {
-	lineValidInMode[address >> 7] = 0xFF;
+	// TODO can we get rid of this method? GLRasterizer still needs it
 }
 
 template <class Pixel>
@@ -422,10 +315,10 @@ void SDLRasterizer<Pixel>::drawBorder(
 		unsigned x = translateX(fromX, (lineWidth == 512));
 		unsigned num = translateX(limitX, (lineWidth == 512)) - x;
 		unsigned width = (lineWidth == 512) ? 640 : 320;
+		MemoryOps::MemSet2<Pixel, MemoryOps::NO_STREAMING> memset;
 		for (int y = startY; y < endY; ++y) {
-			MemoryOps::memset_2<Pixel, MemoryOps::NO_STREAMING>(
-				workFrame->getLinePtr(y, (Pixel*)0) + x, num,
-				                      border0, border1);
+			memset(workFrame->getLinePtr(y, (Pixel*)0) + x,
+			       num, border0, border1);
 			workFrame->setLineWidth(y, width);
 		}
 	}
@@ -435,8 +328,8 @@ template <class Pixel>
 void SDLRasterizer<Pixel>::drawDisplay(
 	int /*fromX*/, int fromY,
 	int displayX, int displayY,
-	int displayWidth, int displayHeight
-) {
+	int displayWidth, int displayHeight)
+{
 	DisplayMode mode = vdp.getDisplayMode();
 	unsigned lineWidth = mode.getLineWidth();
 	if (lineWidth == 256) {
@@ -448,8 +341,7 @@ void SDLRasterizer<Pixel>::drawDisplay(
 	// Clip to screen area.
 	int screenLimitY = std::min(
 		fromY + displayHeight - lineRenderTop,
-		240
-		);
+		240);
 	int screenY = fromY - lineRenderTop;
 	if (screenY < 0) {
 		displayY -= screenY;
@@ -490,13 +382,6 @@ void SDLRasterizer<Pixel>::drawDisplay(
 	}
 
 	if (mode.isBitmapMode()) {
-		// Bring bitmap cache up to date.
-		if (mode.isPlanar()) {
-			renderPlanarBitmapLines(displayY, displayHeight);
-		} else {
-			renderBitmapLines(displayY, displayHeight);
-		}
-
 		// Which bits in the name mask determine the page?
 		int pageMaskOdd = (mode.isPlanar() ? 0x000 : 0x200) |
 		                  vdp.getEvenOddMask();
@@ -504,18 +389,17 @@ void SDLRasterizer<Pixel>::drawDisplay(
 		                 ? (pageMaskOdd & ~0x100)
 		                 : pageMaskOdd;
 
-		// Copy from cache to screen.
 		for (int y = screenY; y < screenLimitY; y++) {
 			const int vramLine[2] = {
 				(vram.nameTable.getMask() >> 7) & (pageMaskEven | displayY),
 				(vram.nameTable.getMask() >> 7) & (pageMaskOdd  | displayY)
-				};
+			};
 
 			int firstPageWidth = pageBorder - displayX;
 			if (firstPageWidth > 0) {
-				const Pixel* src = bitmapDisplayCache->getLinePtr(
-					vramLine[scrollPage1], (Pixel*)0) +
-						displayX + hScroll;
+				Pixel buf[512];
+				renderBitmapLine(buf, vramLine[scrollPage1]);
+				const Pixel* src = buf + displayX + hScroll;
 				Pixel* dst = workFrame->getLinePtr(y, (Pixel*)0)
 				           + leftBackground + displayX;
 				MemoryOps::stream_memcpy(dst, src, firstPageWidth);
@@ -526,8 +410,9 @@ void SDLRasterizer<Pixel>::drawDisplay(
 				unsigned x = displayX < pageBorder
 					? 0 : displayX + hScroll - lineWidth;
 				unsigned num = displayWidth - firstPageWidth;
-				const Pixel* src = bitmapDisplayCache->getLinePtr(
-					vramLine[scrollPage2], (Pixel*)0) + x;
+				Pixel buf[512];
+				renderBitmapLine(buf, vramLine[scrollPage2]);
+				const Pixel* src = buf + x;
 				Pixel* dst = workFrame->getLinePtr(y, (Pixel*)0)
 				           + leftBackground + displayX
 					   + firstPageWidth;
@@ -538,14 +423,13 @@ void SDLRasterizer<Pixel>::drawDisplay(
 			displayY = (displayY + 1) & 255;
 		}
 	} else {
-		renderCharacterLines(displayY, displayHeight);
-
-		// TODO: Implement horizontal scroll high.
+		// horizontal scroll (high) is implemented in CharacterConverter
 		for (int y = screenY; y < screenLimitY; y++) {
 			assert(!vdp.isMSX1VDP() || displayY < 192);
 
-			const Pixel* src = charDisplayCache->getLinePtr(
-				displayY, (Pixel*)0) + displayX;
+			Pixel buf[512];
+			characterConverter->convertLine(buf, displayY);
+			const Pixel* src = buf + displayX;
 			Pixel* dst = workFrame->getLinePtr(y, (Pixel*)0)
 			           + leftBackground + displayX;
 			MemoryOps::stream_memcpy(dst, src, displayWidth);
@@ -560,14 +444,13 @@ template <class Pixel>
 void SDLRasterizer<Pixel>::drawSprites(
 	int /*fromX*/, int fromY,
 	int displayX, int displayY,
-	int displayWidth, int displayHeight
-) {
+	int displayWidth, int displayHeight)
+{
 	// Clip to screen area.
 	// TODO: Code duplicated from drawDisplay.
 	int screenLimitY = std::min(
 		fromY + displayHeight - lineRenderTop,
-		240
-		);
+		240);
 	int screenY = fromY - lineRenderTop;
 	if (screenY < 0) {
 		displayY -= screenY;
@@ -585,8 +468,7 @@ void SDLRasterizer<Pixel>::drawSprites(
 	int limitY = fromY + displayHeight;
 	int screenX = translateX(
 		vdp.getLeftSprites(),
-		vdp.getDisplayMode().getLineWidth() == 512
-		);
+		vdp.getDisplayMode().getLineWidth() == 512);
 	for (int y = fromY; y < limitY; y++, screenY++) {
 		Pixel* pixelPtr = workFrame->getLinePtr(screenY, (Pixel*)0) + screenX;
 		if (spriteMode == 1) {
@@ -597,6 +479,12 @@ void SDLRasterizer<Pixel>::drawSprites(
 	}
 }
 
+template <class Pixel>
+bool SDLRasterizer<Pixel>::isRecording() const
+{
+	return postProcessor->isRecording();
+}
+
 template <class Pixel>
 void SDLRasterizer<Pixel>::update(const Setting& setting)
 {
@@ -606,9 +494,6 @@ void SDLRasterizer<Pixel>::update(const Setting& setting)
 	    (&setting == &renderSettings.getColorMatrix())) {
 		precalcPalette();
 		resetPalette();
-
-		// Invalidate bitmap cache (still needed for non-palette modes)
-		memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
 	}
 }
 
