@@ -1,4 +1,4 @@
-// $Id: SaI3xScaler.cc,v 1.15 2006/05/14 05:49:53 mthuurne Exp $
+// $Id: SaI3xScaler.cc 5907 2006-11-29 17:23:52Z m9710797 $
 
 // 2xSaI is Copyright (c) 1999-2001 by Derek Liauw Kie Fa.
 //   http://elektron.its.tudelft.nl/~dalikifa/
@@ -30,18 +30,16 @@ void SaI3xScaler<Pixel>::scaleBlank1to3(
 	unsigned stopDstY = (dstEndY == dst.getHeight())
 	                  ? dstEndY : dstEndY - 3;
 	unsigned srcY = srcStartY, dstY = dstStartY;
+	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
 		Pixel* dummy = 0;
 		Pixel color = src.getLinePtr(srcY, dummy)[0];
 		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine0, dst.getWidth(), color);
+		memset(dstLine0, dst.getWidth(), color);
 		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine1, dst.getWidth(), color);
+		memset(dstLine1, dst.getWidth(), color);
 		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine2, dst.getWidth(), color);
+		memset(dstLine2, dst.getWidth(), color);
 	}
 	if (dstY != dst.getHeight()) {
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
@@ -90,7 +88,7 @@ unsigned bilinear<unsigned>(unsigned a, unsigned b, unsigned x)
 		(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB;
 	const unsigned result1 =
 		(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB;
-	return ((result0 & (0xFF00FF << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
+	return ((result0 & (0xFF00FFu << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
 }
 
 template <typename Pixel>
@@ -139,7 +137,7 @@ unsigned bilinear4<unsigned>(
 	const unsigned result1 =
 		(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB +
 		(c & 0x0000FF00) * areaC + (d & 0x0000FF00) * areaD;
-	return ((result0 & (0xFF00FF << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
+	return ((result0 & (0xFF00FFu << 8)) | (result1 & (0x00FF00 << 8))) >> 8;
 }
 
 template <typename Pixel>
@@ -180,8 +178,8 @@ inline Pixel Blender<Pixel>::blend(unsigned a, unsigned b)
 			(a & 0x00FF00FF) * areaA + (b & 0x00FF00FF) * areaB;
 		const unsigned result1 =
 			(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB;
-		return ((result0 & (0xFF00FF << bits)) |
-		        (result1 & (0x00FF00 << bits))) >> bits;
+		return ((result0 & (0xFF00FFu << bits)) |
+		        (result1 & (0x00FF00u << bits))) >> bits;
 	}
 }
 
@@ -213,8 +211,8 @@ inline Pixel Blender<Pixel>::blend(
 		const unsigned result1 =
 			(a & 0x0000FF00) * areaA + (b & 0x0000FF00) * areaB +
 			(c & 0x0000FF00) * areaC + (d & 0x0000FF00) * areaD;
-		return ((result0 & (0xFF00FF << bits)) |
-		        (result1 & (0x00FF00 << bits))) >> bits;
+		return ((result0 & (0xFF00FFu << bits)) |
+		        (result1 & (0x00FF00u << bits))) >> bits;
 	}
 }
 
@@ -232,8 +230,8 @@ public:
 	inline static void blendBackslash(
 		Pixel* &dp,
 		unsigned sa, unsigned sb, unsigned sc, unsigned sd,
-		unsigned se, unsigned sg, unsigned sj, unsigned sl
-	) {
+		unsigned se, unsigned sg, unsigned sj, unsigned sl)
+	{
 		// Fractional parts of the fixed point X coordinates.
 		const unsigned x1 = ((NX - i) << 16) / NX;
 		const unsigned y1 = y;
@@ -260,8 +258,8 @@ public:
 	inline static void blendSlash(
 		Pixel* &dp,
 		unsigned sa, unsigned sb, unsigned sc, unsigned sd,
-		unsigned sf, unsigned sh, unsigned si, unsigned sk
-	) {
+		unsigned sf, unsigned sh, unsigned si, unsigned sk)
+	{
 		// Fractional parts of the fixed point X coordinates.
 		const unsigned x1 = ((NX - i) << 16) / NX;
 		const unsigned x2 = 0x10000 - x1;
@@ -288,8 +286,8 @@ public:
 
 	template <unsigned NX, unsigned y, typename Pixel>
 	inline static void blend4(
-		Pixel* &dp, unsigned sa, unsigned sb, unsigned sc, unsigned sd
-	) {
+		Pixel* &dp, unsigned sa, unsigned sb, unsigned sc, unsigned sd)
+	{
 		const unsigned x = ((NX - i) << 16) / NX;
 		*dp++ = Blender<Pixel>::template blend<x, y>(sa, sb, sc, sd);
 		PixelStripRepeater<i - 1>::template blend4<NX, y, Pixel>(dp, sa, sb, sc, sd);
