@@ -1,4 +1,4 @@
-// $Id: Simple3xScaler.cc,v 1.16 2006/03/04 15:39:19 m9710797 Exp $
+// $Id: Simple3xScaler.cc 6097 2007-02-19 11:44:49Z m9710797 $
 
 #include "Simple3xScaler.hh"
 #include "LineScalers.hh"
@@ -16,7 +16,7 @@ template <class Pixel> class Blur_1on3
 public:
 	Blur_1on3(const PixelOperations<Pixel>& pixelOps,
 	          const RenderSettings& settings);
-	void operator()(const Pixel* in, Pixel* out, unsigned dstWidth);
+	void operator()(const Pixel* in, Pixel* out, unsigned long dstWidth);
 private:
 	Multiply32<Pixel> mult0;
 	Multiply32<Pixel> mult1;
@@ -57,7 +57,7 @@ void Simple3xScaler<Pixel>::doScale1(FrameSource& src,
 	copy(prevDstLine0, dstLine1, dst.getWidth());
 
 	for (/* */; (y + 4) < dstEndY; y += 3, srcStartY += 1) {
-		const Pixel* srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
+		srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
 		Pixel* dstLine0 = dst.getLinePtr(y + 3, dummy);
 		scale(srcLine, dstLine0, dst.getWidth());
 
@@ -227,19 +227,17 @@ void Simple3xScaler<Pixel>::scaleBlank1to3(
 	unsigned stopDstY = (dstEndY == dst.getHeight())
 	                  ? dstEndY : dstEndY - 3;
 	unsigned srcY = srcStartY, dstY = dstStartY;
+	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
 		Pixel* dummy = 0;
 		Pixel color0 = src.getLinePtr(srcY, dummy)[0];
 		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine0, dst.getWidth(), color0);
+		memset(dstLine0, dst.getWidth(), color0);
 		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine1, dst.getWidth(), color0);
+		memset(dstLine1, dst.getWidth(), color0);
 		Pixel color1 = scanline.darken(color0, scanlineFactor);
 		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine2, dst.getWidth(), color1);
+		memset(dstLine2, dst.getWidth(), color1);
 	}
 	if (dstY != dst.getHeight()) {
 		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
@@ -256,6 +254,7 @@ void Simple3xScaler<Pixel>::scaleBlank2to3(
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
 	int scanlineFactor = settings.getScanlineFactor();
+	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; srcY += 2, dstY += 3) {
 		Pixel* dummy = 0;
@@ -263,14 +262,11 @@ void Simple3xScaler<Pixel>::scaleBlank2to3(
 		Pixel color1 = src.getLinePtr(srcY + 1, dummy)[0];
 		Pixel color01 = scanline.darken(color0, color1, scanlineFactor);
 		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine0, dst.getWidth(), color0);
+		memset(dstLine0, dst.getWidth(), color0);
 		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine1, dst.getWidth(), color01);
+		memset(dstLine1, dst.getWidth(), color01);
 		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
-		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
-			dstLine2, dst.getWidth(), color1);
+		memset(dstLine2, dst.getWidth(), color1);
 	}
 }
 
@@ -289,7 +285,7 @@ Blur_1on3<Pixel>::Blur_1on3(const PixelOperations<Pixel>& pixelOps,
 }
 
 template <class Pixel>
-void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth)
+void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dstWidth)
 {
 	/* The following code is equivalent to this loop. It is 2x unrolled
 	 * and common subexpressions have been eliminated. The last iteration
@@ -311,8 +307,7 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth
 	 *      curr = next;
 	 *  }
 	 */
-	
-	unsigned alpha = settings.getBlurFactor() / 3;
+	unsigned long alpha = settings.getBlurFactor() / 3;
 	#ifdef ASM_X86
 	const HostCPU& cpu = HostCPU::getInstance();
 	if ((sizeof(Pixel) == 4) && cpu.hasMMXEXT()) {
@@ -337,23 +332,23 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth
 
 		asm volatile (
 			"pxor      %%mm0, %%mm0;"
-			"pshufw    $0,40(%[CONST]),%%mm1;"
-			"pshufw    $0,44(%[CONST]),%%mm2;"
-			"pshufw    $0,48(%[CONST]),%%mm3;"
-			"pshufw    $0,52(%[CONST]),%%mm4;"
-			"movq      %%mm0,   (%[CONST]);"   // zero    store constants
-			"movq      %%mm1,  8(%[CONST]);"   // c0
-			"movq      %%mm2, 16(%[CONST]);"   // c1
-			"movq      %%mm3, 24(%[CONST]);"   // c2
-			"movq      %%mm4, 32(%[CONST]);"   // c3
+			"pshufw    $0,40(%[CNST]),%%mm1;"
+			"pshufw    $0,44(%[CNST]),%%mm2;"
+			"pshufw    $0,48(%[CNST]),%%mm3;"
+			"pshufw    $0,52(%[CNST]),%%mm4;"
+			"movq      %%mm0,   (%[CNST]);"    // zero    store constants
+			"movq      %%mm1,  8(%[CNST]);"    // c0
+			"movq      %%mm2, 16(%[CNST]);"    // c1
+			"movq      %%mm3, 24(%[CNST]);"    // c2
+			"movq      %%mm4, 32(%[CNST]);"    // c3
 
 			"movq      (%[IN]), %%mm0;"        // in[0] | in[1]
 			"movq      %%mm0, %%mm5;"          // in[0] | in[1]
-			"punpcklbw (%[CONST]), %%mm0;"     // p0 = unpack(in[0])
+			"punpcklbw (%[CNST]), %%mm0;"      // p0 = unpack(in[0])
 			"movq      %%mm0, %%mm2;"          // p0
-			"pmulhuw    8(%[CONST]), %%mm2;"   // f0 = c0 * p0
+			"pmulhuw    8(%[CNST]), %%mm2;"    // f0 = c0 * p0
 			"movq      %%mm0, %%mm3;"          // p0
-			"pmulhuw   16(%[CONST]), %%mm3;"   // f1 = c1 * p0
+			"pmulhuw   16(%[CNST]), %%mm3;"    // f1 = c1 * p0
 			"movq      %%mm2, %%mm4;"          // g0 = f0;
 			"movq      %%mm3, %%mm6;"          // g1 = f1;
 
@@ -363,12 +358,12 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth
 			"prefetcht0  320(%[OUT],%[Y]);"    //
 			"movq      %%mm5, %%mm1;"          // in[x + 1]
 			"movq      %%mm0, %%mm7;"          // p0
-			"punpckhbw   (%[CONST]), %%mm1;"   // p1 = unpack(in[x + 1])
-			"pmulhuw   32(%[CONST]), %%mm7;"   // s0 = c3 * p0
+			"punpckhbw   (%[CNST]), %%mm1;"    // p1 = unpack(in[x + 1])
+			"pmulhuw   32(%[CNST]), %%mm7;"    // s0 = c3 * p0
 			"paddw     %%mm2, %%mm7;"          // s0 + f0
 			"movq      %%mm1, %%mm2;"          // p1
-			"pmulhuw   24(%[CONST]), %%mm0;"   // g2 = c2 * p0
-			"pmulhuw    8(%[CONST]), %%mm2;"   // t0 = c0 * p1 (= f0)
+			"pmulhuw   24(%[CNST]), %%mm0;"    // g2 = c2 * p0
+			"pmulhuw    8(%[CNST]), %%mm2;"    // t0 = c0 * p1 (= f0)
 			"movq      8(%[IN]), %%mm5;"       // in[x + 2] | in[x + 3]
 			"paddw     %%mm0, %%mm3;"          // a0 = g2 + f1
 			"paddw     %%mm2, %%mm7;"          // a1 = t0 + s0 + f0
@@ -376,50 +371,50 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth
 			"movq      %%mm3, (%[OUT],%[Y]);"  // out[y + 0] = ...
 			"movq      %%mm1, %%mm3;"          // p1
 			"movq      %%mm1, %%mm7;"          // p1
-			"pmulhuw   16(%[CONST]), %%mm3;"   // f1 = c1 * p1
-			"pmulhuw   24(%[CONST]), %%mm7;"   // f2 = c2 * p1
+			"pmulhuw   16(%[CNST]), %%mm3;"    // f1 = c1 * p1
+			"pmulhuw   24(%[CNST]), %%mm7;"    // f2 = c2 * p1
 			"paddw     %%mm3, %%mm0;"          // a2 = g2 + f1
 			"paddw     %%mm7, %%mm6;"          // b0 = f2 + g1
 			"packuswb  %%mm6, %%mm0;"          // a2 | b0
 			"movq      %%mm0, 8(%[OUT],%[Y]);" // out[y + 2] = ...
 			"movq      %%mm5, %%mm0;"          // in[x + 2]
-			"pmulhuw   32(%[CONST]), %%mm1;"   // s1 = c3 * p1
-			"punpcklbw   (%[CONST]), %%mm0;"   // p0 = unpack(in[x + 2])
+			"pmulhuw   32(%[CNST]), %%mm1;"    // s1 = c3 * p1
+			"punpcklbw   (%[CNST]), %%mm0;"    // p0 = unpack(in[x + 2])
 			"paddw     %%mm4, %%mm1;"          // s1 + g0
 			"movq      %%mm0, %%mm6;"          // p0
 			"movq      %%mm0, %%mm4;"          // p0
-			"pmulhuw   16(%[CONST]), %%mm6;"   // g1 = c1 * p0
-			"pmulhuw    8(%[CONST]), %%mm4;"   // t1 = c0 * p0 (= g0)
+			"pmulhuw   16(%[CNST]), %%mm6;"    // g1 = c1 * p0
+			"pmulhuw    8(%[CNST]), %%mm4;"    // t1 = c0 * p0 (= g0)
 			"paddw     %%mm6, %%mm7;"          // b2 = g1 + f2
 			"paddw     %%mm4, %%mm1;"          // b1 = t1 + s1 + g0
 			"packuswb  %%mm7, %%mm1;"          // b1 | b2
-			"addl      $8, %[IN];"             // x += 2
+			"add       $8, %[IN];"             // x += 2
 			"movq      %%mm1, 16(%[OUT],%[Y]);"// out[y + 4] = 
-			"addl      $24, %[Y];"             // y += 6
+			"add       $24, %[Y];"             // y += 6
 			"jnz       0b;"                    //
 			
 			"movq      %%mm5, %%mm1;"          // in[x + 1]
 			"movq      %%mm0, %%mm7;"          // p0
-			"punpckhbw   (%[CONST]), %%mm1;"   // p1 = unpack(in[x + 1])
-			"pmulhuw   32(%[CONST]), %%mm7;"   // s0 = c3 * p0
+			"punpckhbw   (%[CNST]), %%mm1;"    // p1 = unpack(in[x + 1])
+			"pmulhuw   32(%[CNST]), %%mm7;"    // s0 = c3 * p0
 			"paddw     %%mm2, %%mm7;"          // s0 + f0
 			"movq      %%mm1, %%mm2;"          // p1
-			"pmulhuw   24(%[CONST]), %%mm0;"   // g2 = c2 * p0
-			"pmulhuw    8(%[CONST]), %%mm2;"   // t0 = c0 * p1 (= f0)
+			"pmulhuw   24(%[CNST]), %%mm0;"    // g2 = c2 * p0
+			"pmulhuw    8(%[CNST]), %%mm2;"    // t0 = c0 * p1 (= f0)
 			"paddw     %%mm0, %%mm3;"          // a0 = g2 + f1
 			"paddw     %%mm2, %%mm7;"          // a1 = t0 + s0 + f0
 			"packuswb  %%mm7, %%mm3;"          // a0 | a1
 			"movq      %%mm3, (%[OUT]);"       // out[y + 0] = ...
 			"movq      %%mm1, %%mm3;"          // p1
 			"movq      %%mm1, %%mm7;"          // p1
-			"pmulhuw   16(%[CONST]), %%mm3;"   // f1 = c1 * p1
-			"pmulhuw   24(%[CONST]), %%mm7;"   // f2 = c2 * p1
+			"pmulhuw   16(%[CNST]), %%mm3;"    // f1 = c1 * p1
+			"pmulhuw   24(%[CNST]), %%mm7;"    // f2 = c2 * p1
 			"paddw     %%mm3, %%mm0;"          // a2 = g2 + f1
 			"paddw     %%mm7, %%mm6;"          // b0 = f2 + g1
 			"packuswb  %%mm6, %%mm0;"          // a2 | b0
 			"movq      %%mm0, 8(%[OUT]);"      // out[y + 2] = ...
 			"movq      %%mm1, %%mm7;"          // p1
-			"pmulhuw   32(%[CONST]), %%mm1;"   // s1 = c3 * p1
+			"pmulhuw   32(%[CNST]), %%mm1;"    // s1 = c3 * p1
 			"paddw     %%mm4, %%mm1;"          // s1 + g0
 			"paddw     %%mm2, %%mm1;"          // b1 = t1 + s1 + g0
 			"packuswb  %%mm7, %%mm1;"          // b1 | b2
@@ -428,10 +423,10 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned dstWidth
 			"emms;"
 
 			: "=&r" (t0), "=&r" (t1), "=&r" (t2), "=&r" (t3)
-			: [IN]    "0" (in)
-			, [OUT]   "1" (out + (dstWidth - 6))
-			, [Y]     "2" (-4 * (dstWidth - 6))
-			, [CONST] "3" (&c)
+			: [IN]   "0" (in)
+			, [OUT]  "1" (out + (dstWidth - 6))
+			, [Y]    "2" (-4 * (dstWidth - 6))
+			, [CNST] "3" (&c)
 			: "memory"
 			#ifdef __MMX__
 			, "mm0", "mm1", "mm2", "mm3", "mm4", "mm5", "mm6", "mm7"
