@@ -1,4 +1,4 @@
-// $Id: VDP.cc,v 1.104 2006/05/20 10:11:57 m9710797 Exp $
+// $Id: VDP.cc 6289 2007-04-12 21:48:52Z manuelbi $
 
 /*
 TODO:
@@ -30,6 +30,7 @@ TODO:
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXException.hh"
+#include "MSXCliComm.hh"
 #include <sstream>
 #include <cassert>
 
@@ -38,7 +39,7 @@ namespace openmsx {
 class VDPRegDebug : public SimpleDebuggable
 {
 public:
-	VDPRegDebug(VDP& vdp);
+	explicit VDPRegDebug(VDP& vdp);
 	virtual byte read(unsigned address);
 	virtual void write(unsigned address, byte value, const EmuTime& time);
 private:
@@ -48,7 +49,7 @@ private:
 class VDPStatusRegDebug : public SimpleDebuggable
 {
 public:
-	VDPStatusRegDebug(VDP& vdp);
+	explicit VDPStatusRegDebug(VDP& vdp);
 	virtual byte read(unsigned address, const EmuTime& time);
 private:
 	VDP& vdp;
@@ -57,7 +58,7 @@ private:
 class VDPPaletteDebug : public SimpleDebuggable
 {
 public:
-	VDPPaletteDebug(VDP& vdp);
+	explicit VDPPaletteDebug(VDP& vdp);
 	virtual byte read(unsigned address);
 	virtual void write(unsigned address, byte value, const EmuTime& time);
 private:
@@ -75,6 +76,8 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config,
 	, displayStartSyncTime(time)
 	, vScanSyncTime(time)
 	, hScanSyncTime(time)
+	, cliComm(motherBoard.getMSXCliComm())
+	, warningPrinted(false)
 	, vdpRegDebug      (new VDPRegDebug      (*this))
 	, vdpStatusRegDebug(new VDPStatusRegDebug(*this))
 	, vdpPaletteDebug  (new VDPPaletteDebug  (*this))
@@ -128,7 +131,7 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config,
 
 	// Create command engine.
 	cmdEngine.reset(new VDPCmdEngine(*this, renderSettings,
-		getMotherBoard().getCommandController() ));
+		getMotherBoard().getCommandController()));
 	vram->setCmdEngine(cmdEngine.get());
 
 	resetInit(time); // must be done early to avoid UMRs
@@ -563,6 +566,14 @@ void VDP::writeIO(word port, byte value, const EmuTime& time)
 					// it's not a register write because
 					// that breaks "SNOW26" demo
 				}
+				if (isMSX1VDP()) { 
+					// For these VDP's the VRAM pointer is modified when
+					// writing to VDP registers. Without this some demos won't
+					// run as on real MSX1, e.g. Planet of the Epas, Utopia and
+					// Waves 1.2. Thanks to dvik for finding this out. 
+					// Set read/write address.
+					vramPointer = ((word)value << 8 | dataLatch) & 0x3FFF;
+				}
 			} else {
 				// Set read/write address.
 				vramPointer = ((word)value << 8 | dataLatch) & 0x3FFF;
@@ -639,7 +650,8 @@ byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
 {
 	switch (reg) {
 	case 0:
-		return statusReg0 | spriteChecker->readStatus(time);
+		spriteChecker->sync(time);
+		return statusReg0;
 	case 1:
 		if (controlRegs[0] & 0x10) { // line int enabled
 			return statusReg1 | irqHorizontal.getState();
@@ -649,6 +661,11 @@ byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
 			// TODO: Precalc matchLength?
 			int afterMatch =
 			       getTicksThisFrame(time) - horizontalScanOffset;
+			if (afterMatch < 0) {
+				afterMatch += getTicksPerFrame();
+				// afterMatch can still be negative at this
+				// point, see scheduleHScan()
+			}
 			int matchLength = (displayMode.isTextMode() ? 87 : 59)
 			                  + 27 + 100 + 102;
 			return statusReg1 |
@@ -693,7 +710,7 @@ byte VDP::readStatusReg(byte reg, const EmuTime& time)
 	switch (reg) {
 	case 0:
 		spriteChecker->resetStatus();
-		statusReg0 = 0;
+		statusReg0 &= ~0x80;
 		irqVertical.reset();
 		break;
 	case 1:
@@ -919,6 +936,14 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 		updateSpritePatternBase(time);
 		break;
 	case 9:
+		if ((val & 1) && ! warningPrinted) {
+			warningPrinted=true;
+			cliComm.printWarning 
+				("The running MSX software has set bit 0 of VDP register 9 "
+				 "(dot clock direction) to one. In an ordinary MSX, "
+				 "the screen would go black and the CPU would stop running.");
+			// TODO: Emulate such behaviour.
+		}
 		if (change & 0x80) {
 			/*
 			cerr << "changed to " << (val & 0x80 ? 212 : 192) << " lines"
@@ -1042,12 +1067,7 @@ void VDP::updateSpriteAttributeBase(const EmuTime& time)
 	if (mode == 1) {
 		vram->spriteAttribTable.setMask(base, -1 << 7, time);
 	} else { // mode == 2
-		if (displayMode.isPlanar()) {
-			vram->spriteAttribTable.setMask(
-				((base << 16) | (base >> 1)) & 0x1FFFF, 0x0FE00, time);
-		} else {
-			vram->spriteAttribTable.setMask(base, 0x1FC00, time);
-		}
+		vram->spriteAttribTable.setMask(base, -1 << 10, time);
 	}
 }
 
@@ -1058,7 +1078,6 @@ void VDP::updateSpritePatternBase(const EmuTime& time)
 		return;
 	}
 	int base = (controlRegs[6] << 11) | ~(-1 << 11);
-	if (displayMode.isPlanar()) base = ((base << 16) | (base >> 1)) & 0x1FFFF;
 	vram->spritePatternTable.setMask(base, -1 << 11, time);
 }
 
