@@ -1,23 +1,17 @@
-// $Id: VDPVRAM.cc,v 1.31 2006/05/20 10:11:57 m9710797 Exp $
+// $Id: VDPVRAM.cc 5968 2006-12-29 14:48:37Z m9710797 $
 
 #include "VDPVRAM.hh"
 #include "SpriteChecker.hh"
 #include "Renderer.hh"
+#include "Math.hh"
 
 namespace openmsx {
 
 // class VRAMWindow:
 
-static unsigned roundUpPow2(unsigned a)
-{
-	unsigned result = 1;
-	while (result < a) result <<= 1;
-	return result;
-}
-
 VRAMWindow::VRAMWindow(Ram& vram)
 	: data(&vram[0])
-	, sizeMask(roundUpPow2(vram.getSize()) - 1)
+	, sizeMask(Math::powerOfTwo(vram.getSize()) - 1)
 {
 	observer = NULL;
 	baseAddr  = -1; // disable window
@@ -39,8 +33,10 @@ static unsigned bufferSize(unsigned size)
 VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, const EmuTime& time)
 	: vdp(vdp_)
 	, data(vdp.getMotherBoard(), "VRAM", "Video RAM.", bufferSize(size))
-	, clock(time)
-	, sizeMask(roundUpPow2(data.getSize()) - 1)
+	#ifdef DEBUG
+	, vramTime(time)
+	#endif
+	, sizeMask(Math::powerOfTwo(data.getSize()) - 1)
 	, actualSize(size)
 	, cmdReadWindow(data)
 	, cmdWriteWindow(data)
@@ -52,6 +48,8 @@ VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, const EmuTime& time)
 	, spriteAttribTable(data)
 	, spritePatternTable(data)
 {
+	(void)time;
+
 	// Initialise VRAM data array.
 	// TODO: Fill with checkerboard pattern NMS8250 has.
 	memset(&data[0], 0, data.getSize());
