@@ -1,4 +1,4 @@
-// $Id: V9990.cc,v 1.71 2006/06/05 09:44:00 m9710797 Exp $
+// $Id: V9990.cc 5981 2007-01-07 18:49:26Z manuelbi $
 
 #include "V9990.hh"
 #include "Scheduler.hh"
@@ -16,7 +16,7 @@ namespace openmsx {
 class V9990RegDebug : public SimpleDebuggable
 {
 public:
-	V9990RegDebug(V9990& v9990);
+	explicit V9990RegDebug(V9990& v9990);
 	virtual byte read(unsigned address);
 	virtual void write(unsigned address, byte value, const EmuTime& time);
 private:
@@ -26,7 +26,7 @@ private:
 class V9990PalDebug : public SimpleDebuggable
 {
 public:
-	V9990PalDebug(V9990& v9990);
+	explicit V9990PalDebug(V9990& v9990);
 	virtual byte read(unsigned address);
 	virtual void write(unsigned address, byte value, const EmuTime& time);
 private:
@@ -96,9 +96,11 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config,
 	// create Command Engine
 	cmdEngine.reset(new V9990CmdEngine(*this, time,
 		getMotherBoard().getDisplay().getRenderSettings()));
+	vram->setCmdEngine(*cmdEngine);
 
 	// Start with NTSC timing
 	palTiming = false;
+	interlaced = false;
 	setVerticalTiming();
 
 	// Initialise rendering system
@@ -140,6 +142,7 @@ void V9990::reset(const EmuTime& time)
 
 	palTiming = false;
 	// Reset sub-systems
+	cmdEngine->sync(time);
 	renderer->reset(time);
 	cmdEngine->reset(time);
 
@@ -156,7 +159,7 @@ byte V9990::readIO(word port, const EmuTime& time)
 		case VRAM_DATA: {
 			// read from VRAM
 			unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
-			result = vram->readVRAMSlow(addr);
+			result = vram->readVRAMCPU(addr, time);
 			if (!(regs[VRAM_READ_ADDRESS_2] & 0x80)) {
 				setVRAMAddr(VRAM_READ_ADDRESS_0, addr + 1);
 			}
@@ -185,7 +188,7 @@ byte V9990::readIO(word port, const EmuTime& time)
 			// read register
 			result = readRegister(regSelect & 0x3F, time);
 			if (!(regSelect & 0x40)) {
-				regSelect =   regSelect      & 0xC0 |
+				regSelect = ( regSelect      & 0xC0) |
 				            ((regSelect + 1) & 0x3F);
 			}
 			break;
@@ -195,11 +198,18 @@ byte V9990::readIO(word port, const EmuTime& time)
 			break;
 
 		case STATUS: {
-			int ticks = getUCTicksThisFrame(time);
-			int x = UCtoX(ticks, getDisplayMode());
-			int y = ticks / V9990DisplayTiming::UC_TICKS_PER_LINE;
-			bool hr = (x < 64) || (576 <= x); // TODO not correct
-			bool vr = (y < 14) || (226 <= y); // TODO not correct
+			const V9990DisplayPeriod& hor = getHorizontalTiming();
+			const V9990DisplayPeriod& ver = getVerticalTiming();
+			unsigned left   = hor.blank + hor.border1;
+			unsigned right  = left + hor.display;
+			unsigned top    = ver.blank + ver.border1;
+			unsigned bottom = top + ver.display;
+			unsigned ticks = getUCTicksThisFrame(time);
+			unsigned x = ticks % V9990DisplayTiming::UC_TICKS_PER_LINE;
+			unsigned y = ticks / V9990DisplayTiming::UC_TICKS_PER_LINE;
+			bool hr = (x < left) || (right  <= x);
+			bool vr = (y < top)  || (bottom <= y);
+
 			result = cmdEngine->getStatus(time) |
 				 (vr ? 0x40 : 0x00) |
 				 (hr ? 0x20 : 0x00) |
@@ -247,7 +257,7 @@ void V9990::writeIO(word port, byte val, const EmuTime& time)
 		case VRAM_DATA: {
 			// write VRAM
 			unsigned addr = getVRAMAddr(VRAM_WRITE_ADDRESS_0);
-			vram->writeVRAMSlow(addr, val);
+			vram->writeVRAMCPU(addr, val, time);
 			if (!(regs[VRAM_WRITE_ADDRESS_2] & 0x80)) {
 				setVRAMAddr(VRAM_WRITE_ADDRESS_0, addr + 1);
 			}
@@ -273,7 +283,7 @@ void V9990::writeIO(word port, byte val, const EmuTime& time)
 			// write register
 			writeRegister(regSelect & 0x3F, val, time);
 			if (!(regSelect & 0x80)) {
-				regSelect =   regSelect      & 0xC0 |
+				regSelect = ( regSelect      & 0xC0) |
 				            ((regSelect + 1) & 0x3F);
 			}
 			break;
@@ -295,6 +305,8 @@ void V9990::writeIO(word port, byte val, const EmuTime& time)
 			break;
 
 		case SYSTEM_CONTROL:
+			// TODO investigate: does switching overscan mode
+			//      happen at next line or next frame
 			status = (status & 0xFB) | ((val & 1) << 2);
 			syncAtNextLine(V9990_SET_MODE, time);
 		break;
@@ -449,7 +461,7 @@ inline void V9990::setVRAMAddr(RegisterId base, unsigned addr)
 {
 	regs[base + 0] =   addr &     0xFF;
 	regs[base + 1] =  (addr &   0xFF00) >> 8;
-	regs[base + 2] = ((addr & 0x070000) >> 16) | regs[base + 2] & 0x80;
+	regs[base + 2] = ((addr & 0x070000) >> 16) | (regs[base + 2] & 0x80);
 	// TODO check
 }
 
@@ -610,7 +622,8 @@ void V9990::createRenderer(const EmuTime& time)
 void V9990::frameStart(const EmuTime& time)
 {
 	// Update setings that are fixed at the start of a frame
-	palTiming = regs[SCREEN_MODE_1] & 0x08;
+	palTiming  = regs[SCREEN_MODE_1] & 0x08;
+	interlaced = regs[SCREEN_MODE_1] & 0x02;
 	setVerticalTiming();
 	status ^= 0x02; // flip EO bit
 
