@@ -1,4 +1,4 @@
-// $Id: V9990BitmapConverter.cc,v 1.25 2006/04/10 20:52:12 m9710797 Exp $
+// $Id: V9990BitmapConverter.cc 5906 2006-11-29 13:07:21Z mthuurne $
 
 #include "V9990BitmapConverter.hh"
 #include "V9990VRAM.hh"
@@ -12,30 +12,6 @@ using std::max;
 
 namespace openmsx {
 
-#ifdef COMPONENT_GL
-// On some systems, "GLuint" is not equivalent to "unsigned int",
-// so BitmapConverter must be instantiated separately for those systems.
-// But on systems where it is equivalent, it's an error to expand
-// the same template twice.
-// The following piece of template metaprogramming expands
-// V9990BitmapConverter<GLuint, Renderer::ZOOM_REAL> to an empty class if
-// "GLuint" is equivalent to "unsigned int"; otherwise it is expanded to
-// the actual V9990BitmapConverter implementation.
-//
-// See BitmapConverter.cc
-class NoExpansion {};
-// ExpandFilter::ExpandType = (Type == unsigned int ? NoExpansion : Type)
-template <class Type> class ExpandFilter {
-	typedef Type ExpandType;
-};
-template <> class ExpandFilter<unsigned int> {
-	typedef NoExpansion ExpandType;
-};
-template <> class V9990BitmapConverter<NoExpansion> {};
-template class V9990BitmapConverter<ExpandFilter<GLuint>::ExpandType>;
-#endif // COMPONENT_GL
-
-
 template <class Pixel>
 V9990BitmapConverter<Pixel>::V9990BitmapConverter(
 	V9990& vdp_, Pixel* palette64_, Pixel* palette256_, Pixel* palette32768_)
@@ -281,14 +257,17 @@ void V9990BitmapConverter<Pixel>::convertLine(
 	assert(nrPixels <= 1024);
 	(this->*rasterMethod)(linePtr, address, nrPixels);
 	if (vdp.spritesEnabled()) {
-		drawCursors(linePtr, displayY);
+		drawCursors(linePtr, displayY - vdp.getCursorYOffset());
 	}
 }
 
-
 // Force template instantiation
 template class V9990BitmapConverter<word>;
 template class V9990BitmapConverter<unsigned>;
+#ifdef COMPONENT_GL
+template <> class V9990BitmapConverter<GLUtil::NoExpansion> {};
+template class V9990BitmapConverter<GLUtil::ExpandGL>;
+#endif // COMPONENT_GL
 
 } // namespace openmsx
 
