@@ -1,4 +1,4 @@
-// $Id: V9990PixelRenderer.cc,v 1.34 2006/06/03 10:22:08 m9710797 Exp $
+// $Id: V9990PixelRenderer.cc 5984 2007-01-08 22:03:21Z m9710797 $
 
 #include "V9990PixelRenderer.hh"
 #include "V9990.hh"
@@ -15,6 +15,7 @@
 #include "MSXMotherBoard.hh"
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
+#include "BooleanSetting.hh"
 
 namespace openmsx {
 
@@ -29,6 +30,7 @@ V9990PixelRenderer::V9990PixelRenderer(V9990& vdp_)
 	frameSkipCounter = 999; // force drawing of frame;
 	finishFrameDuration = 0;
 	drawFrame = false; // don't draw before frameStart is called
+	prevDrawFrame = false;
 
 	reset(vdp.getMotherBoard().getScheduler().getCurrentTime());
 
@@ -54,19 +56,35 @@ void V9990PixelRenderer::reset(const EmuTime& time)
 void V9990PixelRenderer::frameStart(const EmuTime& time)
 {
 	if (!rasterizer->isActive()) {
-		frameSkipCounter = 0;
+		frameSkipCounter = 999;
 		drawFrame = false;
-	} else if (frameSkipCounter < renderSettings.getMinFrameSkip().getValue()) {
-		++frameSkipCounter;
-		drawFrame = false;
-	} else if (frameSkipCounter >= renderSettings.getMaxFrameSkip().getValue()) {
-		frameSkipCounter = 0;
-		drawFrame = true;
+		prevDrawFrame = false;
+		return;
+	}
+	prevDrawFrame = drawFrame;
+	if (vdp.isInterlaced() && renderSettings.getDeinterlace().getValue() &&
+	    vdp.getEvenOdd() && vdp.isEvenOddEnabled()) {
+		// deinterlaced odd frame, do same as even frame
 	} else {
-		++frameSkipCounter;
-		drawFrame = realTime.timeLeft((unsigned)finishFrameDuration, time);
-		if (drawFrame) {
+		if (frameSkipCounter <
+		              renderSettings.getMinFrameSkip().getValue()) {
+			++frameSkipCounter;
+			drawFrame = false;
+		} else if (frameSkipCounter >=
+		              renderSettings.getMaxFrameSkip().getValue()) {
 			frameSkipCounter = 0;
+			drawFrame = true;
+		} else {
+			++frameSkipCounter;
+			if (rasterizer->isRecording()) {
+				drawFrame = true;
+			} else {
+				drawFrame = realTime.timeLeft(
+					(unsigned)finishFrameDuration, time);
+			}
+			if (drawFrame) {
+				frameSkipCounter = 0;
+			}
 		}
 	}
 	if (!drawFrame) return;
@@ -84,21 +102,30 @@ void V9990PixelRenderer::frameStart(const EmuTime& time)
 
 void V9990PixelRenderer::frameEnd(const EmuTime& time)
 {
+	bool skipEvent = !drawFrame;
 	if (drawFrame) {
 		// Render last changes in this frame before starting a new frame
 		sync(time, true);
 
 		unsigned long long time1 = Timer::getTime();
-		rasterizer->frameEnd();
+		rasterizer->frameEnd(time);
 		unsigned long long time2 = Timer::getTime();
 		unsigned long long current = time2 - time1;
 		const double ALPHA = 0.2;
 		finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
 		                      current * ALPHA;
+
+		if (vdp.isInterlaced() && vdp.isEvenOddEnabled() &&
+		    renderSettings.getDeinterlace().getValue() &&
+		    !prevDrawFrame) {
+			// dont send event in deinterlace mode when
+			// previous frame was not rendered
+			skipEvent = true;
+		}
+
 	}
 	eventDistributor.distributeEvent(
-		new FinishFrameEvent(VIDEO_GFX9000, !drawFrame)
-		);
+		new FinishFrameEvent(VIDEO_GFX9000, skipEvent));
 }
 
 void V9990PixelRenderer::sync(const EmuTime& time, bool force)
@@ -259,9 +286,9 @@ void V9990PixelRenderer::updateScrollAYLow(const EmuTime& time)
 {
 	if (displayEnabled) {
 		sync(time);
-		if (vdp.getDisplayMode() != P1) {
-			verticalOffset = lastY;
-		}
+		// happens in all display modes (verified)
+		// TODO high byte still seems to be wrong .. need to investigate
+		verticalOffset = lastY;
 	}
 }
 void V9990PixelRenderer::updateScrollBX(const EmuTime& time)
@@ -272,6 +299,7 @@ void V9990PixelRenderer::updateScrollBX(const EmuTime& time)
 void V9990PixelRenderer::updateScrollBY(const EmuTime& time)
 {
 	// TODO only in P1 mode
+	// TODO should probably be the same as scrollAY
 	if (displayEnabled) sync(time);
 }
 
