@@ -1,4 +1,4 @@
-// $Id: V9990SDLRasterizer.cc,v 1.51 2006/06/12 17:20:47 m9710797 Exp $
+// $Id: V9990SDLRasterizer.cc 6131 2007-03-02 12:00:24Z m9710797 $
 
 #include "V9990SDLRasterizer.hh"
 #include "V9990.hh"
@@ -7,14 +7,12 @@
 #include "V9990BitmapConverter.hh"
 #include "V9990P1Converter.hh"
 #include "V9990P2Converter.hh"
-#include "BooleanSetting.hh"
 #include "FloatSetting.hh"
 #include "StringSetting.hh"
 #include "Display.hh"
 #include "VisibleSurface.hh"
 #include "RenderSettings.hh"
 #include "MemoryOps.hh"
-#include "MSXMotherBoard.hh"
 #include <algorithm>
 
 using std::min;
@@ -93,13 +91,14 @@ void V9990SDLRasterizer<Pixel>::frameStart()
 }
 
 template <class Pixel>
-void V9990SDLRasterizer<Pixel>::frameEnd()
+void V9990SDLRasterizer<Pixel>::frameEnd(const EmuTime& time)
 {
 	workFrame = postProcessor->rotateFrames(
 	    workFrame,
-	    vdp.isEvenOddEnabled() ? (vdp.getEvenOdd() ? RawFrame::FIELD_EVEN
-	                                               : RawFrame::FIELD_ODD)
-	                           : RawFrame::FIELD_NONINTERLACED);
+	    vdp.isInterlaced() ? (vdp.getEvenOdd() ? RawFrame::FIELD_EVEN
+	                                           : RawFrame::FIELD_ODD)
+	                       : RawFrame::FIELD_NONINTERLACED,
+	    time);
 }
 
 template <class Pixel>
@@ -143,10 +142,10 @@ void V9990SDLRasterizer<Pixel>::drawBorder(
 	if (startX >= endX) return;
 
 	unsigned lineWidth = vdp.getLineWidth();
+	MemoryOps::MemSet<Pixel, MemoryOps::NO_STREAMING> memset;
 	for (int y = startY; y < endY; ++y) {
-		MemoryOps::memset<Pixel, MemoryOps::NO_STREAMING>(
-			workFrame->getLinePtr(y, (Pixel*)0) + startX,
-			endX - startX, bgColor);
+		memset(workFrame->getLinePtr(y, (Pixel*)0) + startX,
+		       endX - startX, bgColor);
 		workFrame->setLineWidth(y, lineWidth);
 	}
 }
@@ -249,9 +248,9 @@ void V9990SDLRasterizer<Pixel>::drawBxMode(
 
 	unsigned scrollY = vdp.getScrollAY();
 	unsigned rollMask = vdp.getRollMask(0x1FFF);
-	unsigned scrollYBase = scrollY & ~rollMask;
+	unsigned scrollYBase = scrollY & ~rollMask & 0x1FFF;
 	while (displayHeight--) {
-		unsigned y = scrollYBase + (displayY + scrollY) & rollMask;
+		unsigned y = scrollYBase + ((displayY + scrollY) & rollMask);
 		unsigned address = vdp.XYtoVRAM(&x, y, colorMode);
 		Pixel* pixelPtr = workFrame->getLinePtr(fromY, (Pixel*)0) + fromX;
 		bitmapConverter->convertLine(pixelPtr, address, displayWidth,
@@ -317,6 +316,12 @@ void V9990SDLRasterizer<Pixel>::resetPalette()
 	}
 }
 
+template <class Pixel>
+bool V9990SDLRasterizer<Pixel>::isRecording() const
+{
+	return postProcessor->isRecording();
+}
+
 template <class Pixel>
 void V9990SDLRasterizer<Pixel>::update(const Setting& setting)
 {
